@@ -491,8 +491,9 @@ PRIVATE INLINE void
 free_dp_matrices(vrna_fold_compound_t *vc,
                  unsigned int         options)
 {
+  size_t        i;
   char          **ptype;
-  int           i, n, winSize;
+  int           n, winSize;
   FLT_OR_DBL    **pR, **q, **qb, **qm, **qm2, **QI5, **qmb, **q2l;
   vrna_mx_pf_t  *mx;
   vrna_hc_t     *hc;
@@ -568,8 +569,9 @@ rotate_dp_matrices(vrna_fold_compound_t *vc,
                    int                  j,
                    unsigned int         options)
 {
+  size_t        i;
   char          **ptype;
-  int           i, winSize, length;
+  int           winSize, length;
   FLT_OR_DBL    **pR, **q, **qb, **qm, **qm2, **QI5, **qmb, **q2l;
   vrna_mx_pf_t  *mx;
   vrna_hc_t     *hc;
@@ -587,7 +589,7 @@ rotate_dp_matrices(vrna_fold_compound_t *vc,
   sc      = vc->sc;
 
   if (j > 2 * winSize + MAXLOOP + 1) {
-    i = j - (2 * winSize + MAXLOOP + 1);
+    i = (size_t)j - (2 * winSize + MAXLOOP + 1);
     /* free arrays may be faster than pointer rotation and reset to 0 values */
     free(pR[i] + i);
     free(q[i] + i);
@@ -756,10 +758,10 @@ vrna_probs_window(vrna_fold_compound_t        *vc,
   max_real = (sizeof(FLT_OR_DBL) == sizeof(float)) ? FLT_MAX : DBL_MAX;
 
   /* start recursions */
-  for (j = turn + 2; j <= n + winSize; j++) {
+  for (j = 2; j <= n + winSize; j++) {
     if (j <= n) {
       vrna_exp_E_ext_fast_update(vc, j, aux_mx_el);
-      for (i = j - turn - 1; i >= MAX2(1, (j - winSize + 1)); i--) {
+      for (i = j - 1; i >= MAX2(1, (j - winSize + 1)); i--) {
         hc_decompose  = hc->matrix_local[i][j - i];
         qbt1          = 0.;
 
@@ -877,7 +879,7 @@ vrna_probs_window(vrna_fold_compound_t        *vc,
             FLT_OR_DBL *stack_probs = compute_stack_probabilities(vc, start);
             stack_probs -= start + 1;
             cb(stack_probs,
-               MIN2(n - start + turn, pairSize),
+               MIN2(n - start, pairSize),
                start,
                winSize,
                VRNA_PROBS_WINDOW_STACKP,
@@ -915,7 +917,7 @@ vrna_probs_window(vrna_fold_compound_t        *vc,
         FLT_OR_DBL *stack_probs = compute_stack_probabilities(vc, start);
         stack_probs -= start + 1;
         cb(stack_probs,
-           MIN2(n - start + turn, pairSize),
+           MIN2(n - start, pairSize),
            start,
            winSize,
            VRNA_PROBS_WINDOW_STACKP,
@@ -974,8 +976,8 @@ sc_contribution(vrna_fold_compound_t  *vc,
          sc->exp_energy_stack[j];
   }
 
-  if (sc->f)
-    q *= sc->f(i, j, k, l, VRNA_DECOMP_PAIR_IL, sc->data);
+  if (sc->exp_f)
+    q *= sc->exp_f(i, j, k, l, VRNA_DECOMP_PAIR_IL, sc->data);
 
   return q;
 }
@@ -1026,7 +1028,7 @@ compute_probs(vrna_fold_compound_t        *vc,
 {
   char              **ptype;
   short             *S1;
-  int               start_i, i, k, l, n, m, winSize, turn, type, type_2, tt, *rtype;
+  int               start_i, i, k, l, n, m, winSize, type, type_2, tt, *rtype;
   FLT_OR_DBL        *prml, *prm_l, *prm_l1, **pR, **QI5, **qmb, **q2l, **qb, **q, **qm,
                     *scale, *expMLbase, expMLclosing, temp, prm_MLb, prmt1, prmt, *tmp,
                     Qmax;
@@ -1050,7 +1052,6 @@ compute_probs(vrna_fold_compound_t        *vc,
   ptype         = vc->ptype_local;
   pf_params     = vc->exp_params;
   md            = &(pf_params->model_details);
-  turn          = md->min_loop_size;
   rtype         = &(md->rtype[0]);
   expMLclosing  = pf_params->expMLclosing;
   scale         = vc->exp_matrices->scale;
@@ -1093,7 +1094,7 @@ compute_probs(vrna_fold_compound_t        *vc,
   k         = j - winSize;
   prm_l1[k] = 0;
 
-  for (l = k + turn + 1; l <= MIN2(n, k + winSize - 1); l++) {
+  for (l = k + 1; l <= MIN2(n, k + winSize - 1); l++) {
     int a;
     pR[k][l]  = 0; /* set zero at start */
     type      = vrna_get_ptype_window(k, l + k, ptype);
@@ -1124,6 +1125,10 @@ compute_probs(vrna_fold_compound_t        *vc,
                                       (k > 1) ? S1[k - 1] : -1,
                                       (l < n) ? S1[l + 1] : -1,
                                       pf_params);
+
+      if ((sc) &&
+          (sc->exp_f))
+        pR[k][l] *= sc->exp_f(k, l, k, l, VRNA_DECOMP_EXT_STEM, sc->data);
     }
 
     if (hc->matrix_local[k][l - k] & VRNA_CONSTRAINT_CONTEXT_INT_LOOP_ENC) {
@@ -1197,7 +1202,7 @@ compute_probs(vrna_fold_compound_t        *vc,
   prm_MLb = 0.;
   if (k > 1) {
     /* sonst nix! */
-    for (l = MIN2(n - 1, k + winSize - 2); l >= k + turn + 1; l--) {
+    for (l = MIN2(n - 1, k + winSize - 2); l >= k + 1; l--) {
       FLT_OR_DBL ppp;
 
       /* opposite direction */
@@ -1211,10 +1216,14 @@ compute_probs(vrna_fold_compound_t        *vc,
                 exp_E_MLstem(tt, S1[m - 1], S1[i + 1], pf_params) *
                 qm[i + 1][k - 1];
 
-          if (sc)
+          if (sc) {
             if (sc->exp_energy_bp_local)
               ppp *= sc->exp_energy_bp_local[i][m - i];
 
+            if (sc->exp_f)
+              ppp *= sc->exp_f(i, m, i + 1, m - 1, VRNA_DECOMP_PAIR_ML, sc->data);
+          }
+
           prmt += ppp;
         }
       }
@@ -1232,19 +1241,27 @@ compute_probs(vrna_fold_compound_t        *vc,
                              pf_params);
 
 
-        if (sc)
+        if (sc) {
           if (sc->exp_energy_bp_local)
             prmt1 *= sc->exp_energy_bp_local[k - 1][m - k + 1];
+
+          if (sc->exp_f)
+            prmt1 *= sc->exp_f(k - 1, m, k, m - 1, VRNA_DECOMP_PAIR_ML, sc->data);
+        }
       }
 
       /* k-1 is unpaired */
       if (hc->up_ml[k - 1]) {
         ppp = prm_l1[m] * expMLbase[1];
 
-        if (sc)
+        if (sc) {
           if (sc->exp_energy_up)
             ppp *= sc->exp_energy_up[k - 1][1];
 
+          if (sc->exp_f)
+            ppp *= sc->exp_f(k - 1, l, k, l, VRNA_DECOMP_ML_ML, sc->data);
+        }
+
         prm_l[m] = ppp + prmt1;
       } else {
         /* skip configuration where k-1 is unpaired */
@@ -1255,10 +1272,12 @@ compute_probs(vrna_fold_compound_t        *vc,
       if (hc->up_ml[m]) {
         ppp = prm_MLb * expMLbase[1];
 
-        if (sc)
+        if (sc) {
           if (sc->exp_energy_up)
             ppp *= sc->exp_energy_up[m][1];
 
+        }
+
         prm_MLb = ppp + prml[m];
       } else {
         prm_MLb = prml[m];
@@ -1286,6 +1305,11 @@ compute_probs(vrna_fold_compound_t        *vc,
                               pf_params) *
                  scale[2];
 
+          if ((sc) &&
+              (sc->exp_f)) {
+            dang *= sc->exp_f(k, l, k, l, VRNA_DECOMP_ML_STEM, sc->data);
+          }
+
           for (m = MIN2(k + winSize - 2, n); m >= l + 2; m--) {
             qmb[l][m - l - 1] += prml[m] * dang;
             q2l[l][m - l - 1] += (prml[m] - prm_l[m]) * dang;
@@ -1301,7 +1325,14 @@ compute_probs(vrna_fold_compound_t        *vc,
         temp *= exp_E_MLstem(tt,
                              (k > 1) ? S1[k - 1] : -1,
                              (l < n) ? S1[l + 1] : -1,
-                             pf_params) * scale[2];
+                             pf_params) *
+                scale[2];
+
+        if ((sc) &&
+            (sc->exp_f)) {
+          temp *= sc->exp_f(k, l, k, l, VRNA_DECOMP_ML_STEM, sc->data);
+        }
+
         pR[k][l] += temp;
       }
 
@@ -1329,18 +1360,17 @@ PRIVATE void
 probability_correction(vrna_fold_compound_t *vc,
                        int                  i)
 {
-  int         j, howoften, pairdist, turn, n, winSize;
+  int         j, howoften, pairdist, n, winSize;
   FLT_OR_DBL  **qb, **pR;
 
   n         = vc->length;
   winSize   = vc->window_size;
-  turn      = vc->exp_params->model_details.min_loop_size;
   howoften  = 0; /* how many samples do we have for this pair */
 
   qb  = vc->exp_matrices->qb_local;
   pR  = vc->exp_matrices->pR;
 
-  for (j = i + turn; j < MIN2(i + winSize, n + 1); j++) {
+  for (j = i; j < MIN2(i + winSize, n + 1); j++) {
     pairdist = (j - i + 1);
     /* 4cases */
     howoften  = MIN2(winSize - pairdist + 1, i);  /* pairdist,start */
@@ -1447,10 +1477,11 @@ compute_stack_probabilities(vrna_fold_compound_t  *vc,
   /* compute dependent pair probabilities */
   char              **ptype;
   short             *S1;
-  int               j, max_j, *rtype, turn, pairsize, length, type, type_2;
+  int               j, max_j, *rtype, pairsize, length, type, type_2;
   FLT_OR_DBL        **qb, *scale, *probs;
   double            tmp;
   vrna_exp_param_t  *pf_params;
+  vrna_sc_t         *sc;
 
   length    = vc->length;
   S1        = vc->sequence_encoding;
@@ -1459,14 +1490,14 @@ compute_stack_probabilities(vrna_fold_compound_t  *vc,
   qb        = vc->exp_matrices->qb_local;
   scale     = vc->exp_matrices->scale;
   rtype     = &(pf_params->model_details.rtype[0]);
-  turn      = pf_params->model_details.min_loop_size;
   pairsize  = pf_params->model_details.max_bp_span;
+  sc        = vc->sc;
 
   max_j = MIN2(start + pairsize, length) - 1;
 
   probs = (FLT_OR_DBL *)vrna_alloc(sizeof(FLT_OR_DBL) * (max_j - start + 1));
 
-  for (j = start + turn + 1; j <= max_j; j++) {
+  for (j = start + 1; j <= max_j; j++) {
     if ((qb[start][j] * qb[start - 1][(j + 1)]) > 10e-200) {
       type    = vrna_get_ptype_window(start - 1, j + 1 + start - 1, ptype);
       type_2  = rtype[vrna_get_ptype_window(start, j + start, ptype)];
@@ -1482,6 +1513,19 @@ compute_stack_probabilities(vrna_fold_compound_t  *vc,
                               S1[j + 1],
                               pf_params) *
                 scale[2];
+
+      if (sc) {
+        if (sc->exp_energy_stack) {
+          tmp *= sc->exp_energy_stack[start] *
+                 sc->exp_energy_stack[j] *
+                 sc->exp_energy_stack[start - 1] *
+                 sc->exp_energy_stack[j + 1];
+        }
+
+        if (sc->exp_f)
+          tmp *= sc->exp_f(start - 1, j + 1, start, j, VRNA_DECOMP_PAIR_IL, sc->data);
+      }
+
       probs[j - start - 1] = tmp;
     }
   }
@@ -1572,8 +1616,8 @@ compute_pU(vrna_fold_compound_t       *vc,
             if (sc->exp_energy_up)
               qqq *= sc->exp_energy_up[k + 1][j3 - k - 1];
 
-            if (sc->f)
-              qqq *= sc->f(i5, j3, i5 + 1, k, VRNA_DECOMP_PAIR_ML, sc->data);
+            if (sc->exp_f)
+              qqq *= sc->exp_f(i5 + 1, j3 - 1, i5 + 1, k, VRNA_DECOMP_ML_ML, sc->data);
           }
 
           temp += qqq;
@@ -1588,8 +1632,8 @@ compute_pU(vrna_fold_compound_t       *vc,
             if (sc->exp_energy_up)
               qqq *= sc->exp_energy_up[i5 + 1][k + ulength - i5];
 
-            if (sc->f)
-              qqq *= sc->f(i5, j3, k + ulength + 1, j3, VRNA_DECOMP_PAIR_ML, sc->data);
+            if (sc->exp_f)
+              qqq *= sc->exp_f(i5 + 1, j3 - 1, k + ulength + 1, j3 - 1, VRNA_DECOMP_ML_ML, sc->data);
           }
 
           temp += qqq;
@@ -1605,8 +1649,8 @@ compute_pU(vrna_fold_compound_t       *vc,
             if (sc->exp_energy_up)
               qqq *= sc->exp_energy_up[k + 1][ulength];
 
-            if (sc->f)
-              qqq *= sc->f(i5, j3, k, k + ulength + 1, VRNA_DECOMP_PAIR_ML_OUTSIDE, sc->data);
+            if (sc->exp_f)
+              qqq *= sc->exp_f(i5 + 1, j3 - 1, k, k + ulength + 1, VRNA_DECOMP_ML_ML_ML, sc->data);
           }
 
           temp += qqq;
@@ -1620,10 +1664,14 @@ compute_pU(vrna_fold_compound_t       *vc,
               scale[2] *
               expMLclosing;
 
-        if (sc)
+        if (sc) {
           if (sc->exp_energy_bp_local)
             qqq *= sc->exp_energy_bp_local[i5][j3 - i5];
 
+          if (sc->exp_f)
+            qqq *= sc->exp_f(i5, j3, i5 + 1, j3 - 1, VRNA_DECOMP_PAIR_ML, sc->data);
+        }
+
         temp *= qqq;
 
         pU[k + ulength][ulength] += temp * pR[i5][j3];
@@ -1746,6 +1794,9 @@ compute_pU(vrna_fold_compound_t       *vc,
 
             if (sc->exp_energy_bp)
               temp *= sc->exp_energy_bp_local[k][obp - k];
+
+            if (sc->exp_f)
+              temp *= sc->exp_f(k, obp, k + len + 1, obp - 1, VRNA_DECOMP_PAIR_ML, sc->data);
           }
 
           QBM[len]  += temp;
@@ -1770,19 +1821,27 @@ compute_pU(vrna_fold_compound_t       *vc,
       pUI[k + len][len] += pUI[k + len][len + 1] + QBI[len];
     }
     /* open chain */
-    if ((ulength >= winSize) && (k >= ulength) && (hc->up_ext[k - winSize + 1] >= winSize))
+    if ((ulength >= winSize) &&
+        (k >= ulength) &&
+        (hc->up_ext[k - winSize + 1] >= winSize))
       pUO[k][winSize] = scale[winSize] / q[k - winSize + 1][k];
   }
 
   /* open chain */
-  if ((ulength >= winSize) && (k >= ulength) && (hc->up_ext[k - winSize + 1] >= winSize)) {
-    if (sc && sc->exp_energy_up) {
-      pU[k][winSize] = scale[winSize] *
-                       sc->exp_energy_up[k][winSize] /
-                       q[k - winSize + 1][k];
-    } else {
-      pU[k][winSize] = scale[winSize] / q[k - winSize + 1][k];
+  if ((ulength >= winSize) &&
+      (k >= ulength) &&
+      (hc->up_ext[k - winSize + 1] >= winSize)) {
+    temp = scale[winSize] / q[k - winSize + 1][k];
+
+    if (sc) {
+      if (sc->exp_energy_up)
+        temp *= sc->exp_energy_up[k][winSize];
+
+      if (sc->exp_f)
+        temp *= sc->exp_f(k - winSize + 1, k, k - winSize + 1, k, VRNA_DECOMP_EXT_UP, sc->data);
     }
+
+    pU[k][winSize] = temp;
   }
 
   /*
