@@ -23,6 +23,7 @@
 #include "ViennaRNA/constraints/hard.h"
 #include "ViennaRNA/constraints/soft.h"
 #include "ViennaRNA/alphabet.h"
+#include "ViennaRNA/combinatorics.h"
 #include "ViennaRNA/boltzmann_sampling.h"
 
 #include "ViennaRNA/loops/external_sc_pf.inc"
@@ -50,11 +51,15 @@
 #endif
 
 
+struct aux_mem {
+  FLT_OR_DBL *qik;
+};
+
 /* combination of soft constraint wrappers */
 struct sc_wrappers {
-  struct sc_wrapper_exp_ext sc_wrapper_ext;
-  struct sc_wrapper_exp_int sc_wrapper_int;
-  struct sc_wrapper_exp_ml  sc_wrapper_ml;
+  struct sc_ext_exp_dat sc_wrapper_ext;
+  struct sc_int_exp_dat sc_wrapper_int;
+  struct sc_mb_exp_dat  sc_wrapper_ml;
 };
 
 /*
@@ -63,6 +68,8 @@ struct sc_wrappers {
  * - current_node is a pointer to current node in datastructure memorizing the solutions and paths taken
  */
 struct vrna_pbacktrack_memory_s {
+  unsigned int      start;
+  unsigned int      end;
   double            q_remain;
   NR_NODE           *root_node;
   NR_NODE           *current_node;
@@ -110,7 +117,9 @@ PRIVATE char  *info_no_circ =
  */
 
 PRIVATE struct vrna_pbacktrack_memory_s *
-nr_init(vrna_fold_compound_t *fc);
+nr_init(vrna_fold_compound_t  *fc,
+        unsigned int          start,
+        unsigned int          end);
 
 
 PRIVATE struct sc_wrappers *
@@ -123,7 +132,8 @@ sc_free(struct sc_wrappers *sc_wrap);
 
 PRIVATE unsigned int
 wrap_pbacktrack(vrna_fold_compound_t              *vc,
-                unsigned int                      length,
+                unsigned int                      start,
+                unsigned int                      end,
                 unsigned int                      num_samples,
                 vrna_boltzmann_sampling_callback  *bs_cb,
                 void                              *data,
@@ -140,10 +150,11 @@ backtrack(int                             i,
 
 
 PRIVATE int
-backtrack_ext_loop(int                              init_val,
+backtrack_ext_loop(int                              start,
+                   int                              end,
                    char                             *pstruc,
                    vrna_fold_compound_t             *vc,
-                   int                              length,
+                   struct aux_mem                   *helper_arrays,
                    struct sc_wrappers               *sc_wrap,
                    struct vrna_pbacktrack_memory_s  *nr_mem);
 
@@ -187,29 +198,32 @@ pbacktrack_circ(vrna_fold_compound_t              *fc,
  #################################
  */
 PUBLIC unsigned int
-vrna_pbacktrack5_resume_cb(vrna_fold_compound_t             *fc,
-                           unsigned int                     num_samples,
-                           unsigned int                     length,
-                           vrna_boltzmann_sampling_callback *bs_cb,
-                           void                             *data,
-                           vrna_pbacktrack_mem_t            *nr_mem,
-                           unsigned int                     options)
+vrna_pbacktrack_sub_resume_cb(vrna_fold_compound_t              *fc,
+                              unsigned int                      num_samples,
+                              unsigned int                      start,
+                              unsigned int                      end,
+                              vrna_boltzmann_sampling_callback  *bs_cb,
+                              void                              *data,
+                              vrna_pbacktrack_mem_t             *nr_mem,
+                              unsigned int                      options)
 {
   unsigned int i = 0;
 
   if (fc) {
     vrna_mx_pf_t *matrices = fc->exp_matrices;
 
-    if (length > fc->length) {
-      vrna_message_warning("vrna_pbacktrack5*(): length exceeds sequence length");
-    } else if (length == 0) {
-      vrna_message_warning("vrna_pbacktrack5*(): length too small");
+    if (start == 0) {
+      vrna_message_warning("vrna_pbacktrack*(): interval start coordinate must be at least 1");
+    } else if (end > fc->length) {
+      vrna_message_warning("vrna_pbacktrack*(): interval end coordinate exceeds sequence length");
+    } else if (end < start) {
+      vrna_message_warning("vrna_pbacktrack*(): interval end < start");
     } else if ((!matrices) || (!matrices->q) || (!matrices->qb) || (!matrices->qm) ||
                (!fc->exp_params)) {
       vrna_message_warning("vrna_pbacktrack*(): %s", info_call_pf);
     } else if ((!fc->exp_params->model_details.uniq_ML) || (!matrices->qm1)) {
       vrna_message_warning("vrna_pbacktrack*(): %s", info_set_uniq_ml);
-    } else if ((fc->exp_params->model_details.circ) && (length < fc->length)) {
+    } else if ((fc->exp_params->model_details.circ) && (end < fc->length)) {
       vrna_message_warning("vrna_pbacktrack5*(): %s", info_no_circ);
     } else if (options & VRNA_PBACKTRACK_NON_REDUNDANT) {
       if (fc->exp_params->model_details.circ) {
@@ -217,10 +231,16 @@ vrna_pbacktrack5_resume_cb(vrna_fold_compound_t             *fc,
       } else if (!nr_mem) {
         vrna_message_warning("vrna_pbacktrack5*(): Pointer to nr_mem must not be NULL!");
       } else {
-        if (*nr_mem == NULL)
-          *nr_mem = nr_init(fc);
+        if ((*nr_mem == NULL) ||
+            ((*nr_mem)->start != start) ||
+            ((*nr_mem)->end != end)) {
+          if (*nr_mem)
+            vrna_pbacktrack_mem_free(*nr_mem);
 
-        i = wrap_pbacktrack(fc, length, num_samples, bs_cb, data, *nr_mem);
+          *nr_mem = nr_init(fc, start, end);
+        }
+
+        i = wrap_pbacktrack(fc, start, end, num_samples, bs_cb, data, *nr_mem);
 
         /* print warning if we've aborted backtracking too early */
         if ((i > 0) && (i < num_samples)) {
@@ -231,13 +251,13 @@ vrna_pbacktrack5_resume_cb(vrna_fold_compound_t             *fc,
                                i,
                                100. *
                                return_node_weight((*nr_mem)->root_node) /
-                               fc->exp_matrices->q[fc->iindx[1] - length]);
+                               fc->exp_matrices->q[fc->iindx[start] - end]);
         }
       }
     } else if (fc->exp_params->model_details.circ) {
       i = pbacktrack_circ(fc, num_samples, bs_cb, data);
     } else {
-      i = wrap_pbacktrack(fc, length, num_samples, bs_cb, data, NULL);
+      i = wrap_pbacktrack(fc, start, end, num_samples, bs_cb, data, NULL);
     }
   }
 
@@ -269,9 +289,9 @@ sc_init(vrna_fold_compound_t *fc)
 {
   struct sc_wrappers *sc_wrap = (struct sc_wrappers *)vrna_alloc(sizeof(struct sc_wrappers));
 
-  init_sc_wrapper_ext(fc, &(sc_wrap->sc_wrapper_ext));
-  init_sc_wrapper_int(fc, &(sc_wrap->sc_wrapper_int));
-  init_sc_wrapper_ml(fc, &(sc_wrap->sc_wrapper_ml));
+  init_sc_ext_exp(fc, &(sc_wrap->sc_wrapper_ext));
+  init_sc_int_exp(fc, &(sc_wrap->sc_wrapper_int));
+  init_sc_mb_exp(fc, &(sc_wrap->sc_wrapper_ml));
 
   return sc_wrap;
 }
@@ -280,16 +300,18 @@ sc_init(vrna_fold_compound_t *fc)
 PRIVATE void
 sc_free(struct sc_wrappers *sc_wrap)
 {
-  free_sc_wrapper_ext(&(sc_wrap->sc_wrapper_ext));
-  free_sc_wrapper_int(&(sc_wrap->sc_wrapper_int));
-  free_sc_wrapper_ml(&(sc_wrap->sc_wrapper_ml));
+  free_sc_ext_exp(&(sc_wrap->sc_wrapper_ext));
+  free_sc_int_exp(&(sc_wrap->sc_wrapper_int));
+  free_sc_mb_exp(&(sc_wrap->sc_wrapper_ml));
 
   free(sc_wrap);
 }
 
 
 PRIVATE struct vrna_pbacktrack_memory_s *
-nr_init(vrna_fold_compound_t *fc)
+nr_init(vrna_fold_compound_t  *fc,
+        unsigned int          start,
+        unsigned int          end)
 {
   size_t                          block_size;
   double                          pf;
@@ -297,14 +319,17 @@ nr_init(vrna_fold_compound_t *fc)
 
   s = (struct vrna_pbacktrack_memory_s *)vrna_alloc(
     sizeof(struct vrna_pbacktrack_memory_s));
-  pf          = fc->exp_matrices->q[fc->iindx[1] - fc->length];
-  block_size  = 5000 * sizeof(NR_NODE);
 
+  s->start      = start;
+  s->end        = end;
   s->memory_dat = NULL;
   s->q_remain   = 0;
 
+  pf          = fc->exp_matrices->q[fc->iindx[start] - end];
+  block_size  = 5000 * sizeof(NR_NODE);
+
 #ifdef VRNA_NR_SAMPLING_HASH
-  s->root_node = create_root(fc->length, pf);
+  s->root_node = create_root(end, pf);
 #else
   s->memory_dat = create_nr_memory(sizeof(NR_NODE), block_size, NULL);  /* memory pre-allocation */
   s->root_node  = create_ll_root(&(s->memory_dat), pf);
@@ -319,57 +344,48 @@ nr_init(vrna_fold_compound_t *fc)
 /* general expr of vrna5_pbacktrack with possibility of non-redundant sampling */
 PRIVATE unsigned int
 wrap_pbacktrack(vrna_fold_compound_t              *vc,
-                unsigned int                      length,
+                unsigned int                      start,
+                unsigned int                      end,
                 unsigned int                      num_samples,
                 vrna_boltzmann_sampling_callback  *bs_cb,
                 void                              *data,
                 struct vrna_pbacktrack_memory_s   *nr_mem)
 {
   char                *pstruc;
-  unsigned int        i, n;
+  unsigned int        i;
   int                 ret, pf_overflow, is_dup, *my_iindx;
-  FLT_OR_DBL          *q1k, *qln, *q;
+  FLT_OR_DBL          *q;
   vrna_mx_pf_t        *matrices;
+  struct aux_mem      helper_arrays;
   struct sc_wrappers  *sc_wrap;
 
   i           = 0;
   pf_overflow = 0;
   sc_wrap     = sc_init(vc);
 
-  n         = vc->length;
   my_iindx  = vc->iindx;
   matrices  = vc->exp_matrices;
   q         = matrices->q;
-  q1k       = matrices->q1k;
-  qln       = matrices->qln;
 
-  if (!(q1k && qln)) {
-    matrices->q1k = (FLT_OR_DBL *)vrna_alloc(sizeof(FLT_OR_DBL) * (n + 1));
-    matrices->qln = (FLT_OR_DBL *)vrna_alloc(sizeof(FLT_OR_DBL) * (n + 2));
-    q1k           = matrices->q1k;
-    qln           = matrices->qln;
-    for (i = 1; i <= n; i++) {
-      q1k[i]  = q[my_iindx[1] - i];
-      qln[i]  = q[my_iindx[i] - n];
-    }
-    q1k[0]      = 1.0;
-    qln[n + 1]  = 1.0;
-  }
+  helper_arrays.qik = (FLT_OR_DBL *)vrna_alloc(sizeof(FLT_OR_DBL) * (end - start + 2));
+  helper_arrays.qik -= start - 1;
+
+  for (i = start; i <= end; i++)
+    helper_arrays.qik[i] = q[my_iindx[start] - i];
+
+  helper_arrays.qik[start - 1] = 1.0;
 
   for (i = 0; i < num_samples; i++) {
     is_dup  = 1;
-    pstruc  = vrna_alloc((length + 1) * sizeof(char));
+    pstruc  = vrna_alloc(((end - start + 1) + 1) * sizeof(char));
+    memset(pstruc, '.', sizeof(char) * (end - start + 1));
+    pstruc -= start - 1;
 
-    memset(pstruc, '.', sizeof(char) * length);
 
     if (nr_mem)
-      nr_mem->q_remain = vc->exp_matrices->q[vc->iindx[1] - length];
+      nr_mem->q_remain = vc->exp_matrices->q[vc->iindx[start] - end]; /* really */
 
-#ifdef VRNA_WITH_BOUSTROPHEDON
-    ret = backtrack_ext_loop(length, pstruc, vc, length, sc_wrap, nr_mem);
-#else
-    ret = backtrack_ext_loop(1, pstruc, vc, length, sc_wrap, nr_mem);
-#endif
+    ret = backtrack_ext_loop(start, end, pstruc, vc, &helper_arrays, sc_wrap, nr_mem);
 
     if (nr_mem) {
 #ifdef VRNA_NR_SAMPLING_HASH
@@ -386,26 +402,29 @@ wrap_pbacktrack(vrna_fold_compound_t              *vc,
 
       if (pf_overflow) {
         vrna_message_warning("vrna_pbacktrack_nr*(): %s", info_nr_overflow);
-        free(pstruc);
+        free(pstruc + (start - 1));
         break;
       }
 
       if (is_dup) {
         vrna_message_warning("vrna_pbacktrack_nr*(): %s", info_nr_duplicates);
-        free(pstruc);
+        free(pstruc + (start - 1));
         break;
       }
     }
 
     if ((ret > 0) && (bs_cb))
-      bs_cb(pstruc, data);
+      bs_cb(pstruc + (start - 1), data);
 
-    free(pstruc);
+    free(pstruc + (start - 1));
 
     if (ret == 0)
       break;
   }
 
+  helper_arrays.qik += start - 1;
+  free(helper_arrays.qik);
+
   sc_free(sc_wrap);
 
   return i;
@@ -414,28 +433,29 @@ wrap_pbacktrack(vrna_fold_compound_t              *vc,
 
 /* backtrack one external */
 PRIVATE int
-backtrack_ext_loop(int                              init_val,
+backtrack_ext_loop(int                              start,
+                   int                              end,
                    char                             *pstruc,
                    vrna_fold_compound_t             *vc,
-                   int                              length,
+                   struct aux_mem                   *helper_arrays,
                    struct sc_wrappers               *sc_wrap,
                    struct vrna_pbacktrack_memory_s  *nr_mem)
 {
-  unsigned char             *hard_constraints;
-  short                     *S1, *S2, **S, **S5, **S3;
-  unsigned int              **a2s, s, n_seq;
-  int                       ret, i, j, ij, n, k, u, type, *my_iindx, hc_decompose, *hc_up_ext;
-  FLT_OR_DBL                r, fbd, fbds, qt, q_temp, qkl, *q, *qb, *q1k, *qln, *scale;
-  double                    *q_remain;
-  vrna_mx_pf_t              *matrices;
-  vrna_md_t                 *md;
-  vrna_hc_t                 *hc;
-  vrna_exp_param_t          *pf_params;
+  unsigned char         *hard_constraints;
+  short                 *S1, *S2, **S, **S5, **S3;
+  unsigned int          **a2s, s, n_seq;
+  int                   ret, i, j, ij, n, k, type, *my_iindx, hc_decompose, *hc_up_ext;
+  FLT_OR_DBL            r, fbd, fbds, qt, q_temp, qkl, *qb, *q1k, *scale;
+  double                *q_remain;
+  vrna_mx_pf_t          *matrices;
+  vrna_md_t             *md;
+  vrna_hc_t             *hc;
+  vrna_exp_param_t      *pf_params;
 
-  struct nr_memory          **memory_dat;
-  struct sc_wrapper_exp_ext *sc_wrapper_ext;
+  struct nr_memory      **memory_dat;
+  struct sc_ext_exp_dat *sc_wrapper_ext;
 
-  NR_NODE                   **current_node;
+  NR_NODE               **current_node;
 
   if (nr_mem) {
     q_remain      = &(nr_mem->q_remain);
@@ -489,10 +509,8 @@ backtrack_ext_loop(int                              init_val,
   /* assume successful backtracing by default */
   ret = 1;
 
-  q     = matrices->q;
   qb    = matrices->qb;
-  q1k   = matrices->q1k;
-  qln   = matrices->qln;
+  q1k   = helper_arrays->qik;
   scale = matrices->scale;
 
 #ifndef VRNA_NR_SAMPLING_HASH
@@ -505,11 +523,10 @@ backtrack_ext_loop(int                              init_val,
 
   q_temp = 0.;
 
-#ifdef VRNA_WITH_BOUSTROPHEDON
-  j = init_val;
-  if (j > 1) {
+  j = end;
+  if (j > start) {
     /* find j position of first pair */
-    for (; j > 1; j--) {
+    for (; j > start; j--) {
       if (hc_up_ext[j]) {
         if (current_node) {
           fbd = NR_TOTAL_WEIGHT(*current_node) *
@@ -529,7 +546,7 @@ backtrack_ext_loop(int                              init_val,
         q_temp  = q1k[j - 1] * scale[1];
 
         if (sc_wrapper_ext->red_ext)
-          q_temp *= sc_wrapper_ext->red_ext(1, j, 1, j - 1, sc_wrapper_ext);
+          q_temp *= sc_wrapper_ext->red_ext(start, j, start, j - 1, sc_wrapper_ext);
 
         if (current_node) {
           fbds = NR_GET_WEIGHT(*current_node, memorized_node_cur, NRT_UNPAIRED_SG, j - 1, j) *
@@ -558,7 +575,7 @@ backtrack_ext_loop(int                              init_val,
         }
       }
     }
-    if (j <= md->min_loop_size + 1)
+    if (j <= start + md->min_loop_size)
       return ret;         /* no more pairs, but still successful */
 
 #ifndef  VRNA_NR_SAMPLING_HASH
@@ -574,13 +591,13 @@ backtrack_ext_loop(int                              init_val,
     }
 
     r = vrna_urn() * (q1k[j] - q_temp - fbd);
-    u = j - 1;
     i = 2;
 
-    for (qt = 0, k = 1; k < j; k++) {
+    unsigned int *is = vrna_boustrophedon(start, j - 1);
+
+    for (qt = 0, k = 1; k + start <= j; k++) {
       /* apply alternating boustrophedon scheme to variable i */
-      i = (int)(1 + (u - 1) * ((k - 1) % 2)) +
-          (int)((1 - (2 * ((k - 1) % 2))) * ((k - 1) / 2));
+      i             = is[k];
       ij            = my_iindx[i] - j;
       hc_decompose  = hard_constraints[n * j + i];
       if (hc_decompose & VRNA_CONSTRAINT_CONTEXT_EXT_LOOP) {
@@ -645,7 +662,10 @@ backtrack_ext_loop(int                              init_val,
 #endif
       }
     }
-    if (k == j) {
+
+    free(is);
+
+    if (k + start > j) {
       if (current_node) {
         /* exhausted ensemble */
         return 0;
@@ -658,157 +678,9 @@ backtrack_ext_loop(int                              init_val,
 
     backtrack(i, j, pstruc, vc, sc_wrap, nr_mem);
     j   = i - 1;
-    ret = backtrack_ext_loop(j, pstruc, vc, length, sc_wrap, nr_mem);
+    ret = backtrack_ext_loop(start, j, pstruc, vc, helper_arrays, sc_wrap, nr_mem);
   }
 
-#else
-  int start = init_val;
-  if (start < length) {
-    /* find i position of first pair */
-    for (i = start; i < length; i++) {
-      if (hc_up_ext[i]) {
-        if (current_node) {
-          fbd = NR_TOTAL_WEIGHT(*current_node) *
-                qln[i] /
-                (*q_remain);
-        }
-
-        r       = vrna_urn() * (qln[i] - fbd);
-        q_temp  = qln[i + 1] * scale[1];
-
-        if (sc_wrapper_ext->red_ext)
-          q_temp *= sc_wrapper_ext->red_ext(i, length, i + 1, length, sc_wrapper_ext);
-
-        if (current_node) {
-          fbds = NR_GET_WEIGHT(*current_node, memorized_node_cur, NRT_UNPAIRED_SG, i, i + 1) *
-                 qln[i] /
-                 (*q_remain);
-        }
-
-        if (r > (q_temp - fbds)) {
-          break;                /* i is paired */
-        } else if (current_node) {
-          *q_remain *= q_temp / qln[i];
-#ifdef VRNA_NR_SAMPLING_HASH
-          *current_node = add_if_nexists(NRT_UNPAIRED_SG, i, i + 1, *current_node, *q_remain);
-#else
-          *current_node = add_if_nexists_ll(memory_dat,
-                                            NRT_UNPAIRED_SG,
-                                            i,
-                                            i + 1,
-                                            memorized_node_prev,
-                                            memorized_node_cur,
-                                            *current_node,
-                                            *q_remain);
-          reset_cursor(&memorized_node_prev, &memorized_node_cur, *current_node); /* resets cursor */
-#endif
-        }
-      }
-    }
-    if (i >= length)
-      return ret;         /* no more pairs, but still successful */
-
-#ifndef VRNA_NR_SAMPLING_HASH
-    if (current_node)
-      advance_cursor(&memorized_node_prev, &memorized_node_cur, NRT_UNPAIRED_SG, i, i + 1);
-
-#endif
-
-    /* now find the pairing partner j */
-    if (current_node) {
-      fbd = NR_TOTAL_WEIGHT_TYPE(NRT_EXT_LOOP, *current_node) *
-            qln[i] /
-            (*q_remain);
-    }
-
-    r = vrna_urn() * (qln[i] - q_temp - fbd);
-    for (qt = 0, j = i + 1; j <= length; j++) {
-      ij            = my_iindx[i] - j;
-      hc_decompose  = hard_constraints[n * i + j];
-      if (hc_decompose & VRNA_CONSTRAINT_CONTEXT_EXT_LOOP) {
-        qkl = qb[ij];
-        if (vc->type == VRNA_FC_TYPE_SINGLE) {
-          type  = vrna_get_ptype_md(S2[i], S2[j], md);
-          qkl   *= vrna_exp_E_ext_stem(type,
-                                       (i > 1) ? S1[i - 1] : -1,
-                                       (j < n) ? S1[j + 1] : -1,
-                                       pf_params);
-        } else {
-          for (s = 0; s < n_seq; s++) {
-            type  = vrna_get_ptype_md(S[s][i], S[s][j], md);
-            qkl   *= vrna_exp_E_ext_stem(type,
-                                         (a2s[s][i] > 1) ? S5[s][i] : -1,
-                                         (a2s[s][j] < a2s[s][n]) ? S3[s][j] : -1,
-                                         pf_params);
-          }
-        }
-
-        if (j < length) {
-          qkl *= qln[j + 1];
-          if (sc_wrapper_ext->split)
-            qkl *= sc_wrapper_ext->split(i, length, j + 1, sc_wrapper_ext) *
-                   sc_wrapper_ext->red_stem(i, j, i, j, sc_wrapper_ext);
-        } else if (sc_wrapper_ext->red_stem) {
-          qkl *= sc_wrapper_ext->red_stem(i, j, i, j, sc_wrapper_ext);
-        }
-
-        if (current_node) {
-          fbds = NR_GET_WEIGHT(*current_node, memorized_node_cur, NRT_EXT_LOOP, i, j) *
-                 qln[i] /
-                 (*q_remain);
-          qt += qkl - fbds;
-        } else {
-          qt += qkl;
-        }
-
-        if (qt > r) {
-          if (current_node) {
-            *q_remain *= qkl / qln[i];
-#ifdef VRNA_NR_SAMPLING_HASH
-            *current_node = add_if_nexists(NRT_EXT_LOOP, i, j, *current_node, *q_remain);
-#else
-            *current_node = add_if_nexists_ll(memory_dat,
-                                              NRT_EXT_LOOP,
-                                              i,
-                                              j,
-                                              memorized_node_prev,
-                                              memorized_node_cur,
-                                              *current_node,
-                                              *q_remain);
-#endif
-          }
-
-          break;       /* j is paired */
-        }
-
-#ifndef VRNA_NR_SAMPLING_HASH
-        if (current_node)
-          advance_cursor(&memorized_node_prev, &memorized_node_cur, NRT_EXT_LOOP, i, j);
-
-#endif
-      }
-    }
-    if (j == length + 1) {
-      if (current_node) {
-        /* exhausted ensemble */
-        return 0;
-      } else {
-        vrna_message_warning("backtracking failed in ext loop");
-        /* error */
-        return -1;
-      }
-    }
-
-    start = j + 1;
-    ret   = backtrack(i, j, pstruc, vc, sc_wrap, nr_mem);
-    if (!ret)
-      return ret;
-
-    ret = backtrack_ext_loop(start, pstruc, vc, length, sc_wrap, nr_mem);
-  }
-
-#endif
-
   return ret;
 }
 
@@ -823,16 +695,16 @@ backtrack_qm(int                              i,
              struct vrna_pbacktrack_memory_s  *nr_mem)
 {
   /* divide multiloop into qm and qm1  */
-  int                       k, u, cnt, span, turn, is_unpaired, *my_iindx, *jindx, *hc_up_ml, ret;
-  FLT_OR_DBL                qmt, fbd, fbds, r, q_temp, *qm, *qm1, *expMLbase;
-  double                    *q_remain;
-  vrna_hc_t                 *hc;
-  vrna_mx_pf_t              *matrices;
+  int                   k, u, cnt, span, turn, is_unpaired, *my_iindx, *jindx, *hc_up_ml, ret;
+  FLT_OR_DBL            qmt, fbd, fbds, r, q_temp, *qm, *qm1, *expMLbase;
+  double                *q_remain;
+  vrna_hc_t             *hc;
+  vrna_mx_pf_t          *matrices;
 
-  struct sc_wrapper_exp_ml  *sc_wrapper_ml;
-  struct nr_memory          **memory_dat;
+  struct sc_mb_exp_dat  *sc_wrapper_ml;
+  struct nr_memory      **memory_dat;
 
-  NR_NODE                   **current_node;
+  NR_NODE               **current_node;
 
   if (nr_mem) {
     q_remain      = &(nr_mem->q_remain);
@@ -908,12 +780,8 @@ backtrack_qm(int                              i,
 #endif
 
       for (span = j - i, cnt = i + 1; cnt <= j; cnt++) {
-#ifdef VRNA_WITH_BOUSTROPHEDON
         k = (int)(i + 1 + span * ((cnt - i - 1) % 2)) +
             (int)((1 - (2 * ((cnt - i - 1) % 2))) * ((cnt - i) / 2));
-#else
-        k = cnt;
-#endif
         q_temp  = 0.;
         u       = k - i;
         /* [i...k] is unpaired */
@@ -983,7 +851,7 @@ backtrack_qm(int                              i,
         *current_node = add_if_nexists(NRT_QM_PAIR, k, 0, *current_node, *q_remain);
 
 #else
-      if (is_unpaired)
+      if (is_unpaired) {
         *current_node = add_if_nexists_ll(memory_dat,
                                           NRT_QM_UNPAIR,
                                           k,
@@ -992,7 +860,7 @@ backtrack_qm(int                              i,
                                           memorized_node_cur,
                                           *current_node,
                                           *q_remain);
-      else
+      } else {
         *current_node = add_if_nexists_ll(memory_dat,
                                           NRT_QM_PAIR,
                                           k,
@@ -1001,6 +869,7 @@ backtrack_qm(int                              i,
                                           memorized_node_cur,
                                           *current_node,
                                           *q_remain);
+      }
 
 #endif
     }
@@ -1038,22 +907,22 @@ backtrack_qm1(int                             i,
               struct vrna_pbacktrack_memory_s *nr_mem)
 {
   /* i is paired to l, i<l<j; backtrack in qm1 to find l */
-  unsigned char             *hard_constraints;
-  char                      *ptype;
-  short                     *S1, **S, **S5, **S3;
-  unsigned int              n, s, n_seq;
-  int                       ii, l, il, type, turn, u, *my_iindx, *jindx, *hc_up_ml;
-  FLT_OR_DBL                qt, fbd, fbds, r, q_temp, *qm1, *qb, *expMLbase;
-  double                    *q_remain;
-  vrna_exp_param_t          *pf_params;
-  vrna_md_t                 *md;
-  vrna_hc_t                 *hc;
-  vrna_mx_pf_t              *matrices;
+  unsigned char         *hard_constraints;
+  char                  *ptype;
+  short                 *S1, **S, **S5, **S3;
+  unsigned int          n, s, n_seq;
+  int                   ii, l, il, type, turn, u, *my_iindx, *jindx, *hc_up_ml;
+  FLT_OR_DBL            qt, fbd, fbds, r, q_temp, *qm1, *qb, *expMLbase;
+  double                *q_remain;
+  vrna_exp_param_t      *pf_params;
+  vrna_md_t             *md;
+  vrna_hc_t             *hc;
+  vrna_mx_pf_t          *matrices;
 
-  struct nr_memory          **memory_dat;
-  struct sc_wrapper_exp_ml  *sc_wrapper_ml;
+  struct nr_memory      **memory_dat;
+  struct sc_mb_exp_dat  *sc_wrapper_ml;
 
-  NR_NODE                   **current_node;
+  NR_NODE               **current_node;
 
   if (nr_mem) {
     q_remain      = &(nr_mem->q_remain);
@@ -1202,9 +1071,9 @@ backtrack_qm2(int                   k,
               vrna_fold_compound_t  *vc,
               struct sc_wrappers    *sc_wrap)
 {
-  int                       u, turn, *jindx;
-  FLT_OR_DBL                qom2t, r, *qm1, *qm2;
-  struct sc_wrapper_exp_ml  *sc_wrapper_ml;
+  int                   u, turn, *jindx;
+  FLT_OR_DBL            qom2t, r, *qm1, *qm2;
+  struct sc_mb_exp_dat  *sc_wrapper_ml;
 
   jindx         = vc->jindx;
   qm1           = vc->exp_matrices->qm1;
@@ -1247,25 +1116,25 @@ backtrack(int                             i,
           struct sc_wrappers              *sc_wrap,
           struct vrna_pbacktrack_memory_s *nr_mem)
 {
-  unsigned char             *hard_constraints, hc_decompose;
-  char                      *ptype;
-  short                     *S1, **S, **S5, **S3;
-  unsigned int              **a2s, s, n_seq, n, type, type_2, *types, u1_local, u2_local;
-  int                       *my_iindx, *jindx, *hc_up_int, ret, *pscore, turn, *rtype,
-                            k, l, kl, u1, u2, max_k, min_l, ii, jj;
-  FLT_OR_DBL                *qb, *qm, *qm1, *scale, r, fbd, fbds, qbt1, qbr, qt, q_temp,
-                            kTn, closingPair, expMLclosing;
-  double                    *q_remain;
-  vrna_mx_pf_t              *matrices;
-  vrna_exp_param_t          *pf_params;
-  vrna_md_t                 *md;
-  vrna_hc_t                 *hc;
+  unsigned char         *hard_constraints, hc_decompose;
+  char                  *ptype;
+  short                 *S1, **S, **S5, **S3;
+  unsigned int          **a2s, s, n_seq, n, type, type_2, *types, u1_local, u2_local;
+  int                   *my_iindx, *jindx, *hc_up_int, ret, *pscore, turn, *rtype,
+                        k, l, kl, u1, u2, max_k, min_l, ii, jj;
+  FLT_OR_DBL            *qb, *qm, *qm1, *scale, r, fbd, fbds, qbt1, qbr, qt, q_temp,
+                        kTn, closingPair, expMLclosing;
+  double                *q_remain;
+  vrna_mx_pf_t          *matrices;
+  vrna_exp_param_t      *pf_params;
+  vrna_md_t             *md;
+  vrna_hc_t             *hc;
 
-  struct nr_memory          **memory_dat;
-  struct sc_wrapper_exp_int *sc_wrapper_int;
-  struct sc_wrapper_exp_ml  *sc_wrapper_ml;
+  struct nr_memory      **memory_dat;
+  struct sc_int_exp_dat *sc_wrapper_int;
+  struct sc_mb_exp_dat  *sc_wrapper_ml;
 
-  NR_NODE                   **current_node;
+  NR_NODE               **current_node;
 
   if (nr_mem) {
     q_remain      = &(nr_mem->q_remain);
@@ -1541,11 +1410,12 @@ backtrack(int                             i,
         q_temp = qm[ii - (k - 1)] *
                  qm1[jj + k] *
                  closingPair *
-                 sc_wrapper_ml->decomp_ml(i,
-                                          j,
-                                          k - 1,
-                                          k,
-                                          sc_wrapper_ml);
+                 sc_wrapper_ml->
+                 decomp_ml(i,
+                           j,
+                           k - 1,
+                           k,
+                           sc_wrapper_ml);
 
 
         if (current_node) {
@@ -1645,22 +1515,22 @@ pbacktrack_circ(vrna_fold_compound_t              *vc,
                 vrna_boltzmann_sampling_callback  *bs_cb,
                 void                              *data)
 {
-  unsigned char             *hc_mx, eval_loop;
-  char                      *pstruc;
-  short                     *S1, *S2, **S, **S5, **S3;
-  unsigned int              type, type2, *tt, s, n_seq, **a2s, u1_local,
-                            u2_local, u3_local, count;
-  int                       i, j, k, l, n, u, *hc_up, *my_iindx, turn,
-                            ln1, ln2, ln3, lstart;
-  FLT_OR_DBL                r, qt, q_temp, qo, qmo, *scale, *qb, *qm, *qm2,
-                            qb_ij, expMLclosing;
-  vrna_exp_param_t          *pf_params;
-  vrna_md_t                 *md;
-  vrna_mx_pf_t              *matrices;
-  struct sc_wrappers        *sc_wrap;
-  struct sc_wrapper_exp_ext *sc_wrapper_ext;
-  struct sc_wrapper_exp_int *sc_wrapper_int;
-  struct sc_wrapper_exp_ml  *sc_wrapper_ml;
+  unsigned char         *hc_mx, eval_loop;
+  char                  *pstruc;
+  short                 *S1, *S2, **S, **S5, **S3;
+  unsigned int          type, type2, *tt, s, n_seq, **a2s, u1_local,
+                        u2_local, u3_local, count;
+  int                   i, j, k, l, n, u, *hc_up, *my_iindx, turn,
+                        ln1, ln2, ln3, lstart;
+  FLT_OR_DBL            r, qt, q_temp, qo, qmo, *scale, *qb, *qm, *qm2,
+                        qb_ij, expMLclosing;
+  vrna_exp_param_t      *pf_params;
+  vrna_md_t             *md;
+  vrna_mx_pf_t          *matrices;
+  struct sc_wrappers    *sc_wrap;
+  struct sc_ext_exp_dat *sc_wrapper_ext;
+  struct sc_int_exp_dat *sc_wrapper_int;
+  struct sc_mb_exp_dat  *sc_wrapper_ml;
 
   n             = vc->length;
   pf_params     = vc->exp_params;
@@ -1840,11 +1710,12 @@ pbacktrack_circ(vrna_fold_compound_t              *vc,
           qt += qm[my_iindx[1] - k] *
                 qm2[k + 1] *
                 expMLclosing *
-                sc_wrapper_ml->decomp_ml(1,
-                                         n,
-                                         k,
-                                         k + 1,
-                                         sc_wrapper_ml);
+                sc_wrapper_ml->
+                decomp_ml(1,
+                          n,
+                          k,
+                          k + 1,
+                          sc_wrapper_ml);
 
 
           /* backtrack in qm and qm2 if we've found a valid barrier k  */
