@@ -75,10 +75,10 @@
  *  This function allows for energy evaluation of a given pair of structure
  *  and sequence (alignment).
  *  Model details, energy parameters, and possibly soft constraints are used as provided
- *  via the parameter 'vc'. The #vrna_fold_compound_t does not need to contain any DP matrices,
+ *  via the parameter 'fc'. The #vrna_fold_compound_t does not need to contain any DP matrices,
  *  but requires all most basic init values as one would get from a call like this:
  *  @code{.c}
- * vc = vrna_fold_compound(sequence, NULL, VRNA_OPTION_EVAL_ONLY);
+ * fc = vrna_fold_compound(sequence, NULL, VRNA_OPTION_EVAL_ONLY);
  *  @endcode
  *
  *  @note Accepts vrna_fold_compound_t of type #VRNA_FC_TYPE_SINGLE and #VRNA_FC_TYPE_COMPARATIVE
@@ -86,12 +86,12 @@
  *  @see  vrna_eval_structure_pt(), vrna_eval_structure_verbose(), vrna_eval_structure_pt_verbose(),
  *        vrna_fold_compound(), vrna_fold_compound_comparative(), vrna_eval_covar_structure()
  *
- *  @param vc               A vrna_fold_compound_t containing the energy parameters and model details
+ *  @param fc               A vrna_fold_compound_t containing the energy parameters and model details
  *  @param structure        Secondary structure in dot-bracket notation
  *  @return                 The free energy of the input structure given the input sequence in kcal/mol
  */
 float
-vrna_eval_structure(vrna_fold_compound_t  *vc,
+vrna_eval_structure(vrna_fold_compound_t  *fc,
                     const char            *structure);
 
 
@@ -104,19 +104,19 @@ vrna_eval_structure(vrna_fold_compound_t  *vc,
  *  The #vrna_fold_compound_t does not need to contain any DP matrices, but requires all most basic
  *  init values as one would get from a call like this:
  *  @code{.c}
- * vc = vrna_fold_compound_comparative(alignment, NULL, VRNA_OPTION_EVAL_ONLY);
+ * fc = vrna_fold_compound_comparative(alignment, NULL, VRNA_OPTION_EVAL_ONLY);
  *  @endcode
  *
  *  @note Accepts vrna_fold_compound_t of type #VRNA_FC_TYPE_COMPARATIVE only!
  *
  *  @see  vrna_fold_compound_comparative(), vrna_eval_structure()
  *
- *  @param vc               A vrna_fold_compound_t containing the energy parameters and model details
+ *  @param fc               A vrna_fold_compound_t containing the energy parameters and model details
  *  @param structure        Secondary (consensus) structure in dot-bracket notation
  *  @return                 The covariance pseudo energy score of the input structure given the input sequence alignment in kcal/mol
  */
 float
-vrna_eval_covar_structure(vrna_fold_compound_t  *vc,
+vrna_eval_covar_structure(vrna_fold_compound_t  *fc,
                           const char            *structure);
 
 
@@ -128,13 +128,13 @@ vrna_eval_covar_structure(vrna_fold_compound_t  *vc,
  *
  *  @see vrna_eval_structure_pt(), vrna_eval_structure_verbose(), vrna_eval_structure_pt_verbose(),
  *
- *  @param vc               A vrna_fold_compound_t containing the energy parameters and model details
+ *  @param fc               A vrna_fold_compound_t containing the energy parameters and model details
  *  @param structure        Secondary structure in dot-bracket notation
  *  @param file             A file handle where this function should print to (may be NULL).
  *  @return                 The free energy of the input structure given the input sequence in kcal/mol
  */
 float
-vrna_eval_structure_verbose(vrna_fold_compound_t  *vc,
+vrna_eval_structure_verbose(vrna_fold_compound_t  *fc,
                             const char            *structure,
                             FILE                  *file);
 
@@ -150,29 +150,29 @@ vrna_eval_structure_verbose(vrna_fold_compound_t  *vc,
  *  data is printed. A negative parameter @p verbosity_level turns off printing all together.
  *
  *  Model details, energy parameters, and possibly soft constraints are used as provided
- *  via the parameter 'vc'. The fold_compound does not need to contain any DP matrices,
+ *  via the parameter 'fc'. The fold_compound does not need to contain any DP matrices,
  *  but all the most basic init values as one would get from a call like this:
  *  @code{.c}
- * vc = vrna_fold_compound(sequence, NULL, VRNA_OPTION_EVAL_ONLY);
+ * fc = vrna_fold_compound(sequence, NULL, VRNA_OPTION_EVAL_ONLY);
  *  @endcode
  *
  *  @see vrna_eval_structure_pt(), vrna_eval_structure_verbose(), vrna_eval_structure_pt_verbose(),
  *
- *  @param vc               A vrna_fold_compound_t containing the energy parameters and model details
+ *  @param fc               A vrna_fold_compound_t containing the energy parameters and model details
  *  @param structure        Secondary structure in dot-bracket notation
  *  @param verbosity_level  The level of verbosity of this function
  *  @param file             A file handle where this function should print to (may be NULL).
  *  @return                 The free energy of the input structure given the input sequence in kcal/mol
  */
 float
-vrna_eval_structure_v(vrna_fold_compound_t  *vc,
+vrna_eval_structure_v(vrna_fold_compound_t  *fc,
                       const char            *structure,
                       int                   verbosity_level,
                       FILE                  *file);
 
 
 float
-vrna_eval_structure_cstr(vrna_fold_compound_t *vc,
+vrna_eval_structure_cstr(vrna_fold_compound_t *fc,
                          const char           *structure,
                          int                  verbosity_level,
                          vrna_cstr_t          output_stream);
@@ -193,20 +193,20 @@ vrna_eval_structure_cstr(vrna_fold_compound_t *vc,
  *  This function allows for energy evaluation of a given sequence/structure pair where
  *  the structure is provided in pair_table format as obtained from vrna_ptable().
  *  Model details, energy parameters, and possibly soft constraints are used as provided
- *  via the parameter 'vc'. The fold_compound does not need to contain any DP matrices,
+ *  via the parameter 'fc'. The fold_compound does not need to contain any DP matrices,
  *  but all the most basic init values as one would get from a call like this:
  *  @code{.c}
- * vc = vrna_fold_compound(sequence, NULL, VRNA_OPTION_EVAL_ONLY);
+ * fc = vrna_fold_compound(sequence, NULL, VRNA_OPTION_EVAL_ONLY);
  *  @endcode
  *
  *  @see vrna_ptable(), vrna_eval_structure(), vrna_eval_structure_pt_verbose()
  *
- *  @param vc               A vrna_fold_compound_t containing the energy parameters and model details
+ *  @param fc               A vrna_fold_compound_t containing the energy parameters and model details
  *  @param pt               Secondary structure as pair_table
  *  @return                 The free energy of the input structure given the input sequence in 10cal/mol
  */
 int
-vrna_eval_structure_pt(vrna_fold_compound_t *vc,
+vrna_eval_structure_pt(vrna_fold_compound_t *fc,
                        const short          *pt);
 
 
@@ -218,13 +218,13 @@ vrna_eval_structure_pt(vrna_fold_compound_t *vc,
  *
  *  @see vrna_eval_structure_pt_v(), vrna_ptable(), vrna_eval_structure_pt(), vrna_eval_structure_verbose()
  *
- *  @param vc               A vrna_fold_compound_t containing the energy parameters and model details
+ *  @param fc               A vrna_fold_compound_t containing the energy parameters and model details
  *  @param pt               Secondary structure as pair_table
  *  @param file             A file handle where this function should print to (may be NULL).
  *  @return                 The free energy of the input structure given the input sequence in 10cal/mol
  */
 int
-vrna_eval_structure_pt_verbose(vrna_fold_compound_t *vc,
+vrna_eval_structure_pt_verbose(vrna_fold_compound_t *fc,
                                const short          *pt,
                                FILE                 *file);
 
@@ -235,10 +235,10 @@ vrna_eval_structure_pt_verbose(vrna_fold_compound_t *vc,
  *  This function allows for energy evaluation of a given sequence/structure pair where
  *  the structure is provided in pair_table format as obtained from vrna_ptable().
  *  Model details, energy parameters, and possibly soft constraints are used as provided
- *  via the parameter 'vc'. The fold_compound does not need to contain any DP matrices,
+ *  via the parameter 'fc'. The fold_compound does not need to contain any DP matrices,
  *  but all the most basic init values as one would get from a call like this:
  *  @code{.c}
- * vc = vrna_fold_compound(sequence, NULL, VRNA_OPTION_EVAL_ONLY);
+ * fc = vrna_fold_compound(sequence, NULL, VRNA_OPTION_EVAL_ONLY);
  *  @endcode
  *  In contrast to vrna_eval_structure_pt() this function prints detailed energy contributions
  *  based on individual loops to a file handle. If NULL is passed as file handle, this function
@@ -248,14 +248,14 @@ vrna_eval_structure_pt_verbose(vrna_fold_compound_t *vc,
  *
  *  @see vrna_ptable(), vrna_eval_structure_pt(), vrna_eval_structure_verbose()
  *
- *  @param vc               A vrna_fold_compound_t containing the energy parameters and model details
+ *  @param fc               A vrna_fold_compound_t containing the energy parameters and model details
  *  @param pt               Secondary structure as pair_table
  *  @param verbosity_level  The level of verbosity of this function
  *  @param file             A file handle where this function should print to (may be NULL).
  *  @return                 The free energy of the input structure given the input sequence in 10cal/mol
  */
 int
-vrna_eval_structure_pt_v(vrna_fold_compound_t *vc,
+vrna_eval_structure_pt_v(vrna_fold_compound_t *fc,
                          const short          *pt,
                          int                  verbosity_level,
                          FILE                 *file);
@@ -788,10 +788,10 @@ vrna_eval_structure_pt_simple_verbose(const char  *string,
  *  This function allows for energy evaluation of a given sequence/structure pair where
  *  the structure is provided in pair_table format as obtained from vrna_ptable().
  *  Model details, energy parameters, and possibly soft constraints are used as provided
- *  via the parameter 'vc'. The fold_compound does not need to contain any DP matrices,
+ *  via the parameter 'fc'. The fold_compound does not need to contain any DP matrices,
  *  but all the most basic init values as one would get from a call like this:
  *  @code{.c}
- * vc = vrna_fold_compound(sequence, NULL, VRNA_OPTION_EVAL_ONLY);
+ * fc = vrna_fold_compound(sequence, NULL, VRNA_OPTION_EVAL_ONLY);
  *  @endcode
  *  In contrast to vrna_eval_structure_pt_verbose() this function assumes default model details
  *  and default energy parameters in order to evaluate the free energy of the secondary
@@ -879,13 +879,13 @@ vrna_eval_consensus_structure_pt_simple_v(const char  **alignment,
 /**
  * @brief Calculate energy of a loop
  *
- *  @param vc         A vrna_fold_compound_t containing the energy parameters and model details
+ *  @param fc         A vrna_fold_compound_t containing the energy parameters and model details
  *  @param i          position of covering base pair
  *  @param pt         the pair table of the secondary structure
  *  @returns          free energy of the loop in 10cal/mol
  */
 int
-vrna_eval_loop_pt(vrna_fold_compound_t  *vc,
+vrna_eval_loop_pt(vrna_fold_compound_t  *fc,
                   int                   i,
                   const short           *pt);
 
@@ -893,14 +893,14 @@ vrna_eval_loop_pt(vrna_fold_compound_t  *vc,
 /**
  * @brief Calculate energy of a loop
  *
- *  @param vc         A vrna_fold_compound_t containing the energy parameters and model details
+ *  @param fc         A vrna_fold_compound_t containing the energy parameters and model details
  *  @param i          position of covering base pair
  *  @param pt         the pair table of the secondary structure
  *  @param verbosity_level  The level of verbosity of this function
  *  @returns          free energy of the loop in 10cal/mol
  */
 int
-vrna_eval_loop_pt_v(vrna_fold_compound_t  *vc,
+vrna_eval_loop_pt_v(vrna_fold_compound_t  *fc,
                     int                   i,
                     const short           *pt,
                     int                   verbosity_level);
@@ -934,14 +934,14 @@ vrna_eval_loop_pt_v(vrna_fold_compound_t  *vc,
  *  of a base pair, otherwise it is insertion (opening).
  *
  *  @see              vrna_eval_move_pt()
- *  @param vc         A vrna_fold_compound_t containing the energy parameters and model details
+ *  @param fc         A vrna_fold_compound_t containing the energy parameters and model details
  *  @param structure  secondary structure in dot-bracket notation
  *  @param m1         first coordinate of base pair
  *  @param m2         second coordinate of base pair
  *  @returns          energy change of the move in kcal/mol (#INF / 100. upon any error)
  */
 float
-vrna_eval_move(vrna_fold_compound_t *vc,
+vrna_eval_move(vrna_fold_compound_t *fc,
                const char           *structure,
                int                  m1,
                int                  m2);
@@ -955,14 +955,14 @@ vrna_eval_move(vrna_fold_compound_t *vc,
  *  of a base pair, otherwise it is insertion (opening).
  *
  *  @see              vrna_eval_move()
- *  @param vc         A vrna_fold_compound_t containing the energy parameters and model details
+ *  @param fc         A vrna_fold_compound_t containing the energy parameters and model details
  *  @param pt         the pair table of the secondary structure
  *  @param m1         first coordinate of base pair
  *  @param m2         second coordinate of base pair
  *  @returns          energy change of the move in 10cal/mol
  */
 int
-vrna_eval_move_pt(vrna_fold_compound_t  *vc,
+vrna_eval_move_pt(vrna_fold_compound_t  *fc,
                   short                 *pt,
                   int                   m1,
                   int                   m2);
@@ -976,7 +976,7 @@ vrna_eval_move_pt_simple(const char *string,
 
 
 int
-vrna_eval_move_shift_pt(vrna_fold_compound_t  *vc,
+vrna_eval_move_shift_pt(vrna_fold_compound_t  *fc,
                         vrna_move_t           *m,
                         short                 *structure);
 
