@@ -323,67 +323,69 @@ vrna_fold_compound_comparative2(const char                **sequences,
 
   fc = init_fc_comparative();
 
-  fc->n_seq     = n_seq;
-  fc->length    = length;
+  if (fc) {
+    fc->n_seq   = n_seq;
+    fc->length  = length;
 
-  /* get a copy of the model details */
-  if (md_p)
-    md = *md_p;
-  else /* this fallback relies on global parameters and thus is not threadsafe */
-    vrna_md_set_default(&md);
+    /* get a copy of the model details */
+    if (md_p)
+      md = *md_p;
+    else /* this fallback relies on global parameters and thus is not threadsafe */
+      vrna_md_set_default(&md);
 
-  /* now for the energy parameters */
-  add_params(fc, &md, options);
+    /* now for the energy parameters */
+    add_params(fc, &md, options);
 
-  sanitize_bp_span(fc, options);
+    sanitize_bp_span(fc, options);
 
-  vrna_msa_add( fc,
-                sequences,
-                names,
-                orientation,
-                start,
-                genome_size,
-                VRNA_SEQUENCE_RNA);
+    vrna_msa_add(fc,
+                 sequences,
+                 names,
+                 orientation,
+                 start,
+                 genome_size,
+                 VRNA_SEQUENCE_RNA);
 
-  fc->sequences = vrna_alloc(sizeof(char *) * (fc->n_seq + 1));
-  for (s = 0; sequences[s]; s++)
-    fc->sequences[s] = strdup(sequences[s]);
+    fc->sequences = vrna_alloc(sizeof(char *) * (fc->n_seq + 1));
+    for (s = 0; sequences[s]; s++)
+      fc->sequences[s] = strdup(sequences[s]);
 
-  if (options & VRNA_OPTION_WINDOW) {
-    set_fold_compound(fc, options, aux_options);
+    if (options & VRNA_OPTION_WINDOW) {
+      set_fold_compound(fc, options, aux_options);
 
-    fc->pscore_local = vrna_alloc(sizeof(int *) * (fc->length + 1));
+      fc->pscore_local = vrna_alloc(sizeof(int *) * (fc->length + 1));
 
 #if 0
-    for (i = (int)fc->length; (i > (int)fc->length - fc->window_size - 5) && (i >= 0); i--)
-      fc->pscore_local[i] = vrna_alloc(sizeof(int) * (fc->window_size + 5));
+      for (i = (int)fc->length; (i > (int)fc->length - fc->window_size - 5) && (i >= 0); i--)
+        fc->pscore_local[i] = vrna_alloc(sizeof(int) * (fc->window_size + 5));
 #endif
 
-    if (!(options & VRNA_OPTION_EVAL_ONLY)) {
-      /* add minimal hard constraint data structure */
-      vrna_hc_init_window(fc);
+      if (!(options & VRNA_OPTION_EVAL_ONLY)) {
+        /* add minimal hard constraint data structure */
+        vrna_hc_init_window(fc);
 
-      /* add DP matrices */
-      vrna_mx_add(fc, VRNA_MX_WINDOW, options);
-    }
-  } else {
-    /* regular global structure prediction */
+        /* add DP matrices */
+        vrna_mx_add(fc, VRNA_MX_WINDOW, options);
+      }
+    } else {
+      /* regular global structure prediction */
 
-    aux_options |= WITH_PTYPE;
+      aux_options |= WITH_PTYPE;
 
-    if (options & VRNA_OPTION_PF)
-      aux_options |= WITH_PTYPE_COMPAT;
+      if (options & VRNA_OPTION_PF)
+        aux_options |= WITH_PTYPE_COMPAT;
 
-    set_fold_compound(fc, options, aux_options);
+      set_fold_compound(fc, options, aux_options);
 
-    make_pscores(fc);
+      make_pscores(fc);
 
-    if (!(options & VRNA_OPTION_EVAL_ONLY)) {
-      /* add default hard constraints */
-      vrna_hc_init(fc);
+      if (!(options & VRNA_OPTION_EVAL_ONLY)) {
+        /* add default hard constraints */
+        vrna_hc_init(fc);
 
-      /* add DP matrices (if required) */
-      vrna_mx_add(fc, VRNA_MX_DEFAULT, options);
+        /* add DP matrices (if required) */
+        vrna_mx_add(fc, VRNA_MX_DEFAULT, options);
+      }
     }
   }
 
@@ -433,47 +435,49 @@ vrna_fold_compound_TwoD(const char    *sequence,
     return NULL;
   }
 
-  fc            = init_fc_single();
-  fc->length    = length;
-  fc->sequence  = strdup(sequence);
+  fc = init_fc_single();
+  if (fc) {
+    fc->length    = length;
+    fc->sequence  = strdup(sequence);
 
-  /* get a copy of the model details */
-  if (md_p)
-    md = *md_p;
-  else /* this fallback relies on global parameters and thus is not threadsafe */
-    vrna_md_set_default(&md);
+    /* get a copy of the model details */
+    if (md_p)
+      md = *md_p;
+    else /* this fallback relies on global parameters and thus is not threadsafe */
+      vrna_md_set_default(&md);
 
-  /* always make uniq ML decomposition ! */
-  md.uniq_ML      = 1;
-  md.compute_bpp  = 0;
+    /* always make uniq ML decomposition ! */
+    md.uniq_ML      = 1;
+    md.compute_bpp  = 0;
 
-  /* now for the energy parameters */
-  add_params(fc, &md, options);
+    /* now for the energy parameters */
+    add_params(fc, &md, options);
 
-  set_fold_compound(fc, options, WITH_PTYPE | WITH_PTYPE_COMPAT);
+    set_fold_compound(fc, options, WITH_PTYPE | WITH_PTYPE_COMPAT);
 
-  if (!(options & VRNA_OPTION_EVAL_ONLY)) {
-    vrna_hc_init(fc); /* add default hard constraints */
+    if (!(options & VRNA_OPTION_EVAL_ONLY)) {
+      vrna_hc_init(fc); /* add default hard constraints */
 
-    /* add DP matrices */
-    vrna_mx_add(fc, VRNA_MX_2DFOLD, options);
+      /* add DP matrices */
+      vrna_mx_add(fc, VRNA_MX_2DFOLD, options);
+    }
+
+    /* set all fields that are unique to Distance class partitioning... */
+    turn              = fc->params->model_details.min_loop_size;
+    fc->reference_pt1 = vrna_ptable(s1);
+    fc->reference_pt2 = vrna_ptable(s2);
+    fc->referenceBPs1 = vrna_refBPcnt_matrix(fc->reference_pt1, turn);
+    fc->referenceBPs2 = vrna_refBPcnt_matrix(fc->reference_pt2, turn);
+    fc->bpdist        = vrna_refBPdist_matrix(fc->reference_pt1, fc->reference_pt2, turn);
+    /* compute maximum matching with reference structure 1 disallowed */
+    fc->mm1 = maximumMatchingConstraint(fc->sequence, fc->reference_pt1);
+    /* compute maximum matching with reference structure 2 disallowed */
+    fc->mm2 = maximumMatchingConstraint(fc->sequence, fc->reference_pt2);
+
+    fc->maxD1 = fc->mm1[fc->iindx[1] - length] + fc->referenceBPs1[fc->iindx[1] - length];
+    fc->maxD2 = fc->mm2[fc->iindx[1] - length] + fc->referenceBPs2[fc->iindx[1] - length];
   }
 
-  /* set all fields that are unique to Distance class partitioning... */
-  turn              = fc->params->model_details.min_loop_size;
-  fc->reference_pt1 = vrna_ptable(s1);
-  fc->reference_pt2 = vrna_ptable(s2);
-  fc->referenceBPs1 = vrna_refBPcnt_matrix(fc->reference_pt1, turn);
-  fc->referenceBPs2 = vrna_refBPcnt_matrix(fc->reference_pt2, turn);
-  fc->bpdist        = vrna_refBPdist_matrix(fc->reference_pt1, fc->reference_pt2, turn);
-  /* compute maximum matching with reference structure 1 disallowed */
-  fc->mm1 = maximumMatchingConstraint(fc->sequence, fc->reference_pt1);
-  /* compute maximum matching with reference structure 2 disallowed */
-  fc->mm2 = maximumMatchingConstraint(fc->sequence, fc->reference_pt2);
-
-  fc->maxD1 = fc->mm1[fc->iindx[1] - length] + fc->referenceBPs1[fc->iindx[1] - length];
-  fc->maxD2 = fc->mm2[fc->iindx[1] - length] + fc->referenceBPs2[fc->iindx[1] - length];
-
   return fc;
 }
 
@@ -516,6 +520,9 @@ vrna_fold_compound_prepare(vrna_fold_compound_t *fc,
     return 0;
   }
 
+  /* make sure to always provide sane bp-span settings */
+  sanitize_bp_span(fc, options);
+
   /* prepare Boltzmann factors if required */
   vrna_params_prepare(fc, options);
 
@@ -527,7 +534,7 @@ vrna_fold_compound_prepare(vrna_fold_compound_t *fc,
 
     switch (fc->type) {
       case VRNA_FC_TYPE_SINGLE:     /* get pre-computed Boltzmann factors if not present*/
-        if (fc->domains_up)                            /* turn on unique ML decomposition with qm1 array */
+        if (fc->domains_up)         /* turn on unique ML decomposition with qm1 array */
           fc->exp_params->model_details.uniq_ML = 1;
 
         break;
@@ -612,13 +619,10 @@ set_fold_compound(vrna_fold_compound_t  *fc,
 {
   char          *sequence, **sequences, **ptr;
   unsigned int  length, s;
-  int           cp;
-  char          *seq, *seq2;
   vrna_md_t     *md_p;
 
   sequence  = NULL;
   sequences = NULL;
-  cp        = -1;
 
   md_p = &(fc->params->model_details);
 
@@ -641,18 +645,33 @@ set_fold_compound(vrna_fold_compound_t  *fc,
       free(sequences);
       free(sequence);
 
+#ifndef VRNA_DISABLE_BACKWARD_COMPATIBILITY
       if (fc->strands > 1) {
         fc->cutpoint = fc->nucleotides[0].length + 1;
 
+#if 0
         if (md_p->min_loop_size == TURN)
           md_p->min_loop_size = 0;                              /* is it safe to set this here? */
+#endif
       }
 
+#endif
+
       if (!(options & VRNA_OPTION_EVAL_ONLY)) {
-        fc->ptype = (aux & WITH_PTYPE) ? vrna_ptypes(fc->sequence_encoding2, md_p) : NULL;
+        /* temporary hack for multi-strand case */
+        if (fc->strands > 1) {
+          int min_loop_size = md_p->min_loop_size;
+          md_p->min_loop_size = 0;
+          fc->ptype = (aux & WITH_PTYPE) ? vrna_ptypes(fc->sequence_encoding2, md_p) : NULL;
+          md_p->min_loop_size = min_loop_size;
+        } else {
+          fc->ptype = (aux & WITH_PTYPE) ? vrna_ptypes(fc->sequence_encoding2, md_p) : NULL;
+        }
+#ifndef VRNA_DISABLE_BACKWARD_COMPATIBILITY
         /* backward compatibility ptypes */
         fc->ptype_pf_compat =
           (aux & WITH_PTYPE_COMPAT) ? get_ptypes(fc->sequence_encoding2, md_p, 1) : NULL;
+#endif
       }
 
       break;
@@ -721,15 +740,6 @@ make_pscores(vrna_fold_compound_t *fc)
 #define NONE -10000 /* score for forbidden pairs */
 
   int       i, j, k, l, s, max_span, turn;
-  float     **dm;
-  int       olddm[7][7] = { { 0, 0, 0, 0, 0, 0, 0 }, /* hamming distance between pairs */
-                            { 0, 0, 2, 2, 1, 2, 2 } /* CG */,
-                            { 0, 2, 0, 1, 2, 2, 2 } /* GC */,
-                            { 0, 2, 1, 0, 2, 1, 2 } /* GU */,
-                            { 0, 1, 2, 2, 0, 2, 1 } /* UG */,
-                            { 0, 2, 2, 1, 2, 0, 2 } /* AU */,
-                            { 0, 2, 2, 2, 1, 2, 0 } /* UA */ };
-
   short     **S   = fc->S;
   char      **AS  = fc->sequences;
   int       n_seq = fc->n_seq;
@@ -743,17 +753,21 @@ make_pscores(vrna_fold_compound_t *fc)
   turn = md->min_loop_size;
 
   if (md->ribo) {
+    float **dm = NULL;
+
     if (RibosumFile != NULL)
       dm = readribosum(RibosumFile);
     else
       dm = get_ribosum((const char **)AS, n_seq, n);
-  } else {
-    /*use usual matrix*/
-    dm = vrna_alloc(7 * sizeof(float *));
-    for (i = 0; i < 7; i++) {
-      dm[i] = vrna_alloc(7 * sizeof(float));
-      for (j = 0; j < 7; j++)
-        dm[i][j] = (float)olddm[i][j];
+
+    if (dm) {
+      for (i = 0; i < 7; i++) {
+        for (j = 0; j < 7; j++)
+          md->pair_dist[i][j] = dm[i][j];
+
+        free(dm[i]);
+      }
+      free(dm);
     }
   }
 
@@ -765,10 +779,15 @@ make_pscores(vrna_fold_compound_t *fc)
     for (j = i + 1; (j < i + turn + 1) && (j <= n); j++)
       pscore[indx[j] + i] = NONE;
     for (j = i + turn + 1; j <= n; j++) {
-      int     pfreq[8] = {
+      if ((j - i + 1) > max_span) {
+        pscore[indx[j] + i] = NONE;
+        continue;
+      }
+
+      unsigned int pfreq[8] = {
         0, 0, 0, 0, 0, 0, 0, 0
       };
-      double  score;
+
       for (s = 0; s < n_seq; s++) {
         int type;
         if (S[s][i] == 0 && S[s][j] == 0) {
@@ -785,21 +804,8 @@ make_pscores(vrna_fold_compound_t *fc)
 
         pfreq[type]++;
       }
-      if (pfreq[0] * 2 + pfreq[7] > n_seq) {
-        pscore[indx[j] + i] = NONE;
-        continue;
-      }
 
-      for (k = 1, score = 0; k <= 6; k++) /* ignore pairtype 7 (gap-gap) */
-        for (l = k; l <= 6; l++)
-          score += pfreq[k] * pfreq[l] * dm[k][l];
-      /* counter examples score -1, gap-gap scores -0.25   */
-      pscore[indx[j] + i] = md->cv_fact *
-                            ((UNIT * score) / n_seq - md->nc_fact * UNIT *
-                             (pfreq[0] + pfreq[7] * 0.25));
-
-      if ((j - i + 1) > max_span)
-        pscore[indx[j] + i] = NONE;
+      pscore[indx[j] + i] = vrna_pscore_freq(fc, &pfreq[0], 6);
     }
   }
 
@@ -815,8 +821,9 @@ make_pscores(vrna_fold_compound_t *fc)
           if ((i > 1) && (j < n))
             ntype = pscore[indx[j + 1] + i - 1];
 
-          if ((otype < md->cv_fact * MINPSCORE) && (ntype < md->cv_fact * MINPSCORE)) /* too many counterexamples */
-            pscore[indx[j] + i] = NONE;                                               /* i.j can only form isolated pairs */
+          if ((otype < md->cv_fact * MINPSCORE) &&
+              (ntype < md->cv_fact * MINPSCORE))  /* too many counterexamples */
+            pscore[indx[j] + i] = NONE;           /* i.j can only form isolated pairs */
 
           otype = type;
           type  = ntype;
@@ -826,11 +833,6 @@ make_pscores(vrna_fold_compound_t *fc)
       }
   }
 
-  /*free dm */
-  for (i = 0; i < 7; i++)
-    free(dm[i]);
-  free(dm);
-
   /* copy over pscores for backward compatibility */
   if (fc->pscore_pf_compat) {
     for (i = 1; i < n; i++)
@@ -849,7 +851,7 @@ init_fc_single(void)
   vrna_fold_compound_t  *fc = vrna_alloc(sizeof(vrna_fold_compound_t));
 
   if (fc) {
-    memcpy(fc, &init, sizeof(vrna_fold_compound));
+    memcpy(fc, &init, sizeof(vrna_fold_compound_t));
     nullify(fc);
   }
 
@@ -866,7 +868,7 @@ init_fc_comparative(void)
   vrna_fold_compound_t  *fc = vrna_alloc(sizeof(vrna_fold_compound_t));
 
   if (fc) {
-    memcpy(fc, &init, sizeof(vrna_fold_compound));
+    memcpy(fc, &init, sizeof(vrna_fold_compound_t));
     nullify(fc);
   }
 
@@ -878,15 +880,16 @@ PRIVATE INLINE void
 nullify(vrna_fold_compound_t *fc)
 {
   if (fc) {
-    fc->length        = 0;
-    fc->strands       = 0;
-    fc->cutpoint      = -1;
-    fc->strand_number = NULL;
-    fc->strand_order  = NULL;
-    fc->strand_start  = NULL;
-    fc->strand_end    = NULL;
-    fc->nucleotides   = NULL;
-    fc->alignment     = NULL;
+    fc->length            = 0;
+    fc->strands           = 0;
+    fc->cutpoint          = -1;
+    fc->strand_number     = NULL;
+    fc->strand_order      = NULL;
+    fc->strand_order_uniq = NULL;
+    fc->strand_start      = NULL;
+    fc->strand_end        = NULL;
+    fc->nucleotides       = NULL;
+    fc->alignment         = NULL;
 
     fc->hc            = NULL;
     fc->matrices      = NULL;
@@ -908,6 +911,8 @@ nullify(vrna_fold_compound_t *fc)
       case VRNA_FC_TYPE_SINGLE:
         fc->sequence            = NULL;
         fc->sequence_encoding   = NULL;
+        fc->encoding5           = NULL;
+        fc->encoding3           = NULL;
         fc->sequence_encoding2  = NULL;
         fc->ptype               = NULL;
         fc->ptype_pf_compat     = NULL;
