@@ -176,13 +176,13 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
   short                     mm5, mm3, *S1;
   unsigned int              *sn, type;
   int                       length, fij, fi, jj, u, en, e, *my_f5, *my_c, *my_ggg, *idx,
-                            dangle_model, turn, with_gquad, cnt, ii, with_ud;
+                            dangle_model, with_gquad, cnt, ii, with_ud;
   vrna_param_t              *P;
   vrna_md_t                 *md;
   vrna_sc_t                 *sc;
   vrna_ud_t                 *domains_up;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
+  struct hc_ext_def_dat     hc_dat_local;
 
   length        = fc->length;
   P             = fc->params;
@@ -197,10 +197,9 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
   ptype         = fc->ptype;
   S1            = fc->sequence_encoding;
   dangle_model  = md->dangles;
-  turn          = md->min_loop_size;
   with_gquad    = md->gquad;
   with_ud       = (domains_up && domains_up->energy_cb) ? 1 : 0;
-  evaluate      = prepare_hc_default(fc, &hc_dat_local);
+  evaluate      = prepare_hc_ext_def(fc, &hc_dat_local);
 
   jj = *k;
 
@@ -295,7 +294,7 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
     jj++;
   }
 
-  if (jj < turn + 2) {
+  if (jj < 2) {
     /* no more pairs */
     *i  = *j = -1;
     *k  = 0;
@@ -305,7 +304,7 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
   /* must have found a decomposition */
   switch (dangle_model) {
     case 0:   /* j is paired. Find pairing partner */
-      for (u = jj - turn - 1; u >= 1; u--) {
+      for (u = jj - 1; u >= 1; u--) {
         if (with_gquad) {
           if (fij == my_f5[u - 1] + my_ggg[idx[jj] + u]) {
             *i  = *j = -1;
@@ -322,8 +321,10 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
             if (sc->f)
               en += sc->f(1, jj, u - 1, u, VRNA_DECOMP_EXT_EXT_STEM, sc->data);
 
+#if 0
           if (sn[jj] != sn[u])
             en += P->DuplexInit;
+#endif
 
           if (fij == vrna_E_ext_stem(type, -1, -1, P) + en + my_f5[u - 1]) {
             *i                            = u;
@@ -339,7 +340,7 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
 
     case 2:
       mm3 = ((jj < length) && (sn[jj + 1] == sn[jj])) ? S1[jj + 1] : -1;
-      for (u = jj - turn - 1; u >= 1; u--) {
+      for (u = jj - 1; u >= 1; u--) {
         if (with_gquad) {
           if (fij == my_f5[u - 1] + my_ggg[idx[jj] + u]) {
             *i  = *j = -1;
@@ -357,8 +358,10 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
             if (sc->f)
               en += sc->f(1, jj, u - 1, u, VRNA_DECOMP_EXT_EXT_STEM, sc->data);
 
+#if 0
           if (sn[jj] != sn[u])
             en += P->DuplexInit;
+#endif
 
           if (fij == vrna_E_ext_stem(type, mm5, mm3, P) + en + my_f5[u - 1]) {
             *i                            = u;
@@ -389,8 +392,10 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
           if (sc->f)
             en += sc->f(1, jj, 1, jj, VRNA_DECOMP_EXT_STEM, sc->data);
 
+#if 0
         if (sn[jj] != sn[1])
           en += P->DuplexInit;
+#endif
 
         if (fij == en + vrna_E_ext_stem(type, -1, -1, P)) {
           *i                            = 1;
@@ -416,8 +421,10 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
               en += sc->f(1, jj, 1, jj - 1, VRNA_DECOMP_EXT_STEM, sc->data);
           }
 
+#if 0
           if (sn[jj - 1] != sn[1])
             en += P->DuplexInit;
+#endif
 
           if (fij == en + vrna_E_ext_stem(type, -1, mm3, P)) {
             *i                            = 1;
@@ -430,7 +437,7 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
         }
       }
 
-      for (u = jj - turn - 1; u > 1; u--) {
+      for (u = jj - 1; u > 1; u--) {
         if (with_gquad) {
           if (fij == my_f5[u - 1] + my_ggg[idx[jj] + u]) {
             *i  = *j = -1;
@@ -442,8 +449,10 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
         type = vrna_get_ptype(idx[jj] + u, ptype);
 
         en = my_c[idx[jj] + u];
+#if 0
         if (sn[jj] != sn[u])
           en += P->DuplexInit;
+#endif
 
         if (evaluate(1, jj, u - 1, u, VRNA_DECOMP_EXT_EXT_STEM, &hc_dat_local)) {
           e = my_f5[u - 1] +
@@ -493,8 +502,11 @@ BT_ext_loop_f5(vrna_fold_compound_t *fc,
         type = vrna_get_ptype(idx[jj - 1] + u, ptype);
 
         en = my_c[idx[jj - 1] + u];
+
+#if 0
         if (sn[jj - 1] != sn[u])
           en += P->DuplexInit;
+#endif
 
         mm5 = (sn[u] == sn[u - 1]) ? S1[u - 1] : -1;
         mm3 = (sn[jj] == sn[jj - 1]) ? S1[jj] : -1;
@@ -562,12 +574,12 @@ BT_ext_loop_f5_comparative(vrna_fold_compound_t *fc,
   short                     **S, **S5, **S3;
   unsigned int              tt;
   int                       fij, fi, jj, u, en, *my_f5, *my_c, *my_ggg, *idx,
-                            dangle_model, turn, with_gquad, n_seq, ss, mm5, mm3;
+                            dangle_model, with_gquad, n_seq, ss, mm5, mm3;
   vrna_param_t              *P;
   vrna_md_t                 *md;
   vrna_sc_t                 **scs;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
+  struct hc_ext_def_dat     hc_dat_local;
 
   n_seq         = fc->n_seq;
   n             = fc->length;
@@ -583,9 +595,8 @@ BT_ext_loop_f5_comparative(vrna_fold_compound_t *fc,
   my_ggg        = fc->matrices->ggg;
   idx           = fc->jindx;
   dangle_model  = md->dangles;
-  turn          = md->min_loop_size;
   with_gquad    = md->gquad;
-  evaluate      = prepare_hc_default(fc, &hc_dat_local);
+  evaluate      = prepare_hc_ext_def(fc, &hc_dat_local);
 
   jj = *k;
 
@@ -614,7 +625,7 @@ BT_ext_loop_f5_comparative(vrna_fold_compound_t *fc,
   } while (fij == fi);
   jj++;
 
-  if (jj < turn + 2) {
+  if (jj < 2) {
     /* no more pairs */
     *i  = *j = -1;
     *k  = 0;
@@ -624,7 +635,7 @@ BT_ext_loop_f5_comparative(vrna_fold_compound_t *fc,
   /* must have found a decomposition */
   switch (dangle_model) {
     case 0:   /* j is paired. Find pairing partner */
-      for (u = jj - turn - 1; u >= 1; u--) {
+      for (u = jj - 1; u >= 1; u--) {
         if (with_gquad) {
           if (fij == my_f5[u - 1] + my_ggg[idx[jj] + u]) {
             *i  = *j = -1;
@@ -661,7 +672,7 @@ BT_ext_loop_f5_comparative(vrna_fold_compound_t *fc,
       break;
 
     case 2:
-      for (u = jj - turn - 1; u >= 1; u--) {
+      for (u = jj - 1; u >= 1; u--) {
         if (with_gquad) {
           if (fij == my_f5[u - 1] + my_ggg[idx[jj] + u]) {
             *i  = *j = -1;
@@ -717,12 +728,12 @@ BT_ext_loop_f3(vrna_fold_compound_t *fc,
   short                     mm5, mm3, *S1;
   unsigned int              type;
   int                       length, fij, fj, ii, u, *f3, **c, **ggg,
-                            dangle_model, turn, with_gquad, en;
+                            dangle_model, with_gquad, en;
   vrna_param_t              *P;
   vrna_md_t                 *md;
   vrna_sc_t                 *sc;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
+  struct hc_ext_def_dat     hc_dat_local;
 
   length        = fc->length;
   P             = fc->params;
@@ -734,9 +745,8 @@ BT_ext_loop_f3(vrna_fold_compound_t *fc,
   ptype         = fc->ptype_local;
   S1            = fc->sequence_encoding;
   dangle_model  = md->dangles;
-  turn          = md->min_loop_size;
   with_gquad    = md->gquad;
-  evaluate      = prepare_hc_default_window(fc, &hc_dat_local);
+  evaluate      = prepare_hc_ext_def_window(fc, &hc_dat_local);
 
   ii = *k;
 
@@ -761,7 +771,7 @@ BT_ext_loop_f3(vrna_fold_compound_t *fc,
   } while (fij == fj);
   ii--;
 
-  if (ii > maxdist - turn + 1) {
+  if (ii >= maxdist) {
     /* no more pairs */
     *i  = *j = -1;
     *k  = 0;
@@ -775,7 +785,7 @@ BT_ext_loop_f3(vrna_fold_compound_t *fc,
   switch (dangle_model) {
     /* no dangles */
     case 0:
-      for (u = maxdist; u > ii + turn; u--) {
+      for (u = maxdist; u > ii; u--) {
         if (with_gquad) {
           if (fij == ggg[ii][u - ii] + f3[u + 1]) {
             *i  = *j = -1;
@@ -809,7 +819,7 @@ BT_ext_loop_f3(vrna_fold_compound_t *fc,
     /* dangles on both sides */
     case 2:
       mm5 = (ii > 1) ? S1[ii - 1] : -1;
-      for (u = maxdist; u > ii + turn; u--) {
+      for (u = maxdist; u > ii; u--) {
         if (with_gquad) {
           if (fij == ggg[ii][u - ii] + f3[u + 1]) {
             *i  = *j = -1;
@@ -843,7 +853,7 @@ BT_ext_loop_f3(vrna_fold_compound_t *fc,
 
     default:
       mm5 = S1[ii];
-      for (u = maxdist; u > ii + turn; u--) {
+      for (u = maxdist; u > ii; u--) {
         if (with_gquad) {
           if (fij == ggg[ii][u - ii] + f3[u + 1]) {
             *i  = *j = -1;
@@ -1025,12 +1035,12 @@ BT_ext_loop_f3_comparative(vrna_fold_compound_t *fc,
   short                     **S, **S5, **S3;
   unsigned int              type, ss, n_seq, **a2s;
   int                       n, fij, cc, fj, ii, u, *f3, **c, **ggg,
-                            dangle_model, turn, with_gquad;
+                            dangle_model, with_gquad;
   vrna_param_t              *P;
   vrna_md_t                 *md;
   vrna_sc_t                 **scs;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
+  struct hc_ext_def_dat     hc_dat_local;
 
   n             = fc->length;
   n_seq         = fc->n_seq;
@@ -1045,9 +1055,8 @@ BT_ext_loop_f3_comparative(vrna_fold_compound_t *fc,
   c             = fc->matrices->c_local;
   ggg           = fc->matrices->ggg_local;
   dangle_model  = md->dangles;
-  turn          = md->min_loop_size;
   with_gquad    = md->gquad;
-  evaluate      = prepare_hc_default_window(fc, &hc_dat_local);
+  evaluate      = prepare_hc_ext_def_window(fc, &hc_dat_local);
 
   ii = *k;
 
@@ -1075,7 +1084,7 @@ BT_ext_loop_f3_comparative(vrna_fold_compound_t *fc,
   } while (fij == fj);
   ii--;
 
-  if (ii > maxdist - turn + 1) {
+  if (ii >= maxdist) {
     /* no more pairs */
     *i  = *j = -1;
     *k  = 0;
@@ -1089,7 +1098,7 @@ BT_ext_loop_f3_comparative(vrna_fold_compound_t *fc,
   switch (dangle_model) {
     /* no dangles */
     case 0:
-      for (u = maxdist; u > ii + turn; u--) {
+      for (u = maxdist; u > ii; u--) {
         if (with_gquad) {
           if (fij == ggg[ii][u - ii] + f3[u + 1]) {
             *i  = *j = -1;
@@ -1119,7 +1128,7 @@ BT_ext_loop_f3_comparative(vrna_fold_compound_t *fc,
 
     /* dangles on both sides */
     case 2:
-      for (u = maxdist; u > ii + turn; u--) {
+      for (u = maxdist; u > ii; u--) {
         if (with_gquad) {
           if (fij == ggg[ii][u - ii] + f3[u + 1]) {
             *i  = *j = -1;
@@ -1168,13 +1177,13 @@ BT_ext_loop_f3_pp(vrna_fold_compound_t  *fc,
     char                      **ptype;
     short                     *S1;
     unsigned int              type;
-    int                       traced2, length, turn, dangle_model, with_gquad, maxdist, cc,
+    int                       traced2, length, dangle_model, with_gquad, maxdist, cc,
                               **c, **ggg, *f3, fij, ii;
     vrna_param_t              *P;
     vrna_md_t                 *md;
     vrna_sc_t                 *sc;
     vrna_callback_hc_evaluate *evaluate;
-    struct default_data       hc_dat_local;
+    struct hc_ext_def_dat     hc_dat_local;
 #ifdef VRNA_WITH_SVM
     int                       zsc_pre_filter;
     vrna_zsc_dat_t            zsc_data;
@@ -1189,13 +1198,12 @@ BT_ext_loop_f3_pp(vrna_fold_compound_t  *fc,
     sc            = fc->sc;
     P             = fc->params;
     md            = &(P->model_details);
-    turn          = md->min_loop_size;
     dangle_model  = md->dangles;
     with_gquad    = md->gquad;
     maxdist       = MIN2(fc->window_size, maxj);
     traced2       = 0;
     ii            = start;
-    evaluate      = prepare_hc_default_window(fc, &hc_dat_local);
+    evaluate      = prepare_hc_ext_def_window(fc, &hc_dat_local);
 #ifdef VRNA_WITH_SVM
     zsc_data        = fc->zscore_data;
     zsc_pre_filter  = ((zsc_data) &&
@@ -1224,12 +1232,13 @@ BT_ext_loop_f3_pp(vrna_fold_compound_t  *fc,
     /* get pairing partner j */
     switch (dangle_model) {
       case 0:
-        for (j = start + turn; j <= ii + maxdist; j++) {
+        for (j = start + 1; j <= ii + maxdist; j++) {
           if (evaluate(start, length, j, j + 1, VRNA_DECOMP_EXT_STEM_EXT, &hc_dat_local)) {
 #ifdef VRNA_WITH_SVM
             if ((zsc_pre_filter) &&
                 (zsc_data->current_z[j] > zsc_data->min_z))
               continue;
+
 #endif
             type = vrna_get_ptype_window(start, j, ptype);
 
@@ -1257,12 +1266,13 @@ BT_ext_loop_f3_pp(vrna_fold_compound_t  *fc,
         break;
 
       case 2:
-        for (j = start + turn; j <= ii + maxdist; j++) {
+        for (j = start + 1; j <= ii + maxdist; j++) {
           if (evaluate(start, length, j, j + 1, VRNA_DECOMP_EXT_STEM_EXT, &hc_dat_local)) {
 #ifdef VRNA_WITH_SVM
             if ((zsc_pre_filter) &&
                 (zsc_data->current_z[j] > zsc_data->min_z))
               continue;
+
 #endif
             type = vrna_get_ptype_window(start, j, ptype);
 
@@ -1293,7 +1303,7 @@ BT_ext_loop_f3_pp(vrna_fold_compound_t  *fc,
         break;
 
       default:
-        for (j = start + turn; j <= ii + maxdist; j++) {
+        for (j = start + 1; j <= ii + maxdist; j++) {
           if (evaluate(start, length, j, j + 1, VRNA_DECOMP_EXT_STEM_EXT, &hc_dat_local)) {
             type = vrna_get_ptype_window(start, j, ptype);
 
@@ -1411,13 +1421,13 @@ BT_ext_loop_f3_pp_comparative(vrna_fold_compound_t  *fc,
   if (fc) {
     short                     **S, **S5, **S3;
     unsigned int              tt, s, n_seq, **a2s;
-    int                       traced2, length, turn, dangle_model, with_gquad, maxdist, cc, **c,
+    int                       traced2, length, dangle_model, with_gquad, maxdist, cc, **c,
                               **ggg, *f3, fij;
     vrna_param_t              *P;
     vrna_md_t                 *md;
     vrna_sc_t                 **scs;
     vrna_callback_hc_evaluate *evaluate;
-    struct default_data       hc_dat_local;
+    struct hc_ext_def_dat     hc_dat_local;
 
     length        = fc->length;
     n_seq         = fc->n_seq;
@@ -1431,13 +1441,12 @@ BT_ext_loop_f3_pp_comparative(vrna_fold_compound_t  *fc,
     scs           = fc->scs;
     P             = fc->params;
     md            = &(P->model_details);
-    turn          = md->min_loop_size;
     dangle_model  = md->dangles;
     with_gquad    = md->gquad;
     maxdist       = MIN2(fc->window_size, maxj);
     traced2       = 0;
     start         = *i;
-    evaluate      = prepare_hc_default_window(fc, &hc_dat_local);
+    evaluate      = prepare_hc_ext_def_window(fc, &hc_dat_local);
 
     fij = f3[start];
 
@@ -1451,8 +1460,12 @@ BT_ext_loop_f3_pp_comparative(vrna_fold_compound_t  *fc,
             cc += scs[s]->energy_up[start][1];
 
           if (scs[s]->f)
-            cc +=
-              scs[s]->f(start, length, start + 1, length, VRNA_DECOMP_EXT_EXT, scs[s]->data);
+            cc +=scs[s]->f(start,
+                           length,
+                           start + 1,
+                           length,
+                           VRNA_DECOMP_EXT_EXT,
+                           scs[s]->data);
         }
 
       if (fij == cc)
@@ -1463,25 +1476,34 @@ BT_ext_loop_f3_pp_comparative(vrna_fold_compound_t  *fc,
     /* get pairing partner j */
     switch (dangle_model) {
       case 0:
-        for (j = start + turn; j <= MIN2(start + maxdist, length - 1); j++) {
+        for (j = start + 1; j <= MIN2(start + maxdist, length - 1); j++) {
           if (evaluate(start, length, j, j + 1, VRNA_DECOMP_EXT_STEM_EXT, &hc_dat_local)) {
-            cc = c[start][j - start];
+            cc = c[start][j - start] +
+                 f3[j + 1];
 
             for (s = 0; s < n_seq; s++) {
               tt  = vrna_get_ptype_md(S[s][start], S[s][j], md);
               cc  += vrna_E_ext_stem(tt, -1, -1, P);
             }
 
-            if (fij == cc + f3[j + 1]) {
+            if (scs) {
+              for (s = 0; s < n_seq; s++) {
+                if ((scs[s]) && (scs[s]->f))
+                  cc += scs[s]->f(start, length, j, j + 1, VRNA_DECOMP_EXT_STEM_EXT, scs[s]->data);
+              }
+            }
+
+            if (fij == cc) {
               traced2 = 1;
               break;
             }
           }
 
           if (with_gquad) {
-            cc = ggg[start][j - start];
+            cc = ggg[start][j - start] +
+                 f3[j + 1];
 
-            if (fij == cc + f3[j + 1]) {
+            if (fij == cc) {
               traced2 = 1;
               break;
             }
@@ -1498,6 +1520,13 @@ BT_ext_loop_f3_pp_comparative(vrna_fold_compound_t  *fc,
               cc  += vrna_E_ext_stem(tt, -1, -1, P);
             }
 
+            if (scs) {
+              for (s = 0; s < n_seq; s++) {
+                if ((scs[s]) && (scs[s]->f))
+                  cc += scs[s]->f(start, length, start, j, VRNA_DECOMP_EXT_STEM, scs[s]->data);
+              }
+            }
+
             if (fij == cc) {
               traced2 = 1;
               break;
@@ -1517,9 +1546,10 @@ BT_ext_loop_f3_pp_comparative(vrna_fold_compound_t  *fc,
         break;
 
       case 2:
-        for (j = start + turn; j <= MIN2(start + maxdist, length - 1); j++) {
+        for (j = start + 1; j <= MIN2(start + maxdist, length - 1); j++) {
           if (evaluate(start, length, j, j + 1, VRNA_DECOMP_EXT_STEM_EXT, &hc_dat_local)) {
-            cc = c[start][j - start];
+            cc = c[start][j - start] +
+                 f3[j + 1];
 
             for (s = 0; s < n_seq; s++) {
               tt  = vrna_get_ptype_md(S[s][start], S[s][j], md);
@@ -1530,16 +1560,24 @@ BT_ext_loop_f3_pp_comparative(vrna_fold_compound_t  *fc,
                                 P);
             }
 
-            if (fij == cc + f3[j + 1]) {
+            if (scs) {
+              for (s = 0; s < n_seq; s++) {
+                if ((scs[s]) && (scs[s]->f))
+                  cc += scs[s]->f(start, length, j, j + 1, VRNA_DECOMP_EXT_STEM_EXT, scs[s]->data);
+              }
+            }
+
+            if (fij == cc) {
               traced2 = 1;
               break;
             }
           }
 
           if (with_gquad) {
-            cc = ggg[start][j - start];
+            cc = ggg[start][j - start] +
+                 f3[j + 1];
 
-            if (fij == cc + f3[j + 1]) {
+            if (fij == cc) {
               traced2 = 1;
               break;
             }
@@ -1555,6 +1593,13 @@ BT_ext_loop_f3_pp_comparative(vrna_fold_compound_t  *fc,
               cc  += vrna_E_ext_stem(tt, (a2s[s][start] > 1) ? S5[s][start] :  -1, -1, P);
             }
 
+            if (scs) {
+              for (s = 0; s < n_seq; s++) {
+                if ((scs[s]) && (scs[s]->f))
+                  cc += scs[s]->f(start, length, start, j, VRNA_DECOMP_EXT_STEM, scs[s]->data);
+              }
+            }
+
             if (fij == cc) {
               traced2 = 1;
               break;
@@ -1572,6 +1617,8 @@ BT_ext_loop_f3_pp_comparative(vrna_fold_compound_t  *fc,
         }
 
         break;
+
+
     }
 
     if (!traced2)
