@@ -26,7 +26,7 @@
 # define INLINE
 #endif
 
-#define SPEEDUP_HC  1
+#define SPEEDUP_HC  0
 
 #include "external_hc.inc"
 #include "external_sc_pf.inc"
@@ -51,8 +51,8 @@ reduce_ext_ext_fast(vrna_fold_compound_t        *fc,
                     int                         j,
                     struct vrna_mx_pf_aux_el_s  *aux_mx,
                     vrna_callback_hc_evaluate   *evaluate,
-                    struct default_data         *hc_dat_local,
-                    struct sc_wrapper_exp_ext   *sc_wrapper);
+                    struct hc_ext_def_dat       *hc_dat_local,
+                    struct sc_ext_exp_dat       *sc_wrapper);
 
 
 PRIVATE INLINE FLT_OR_DBL
@@ -61,8 +61,8 @@ reduce_ext_stem_fast(vrna_fold_compound_t       *fc,
                      int                        j,
                      struct vrna_mx_pf_aux_el_s *aux_mx,
                      vrna_callback_hc_evaluate  *evaluate,
-                     struct default_data        *hc_dat_local,
-                     struct sc_wrapper_exp_ext  *sc_wrapper);
+                     struct hc_ext_def_dat      *hc_dat_local,
+                     struct sc_ext_exp_dat      *sc_wrapper);
 
 
 PRIVATE INLINE FLT_OR_DBL
@@ -71,8 +71,8 @@ reduce_ext_up_fast(vrna_fold_compound_t       *fc,
                    int                        j,
                    struct vrna_mx_pf_aux_el_s *aux_mx,
                    vrna_callback_hc_evaluate  *evaluate,
-                   struct default_data        *hc_dat_local,
-                   struct sc_wrapper_exp_ext  *sc_wrapper);
+                   struct hc_ext_def_dat      *hc_dat_local,
+                   struct sc_ext_exp_dat      *sc_wrapper);
 
 
 PRIVATE INLINE FLT_OR_DBL
@@ -81,8 +81,8 @@ split_ext_fast(vrna_fold_compound_t       *fc,
                int                        j,
                struct vrna_mx_pf_aux_el_s *aux_mx,
                vrna_callback_hc_evaluate  *evaluate,
-               struct default_data        *hc_dat_local,
-               struct sc_wrapper_exp_ext  *sc_wrapper);
+               struct hc_ext_def_dat      *hc_dat_local,
+               struct sc_ext_exp_dat      *sc_wrapper);
 
 
 PRIVATE FLT_OR_DBL
@@ -129,8 +129,8 @@ vrna_exp_E_ext_fast_init(vrna_fold_compound_t *fc)
     int                       i, j, max_j, d, n, turn, ij, *iidx, with_ud;
     FLT_OR_DBL                *q, **q_local;
     vrna_callback_hc_evaluate *evaluate;
-    struct default_data       hc_dat_local;
-    struct sc_wrapper_exp_ext sc_wrapper;
+    struct hc_ext_def_dat     hc_dat_local;
+    struct sc_ext_exp_dat     sc_wrapper;
     vrna_ud_t                 *domains_up;
 
     n           = (int)fc->length;
@@ -140,11 +140,11 @@ vrna_exp_E_ext_fast_init(vrna_fold_compound_t *fc)
     with_ud     = (domains_up && domains_up->exp_energy_cb);
 
     if (fc->hc->type == VRNA_HC_WINDOW)
-      evaluate = prepare_hc_default_window(fc, &hc_dat_local);
+      evaluate = prepare_hc_ext_def_window(fc, &hc_dat_local);
     else
-      evaluate = prepare_hc_default(fc, &hc_dat_local);
+      evaluate = prepare_hc_ext_def(fc, &hc_dat_local);
 
-    init_sc_wrapper_ext(fc, &sc_wrapper);
+    init_sc_ext_exp(fc, &sc_wrapper);
 
     /* allocate memory for helper arrays */
     aux_mx =
@@ -289,11 +289,11 @@ vrna_exp_E_ext_fast_update(vrna_fold_compound_t       *fc,
                            int                        j,
                            struct vrna_mx_pf_aux_el_s *aux_mx)
 {
-  int                       k, turn;
+  int                       k;
   FLT_OR_DBL                **q;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
-  struct sc_wrapper_exp_ext sc_wrapper;
+  struct hc_ext_def_dat     hc_dat_local;
+  struct sc_ext_exp_dat     sc_wrapper;
 
   /*
    *  init exterior loop contributions for small segments [i, j]
@@ -301,13 +301,12 @@ vrna_exp_E_ext_fast_update(vrna_fold_compound_t       *fc,
    *  j just before any contributions for [i,j] will be computed
    */
   if ((fc) && (fc->hc->type == VRNA_HC_WINDOW)) {
-    turn      = fc->exp_params->model_details.min_loop_size;
     q         = fc->exp_matrices->q_local;
-    evaluate  = prepare_hc_default_window(fc, &hc_dat_local);
-    init_sc_wrapper_ext(fc, &sc_wrapper);
+    evaluate  = prepare_hc_ext_def_window(fc, &hc_dat_local);
+    init_sc_ext_exp(fc, &sc_wrapper);
 
 
-    for (k = j; k >= MAX2(1, j - turn); k--)
+    for (k = j; k >= MAX2(1, j); k--)
       q[k][j] = reduce_ext_up_fast(fc, k, j, aux_mx, evaluate, &hc_dat_local, &sc_wrapper);
   }
 }
@@ -319,13 +318,13 @@ reduce_ext_ext_fast(vrna_fold_compound_t        *fc,
                     int                         j,
                     struct vrna_mx_pf_aux_el_s  *aux_mx,
                     vrna_callback_hc_evaluate   *evaluate,
-                    struct default_data         *hc_dat_local,
-                    struct sc_wrapper_exp_ext   *sc_wrapper)
+                    struct hc_ext_def_dat       *hc_dat_local,
+                    struct sc_ext_exp_dat       *sc_wrapper)
 {
-  int             u;
-  FLT_OR_DBL      q_temp, q_temp2, q, *qq1, **qqu, *scale;
-  vrna_ud_t       *domains_up;
-  sc_ext_exp_red  *sc_red_ext;
+  int           u;
+  FLT_OR_DBL    q_temp, q_temp2, q, *qq1, **qqu, *scale;
+  vrna_ud_t     *domains_up;
+  sc_ext_exp_cb *sc_red_ext;
 
   domains_up  = fc->domains_up;
   qq1         = aux_mx->qq1;
@@ -377,8 +376,8 @@ reduce_ext_stem_fast(vrna_fold_compound_t       *fc,
                      int                        j,
                      struct vrna_mx_pf_aux_el_s *aux_mx,
                      vrna_callback_hc_evaluate  *evaluate,
-                     struct default_data        *hc_dat_local,
-                     struct sc_wrapper_exp_ext  *sc_wrapper)
+                     struct hc_ext_def_dat      *hc_dat_local,
+                     struct sc_ext_exp_dat      *sc_wrapper)
 {
   short             **S, **S5, **S3, *S1, *S2, s5, s3;
   unsigned int      type, *sn, n, s, n_seq, **a2s;
@@ -386,7 +385,7 @@ reduce_ext_stem_fast(vrna_fold_compound_t       *fc,
   FLT_OR_DBL        qbt, q_temp, qb;
   vrna_exp_param_t  *pf_params;
   vrna_md_t         *md;
-  sc_ext_exp_stem   *sc_red_stem;
+  sc_ext_exp_cb     *sc_red_stem;
 
   sc_red_stem = sc_wrapper->red_stem;
   n           = fc->length;
@@ -446,8 +445,8 @@ reduce_ext_up_fast(vrna_fold_compound_t       *fc,
                    int                        j,
                    struct vrna_mx_pf_aux_el_s *aux_mx,
                    vrna_callback_hc_evaluate  *evaluate,
-                   struct default_data        *hc_dat_local,
-                   struct sc_wrapper_exp_ext  *sc_wrapper)
+                   struct hc_ext_def_dat      *hc_dat_local,
+                   struct sc_ext_exp_dat      *sc_wrapper)
 {
   int               u;
   FLT_OR_DBL        qbt, q_temp, *scale;
@@ -488,8 +487,8 @@ split_ext_fast(vrna_fold_compound_t       *fc,
                int                        j,
                struct vrna_mx_pf_aux_el_s *aux_mx,
                vrna_callback_hc_evaluate  *evaluate,
-               struct default_data        *hc_dat_local,
-               struct sc_wrapper_exp_ext  *sc_wrapper)
+               struct hc_ext_def_dat      *hc_dat_local,
+               struct sc_ext_exp_dat      *sc_wrapper)
 {
   int               *idx, k, ij1;
   FLT_OR_DBL        qbt, *q, *qq, *qqq;
@@ -545,29 +544,30 @@ split_ext_fast(vrna_fold_compound_t       *fc,
    *  increases speed. However, once we check for the split point between
    *  strands in hard constraints, we have to think of something else...
    */
-  if ((evaluate == &hc_default) || (evaluate == &hc_default_window)) {
+  if ((evaluate == &hc_ext_cb_def) || (evaluate == &hc_ext_cb_def_window)) {
     for (k = j; k > i; k--) {
       qbt += q[ij1] *
              qqq[k];
       ij1 -= factor;
     }
   } else {
-    for (k = j; k > i; k--)
-      if (evaluate(i, j, k - 1, k, VRNA_DECOMP_EXT_EXT_EXT, hc_dat_local)) {
+    for (k = j; k > i; k--) {
+      if (evaluate(i, j, k - 1, k, VRNA_DECOMP_EXT_EXT_EXT, hc_dat_local))
         qbt += q[ij1] *
                qqq[k];
-        ij1 -= factor;
-      }
+
+      ij1 -= factor;
+    }
   }
 
 #else
-  for (k = j; k > i; k--)
-    if (evaluate(i, j, k - 1, k, VRNA_DECOMP_EXT_EXT_EXT, hc_dat_local)) {
+  for (k = j; k > i; k--) {
+    if (evaluate(i, j, k - 1, k, VRNA_DECOMP_EXT_EXT_EXT, hc_dat_local))
       qbt += q[ij1] *
              qqq[k];
-      ij1 -= factor;
-    }
 
+    ij1 -= factor;
+  }
 #endif
 
   if (qqq != qq) {
@@ -591,8 +591,8 @@ exp_E_ext_fast(vrna_fold_compound_t       *fc,
   vrna_exp_param_t          *pf_params;
   vrna_ud_t                 *domains_up;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
-  struct sc_wrapper_exp_ext sc_wrapper;
+  struct hc_ext_def_dat     hc_dat_local;
+  struct sc_ext_exp_dat     sc_wrapper;
 
   qq          = aux_mx->qq;
   qqu         = aux_mx->qqu;
@@ -603,11 +603,11 @@ exp_E_ext_fast(vrna_fold_compound_t       *fc,
   with_ud     = (domains_up && domains_up->exp_energy_cb);
 
   if (fc->hc->type == VRNA_HC_WINDOW)
-    evaluate = prepare_hc_default_window(fc, &hc_dat_local);
+    evaluate = prepare_hc_ext_def_window(fc, &hc_dat_local);
   else
-    evaluate = prepare_hc_default(fc, &hc_dat_local);
+    evaluate = prepare_hc_ext_def(fc, &hc_dat_local);
 
-  init_sc_wrapper_ext(fc, &sc_wrapper);
+  init_sc_ext_exp(fc, &sc_wrapper);
 
   qbt1 = 0.;
 
@@ -642,7 +642,7 @@ exp_E_ext_fast(vrna_fold_compound_t       *fc,
   if ((fc->aux_grammar) && (fc->aux_grammar->cb_aux_exp_f))
     qbt1 += fc->aux_grammar->cb_aux_exp_f(fc, i, j, fc->aux_grammar->data);
 
-  free_sc_wrapper_ext(&sc_wrapper);
+  free_sc_ext_exp(&sc_wrapper);
 
   return qbt1;
 }
