@@ -147,20 +147,18 @@ eval_int_loop(vrna_fold_compound_t  *fc,
               int                   k,
               int                   l)
 {
-  unsigned int          *sn, *ss, n_seq, s, **a2s;
-  int                   e, *rtype, type, type2, with_ud;
-  short                 *S, *S2, **SS, **S5, **S3;
-  vrna_param_t          *P;
-  vrna_md_t             *md;
-  vrna_ud_t             *domains_up;
-  struct sc_wrapper_int sc_wrapper;
+  unsigned int      *sn, n_seq, s, **a2s;
+  int               e, type, type2, with_ud;
+  short             *S, *S2, **SS, **S5, **S3;
+  vrna_param_t      *P;
+  vrna_md_t         *md;
+  vrna_ud_t         *domains_up;
+  struct sc_int_dat sc_wrapper;
 
   n_seq       = (fc->type == VRNA_FC_TYPE_SINGLE) ? 1 : fc->n_seq;
   P           = fc->params;
   md          = &(P->model_details);
   sn          = fc->strand_number;
-  ss          = fc->strand_start;
-  rtype       = &(md->rtype[0]);
   S           = (fc->type == VRNA_FC_TYPE_SINGLE) ? fc->sequence_encoding : NULL;
   S2          = (fc->type == VRNA_FC_TYPE_SINGLE) ? fc->sequence_encoding2 : NULL;
   SS          = (fc->type == VRNA_FC_TYPE_SINGLE) ? NULL : fc->S;
@@ -171,7 +169,7 @@ eval_int_loop(vrna_fold_compound_t  *fc,
   with_ud     = ((domains_up) && (domains_up->energy_cb)) ? 1 : 0;
   e           = INF;
 
-  init_sc_wrapper(fc, &sc_wrapper);
+  init_sc_int(fc, &sc_wrapper);
 
   {
     int energy, e5, e3, u1, u2;
@@ -190,6 +188,7 @@ eval_int_loop(vrna_fold_compound_t  *fc,
           /* regular interior loop */
           energy = E_IntLoop(u1, u2, type, type2, S[i + 1], S[j - 1], S[k - 1], S[l + 1], P);
         } else {
+#if 0
           /* interior loop like cofold structure */
           short Si, Sj;
           Si      = (sn[i + 1] == sn[i]) ? S[i + 1] : -1;
@@ -201,6 +200,10 @@ eval_int_loop(vrna_fold_compound_t  *fc,
                                  S[k - 1], S[l + 1],
                                  md->dangles,
                                  P);
+#else
+          free_sc_int(&sc_wrapper);
+          return INF;
+#endif
         }
 
         break;
@@ -249,7 +252,7 @@ eval_int_loop(vrna_fold_compound_t  *fc,
     }
   }
 
-  free_sc_wrapper(&sc_wrapper);
+  free_sc_int(&sc_wrapper);
 
   return e;
 }
@@ -262,13 +265,13 @@ eval_ext_int_loop(vrna_fold_compound_t  *fc,
                   int                   k,
                   int                   l)
 {
-  unsigned int          n, n_seq, s, **a2s;
-  int                   e, type, type2, with_ud;
-  short                 *S, *S2, **SS, **S5, **S3;
-  vrna_param_t          *P;
-  vrna_md_t             *md;
-  vrna_ud_t             *domains_up;
-  struct sc_wrapper_int sc_wrapper;
+  unsigned int      n, n_seq, s, **a2s;
+  int               e, type, type2, with_ud;
+  short             *S, *S2, **SS, **S5, **S3;
+  vrna_param_t      *P;
+  vrna_md_t         *md;
+  vrna_ud_t         *domains_up;
+  struct sc_int_dat sc_wrapper;
 
   n           = fc->length;
   n_seq       = (fc->type == VRNA_FC_TYPE_SINGLE) ? 1 : fc->n_seq;
@@ -284,7 +287,7 @@ eval_ext_int_loop(vrna_fold_compound_t  *fc,
   with_ud     = ((domains_up) && (domains_up->energy_cb)) ? 1 : 0;
   e           = INF;
 
-  init_sc_wrapper(fc, &sc_wrapper);
+  init_sc_int(fc, &sc_wrapper);
 
   {
     int energy, e5, e3, u1, u2, u3;
@@ -351,7 +354,7 @@ eval_ext_int_loop(vrna_fold_compound_t  *fc,
     }
   }
 
-  free_sc_wrapper(&sc_wrapper);
+  free_sc_int(&sc_wrapper);
 
   return e;
 }
@@ -365,25 +368,24 @@ E_internal_loop(vrna_fold_compound_t  *fc,
   unsigned char         sliding_window, hc_decompose, *hc_mx, **hc_mx_local;
   char                  *ptype, **ptype_local;
   short                 *S, **SS, **S5, **S3;
-  unsigned int          *sn, *ss, **a2s, n_seq, s, n;
+  unsigned int          *sn, **a2s, n_seq, s, n;
   int                   e, eee, *idx, ij, *c, *ggg, *rtype, with_ud, with_gquad, noclose,
                         *hc_up, **c_local, **ggg_local;
   vrna_param_t          *P;
   vrna_md_t             *md;
   vrna_ud_t             *domains_up;
-  struct default_data   hc_dat_local;
+  struct hc_int_def_dat hc_dat_local;
   eval_hc               *evaluate;
-  struct sc_wrapper_int sc_wrapper;
+  struct sc_int_dat     sc_wrapper;
 
-  evaluate = prepare_hc_default(fc, &hc_dat_local);
-  init_sc_wrapper(fc, &sc_wrapper);
+  evaluate = prepare_hc_int_def(fc, &hc_dat_local);
+  init_sc_int(fc, &sc_wrapper);
 
   e = INF;
 
   n               = fc->length;
   sliding_window  = (fc->hc->type == VRNA_HC_WINDOW) ? 1 : 0;
   sn              = fc->strand_number;
-  ss              = fc->strand_start;
   n_seq           = (fc->type == VRNA_FC_TYPE_SINGLE) ? 1 : fc->n_seq;
   idx             = fc->jindx;
   ij              = (sliding_window) ? 0 : idx[j] + i;
@@ -413,10 +415,9 @@ E_internal_loop(vrna_fold_compound_t  *fc,
 
   if (hc_decompose & VRNA_CONSTRAINT_CONTEXT_INT_LOOP) {
     unsigned int  type, type2, has_nick, *tt;
-    int           k, l, kl, last_k, first_l, u1, u2, turn, noGUclosure;
+    int           k, l, kl, last_k, first_l, u1, u2, noGUclosure;
 
     has_nick    = sn[i] != sn[j] ? 1 : 0;
-    turn        = md->min_loop_size;
     noGUclosure = md->noGUclosure;
     tt          = NULL;
     type        = 0;
@@ -453,6 +454,7 @@ E_internal_loop(vrna_fold_compound_t  *fc,
                       rtype[vrna_get_ptype(kl, ptype)];
 
               if ((has_nick) && ((sn[i] != sn[i + 1]) || (sn[j - 1] != sn[j]))) {
+#if 0
                 /* interior loop like cofold structure */
                 short Si, Sj;
                 Si  = (sn[i + 1] == sn[i]) ? S[i + 1] : -1;
@@ -464,6 +466,9 @@ E_internal_loop(vrna_fold_compound_t  *fc,
                                     S[i], S[j],
                                     md->dangles,
                                     P);
+#else
+                eee = INF;
+#endif
               } else {
                 eee += E_IntLoop(0, 0, type, type2, S[i + 1], S[j - 1], S[i], S[j], P);
               }
@@ -493,7 +498,7 @@ E_internal_loop(vrna_fold_compound_t  *fc,
       /* handle bulges in 5' side */
       l = j - 1;
       if (l > i + 2) {
-        last_k = l - turn - 1;
+        last_k = l - 1;
 
         if (last_k > i + 1 + MAXLOOP)
           last_k = i + 1 + MAXLOOP;
@@ -526,6 +531,7 @@ E_internal_loop(vrna_fold_compound_t  *fc,
                     continue;
 
                   if ((has_nick) && ((sn[i] != sn[k]) || (sn[j - 1] != sn[j]))) {
+#if 0
                     /* interior loop like cofold structure */
                     short Si, Sj;
                     Si  = (sn[i + 1] == sn[i]) ? S[i + 1] : -1;
@@ -537,6 +543,9 @@ E_internal_loop(vrna_fold_compound_t  *fc,
                                         S[k - 1], S[j],
                                         md->dangles,
                                         P);
+#else
+                    eee = INF;
+#endif
                   } else {
                     eee += E_IntLoop(u1, 0, type, type2, S[i + 1], S[j - 1], S[k - 1], S[l + 1], P);
                   }
@@ -548,7 +557,8 @@ E_internal_loop(vrna_fold_compound_t  *fc,
                     int u1_local = a2s[s][k - 1] - a2s[s][i];
                     type2 = vrna_get_ptype_md(SS[s][l], SS[s][k], md);
                     eee   +=
-                      E_IntLoop(u1_local, 0, tt[s], type2, S3[s][i], S5[s][j], S5[s][k], S3[s][l], P);
+                      E_IntLoop(u1_local, 0, tt[s], type2, S3[s][i], S5[s][j], S5[s][k], S3[s][l],
+                                P);
                   }
 
                   break;
@@ -576,7 +586,7 @@ E_internal_loop(vrna_fold_compound_t  *fc,
       /* handle bulges in 3' side */
       k = i + 1;
       if (k < j - 2) {
-        first_l = k + turn + 1;
+        first_l = k + 1;
         if (first_l < j - 1 - MAXLOOP)
           first_l = j - 1 - MAXLOOP;
 
@@ -605,6 +615,7 @@ E_internal_loop(vrna_fold_compound_t  *fc,
                     continue;
 
                   if ((has_nick) && ((sn[i] != sn[i + 1]) || (sn[j] != sn[l]))) {
+#if 0
                     /* interior loop like cofold structure */
                     short Si, Sj;
                     Si  = (sn[i + 1] == sn[i]) ? S[i + 1] : -1;
@@ -616,6 +627,9 @@ E_internal_loop(vrna_fold_compound_t  *fc,
                                         S[i], S[l + 1],
                                         md->dangles,
                                         P);
+#else
+                    eee = INF;
+#endif
                   } else {
                     eee += E_IntLoop(0, u2, type, type2, S[i + 1], S[j - 1], S[k - 1], S[l + 1], P);
                   }
@@ -627,7 +641,8 @@ E_internal_loop(vrna_fold_compound_t  *fc,
                     int u2_local = a2s[s][j - 1] - a2s[s][l];
                     type2 = vrna_get_ptype_md(SS[s][l], SS[s][k], md);
                     eee   +=
-                      E_IntLoop(0, u2_local, tt[s], type2, S3[s][i], S5[s][j], S5[s][k], S3[s][l], P);
+                      E_IntLoop(0, u2_local, tt[s], type2, S3[s][i], S5[s][j], S5[s][k], S3[s][l],
+                                P);
                   }
 
                   break;
@@ -653,7 +668,7 @@ E_internal_loop(vrna_fold_compound_t  *fc,
       }
 
       /* last but not least, all other internal loops */
-      first_l = i + 2 + turn + 1;
+      first_l = i + 2 + 1;
       if (first_l < j - 1 - MAXLOOP)
         first_l = j - 1 - MAXLOOP;
 
@@ -662,7 +677,7 @@ E_internal_loop(vrna_fold_compound_t  *fc,
         if (u2 > hc_up[l + 1])
           break;
 
-        last_k = l - turn - 1;
+        last_k = l - 1;
 
         if (last_k > i + 1 + MAXLOOP - u2)
           last_k = i + 1 + MAXLOOP - u2;
@@ -694,6 +709,7 @@ E_internal_loop(vrna_fold_compound_t  *fc,
                     continue;
 
                   if ((has_nick) && ((sn[i] != sn[k]) || (sn[j] != sn[l]))) {
+#if 0
                     /* interior loop like cofold structure */
                     short Si, Sj;
                     Si  = (sn[i + 1] == sn[i]) ? S[i + 1] : -1;
@@ -705,8 +721,12 @@ E_internal_loop(vrna_fold_compound_t  *fc,
                                         S[k - 1], S[l + 1],
                                         md->dangles,
                                         P);
+#else
+                    eee = INF;
+#endif
                   } else {
-                    eee += E_IntLoop(u1, u2, type, type2, S[i + 1], S[j - 1], S[k - 1], S[l + 1], P);
+                    eee +=
+                      E_IntLoop(u1, u2, type, type2, S[i + 1], S[j - 1], S[k - 1], S[l + 1], P);
                   }
 
                   break;
@@ -807,7 +827,7 @@ E_internal_loop(vrna_fold_compound_t  *fc,
     }
   }
 
-  free_sc_wrapper(&sc_wrapper);
+  free_sc_int(&sc_wrapper);
 
   return e;
 }
@@ -820,15 +840,15 @@ E_ext_internal_loop(vrna_fold_compound_t  *fc,
                     int                   *ip,
                     int                   *iq)
 {
-  int                 q, p, e, s, u1, u2, qmin, energy,
-                      n, *indx, *hc_up, *c, turn, n_seq;
-  unsigned char       *hc, eval_loop;
-  unsigned int        *tt;
-  short               **SS;
-  vrna_md_t           *md;
-  vrna_param_t        *P;
-  eval_hc             *evaluate;
-  struct default_data hc_dat_local;
+  int                   q, p, e, s, u1, u2, qmin, energy,
+                        n, *indx, *hc_up, *c, n_seq;
+  unsigned char         *hc, eval_loop;
+  unsigned int          *tt;
+  short                 **SS;
+  vrna_md_t             *md;
+  vrna_param_t          *P;
+  eval_hc               *evaluate;
+  struct hc_int_def_dat hc_dat_local;
 
   n     = fc->length;
   n_seq = (fc->type == VRNA_FC_TYPE_SINGLE) ? 1 : fc->n_seq;
@@ -839,12 +859,11 @@ E_ext_internal_loop(vrna_fold_compound_t  *fc,
   hc_up = fc->hc->up_int;
   P     = fc->params;
   md    = &(P->model_details);
-  turn  = md->min_loop_size;
   tt    = NULL;
 
   e = INF;
 
-  evaluate = prepare_hc_default(fc, &hc_dat_local);
+  evaluate = prepare_hc_int_def(fc, &hc_dat_local);
 
   /* CONSTRAINED INTERIOR LOOP start */
   if (hc[n * i + j] & VRNA_CONSTRAINT_CONTEXT_INT_LOOP) {
@@ -865,8 +884,8 @@ E_ext_internal_loop(vrna_fold_compound_t  *fc,
         break;
 
       qmin = u1 + i - 1 + n - MAXLOOP;
-      if (qmin < p + turn + 1)
-        qmin = p + turn + 1;
+      if (qmin < p + 1)
+        qmin = p + 1;
 
       for (q = n; q >= qmin; q--) {
         u2 = i - 1 + n - q;
@@ -912,14 +931,14 @@ E_stack(vrna_fold_compound_t  *fc,
   unsigned char         sliding_window, hc_decompose_ij, hc_decompose_pq,
                         *hc_mx, **hc_mx_local, eval_loop;
   char                  *ptype, **ptype_local;
-  short                 *S, **SS;
-  unsigned int          n, *sn, *ss, type, type_2;
+  short                 **SS;
+  unsigned int          n, *sn, type, type_2;
   int                   e, ij, pq, p, q, s, n_seq, *rtype, *indx;
   vrna_param_t          *P;
   vrna_md_t             *md;
   eval_hc               *evaluate;
-  struct default_data   hc_dat_local;
-  struct sc_wrapper_int sc_wrapper;
+  struct hc_int_def_dat hc_dat_local;
+  struct sc_int_dat     sc_wrapper;
 
   e               = INF;
   sliding_window  = (fc->hc->type == VRNA_HC_WINDOW) ? 1 : 0;
@@ -928,8 +947,6 @@ E_stack(vrna_fold_compound_t  *fc,
   q               = j - 1;
   n_seq           = (fc->type == VRNA_FC_TYPE_SINGLE) ? 1 : fc->n_seq;
   sn              = fc->strand_number;
-  ss              = fc->strand_start;
-  S               = (fc->type == VRNA_FC_TYPE_SINGLE) ? fc->sequence_encoding : NULL;
   SS              = (fc->type == VRNA_FC_TYPE_SINGLE) ? NULL : fc->S;
   ptype           = (fc->type == VRNA_FC_TYPE_SINGLE) ?
                     (sliding_window ? NULL : fc->ptype) :
@@ -945,15 +962,15 @@ E_stack(vrna_fold_compound_t  *fc,
   hc_mx_local = (sliding_window) ? fc->hc->matrix_local : NULL;
   ij          = (sliding_window) ? 0 : indx[j] + i;
   pq          = (sliding_window) ? 0 : indx[q] + p;
-  evaluate    = prepare_hc_default(fc, &hc_dat_local);
+  evaluate    = prepare_hc_int_def(fc, &hc_dat_local);
 
-  init_sc_wrapper(fc, &sc_wrapper);
+  init_sc_int(fc, &sc_wrapper);
 
   hc_decompose_ij = (sliding_window) ? hc_mx_local[i][j - i] : hc_mx[n * i + j];
   hc_decompose_pq = (sliding_window) ? hc_mx_local[p][q - p] : hc_mx[n * p + q];
 
-  eval_loop = (hc_decompose_ij & VRNA_CONSTRAINT_CONTEXT_INT_LOOP_ENC) &&
-              (hc_decompose_pq & VRNA_CONSTRAINT_CONTEXT_INT_LOOP);
+  eval_loop = (hc_decompose_ij & VRNA_CONSTRAINT_CONTEXT_INT_LOOP) &&
+              (hc_decompose_pq & VRNA_CONSTRAINT_CONTEXT_INT_LOOP_ENC);
 
   if (eval_loop && evaluate(i, j, p, q, &hc_dat_local)) {
     switch (fc->type) {
@@ -969,6 +986,7 @@ E_stack(vrna_fold_compound_t  *fc,
           /* regular stack */
           e = P->stack[type][type_2];
         } else {
+#if 0
           /* stack like cofold structure */
           short si, sj;
           si  = (sn[i + 1] == sn[i]) ? S[i + 1] : -1;
@@ -980,6 +998,9 @@ E_stack(vrna_fold_compound_t  *fc,
                              S[p - 1], S[q + 1],
                              md->dangles,
                              P);
+#else
+          e = INF;
+#endif
         }
 
         break;
@@ -1000,7 +1021,7 @@ E_stack(vrna_fold_compound_t  *fc,
       e += sc_wrapper.pair(i, j, p, q, &sc_wrapper);
   }
 
-  free_sc_wrapper(&sc_wrapper);
+  free_sc_int(&sc_wrapper);
 
   return e;
 }
