@@ -105,20 +105,20 @@ exp_E_int_loop(vrna_fold_compound_t *fc,
                int                  i,
                int                  j)
 {
-  unsigned char             sliding_window, hc_decompose_ij, hc_decompose_kl;
-  char                      *ptype, **ptype_local;
-  unsigned char             *hc_mx, **hc_mx_local;
-  short                     *S1, **SS, **S5, **S3;
-  unsigned int              *sn, *se, *ss, n_seq, s, **a2s, n;
-  int                       *rtype, noclose, *my_iindx, *jindx, *hc_up, ij,
-                            with_gquad, with_ud;
-  FLT_OR_DBL                qbt1, q_temp, *qb, **qb_local, *G, *scale;
-  vrna_exp_param_t          *pf_params;
-  vrna_md_t                 *md;
-  vrna_ud_t                 *domains_up;
-  eval_hc                   *evaluate;
-  struct  default_data      hc_dat_local;
-  struct sc_wrapper_exp_int sc_wrapper;
+  unsigned char         sliding_window, hc_decompose_ij, hc_decompose_kl;
+  char                  *ptype, **ptype_local;
+  unsigned char         *hc_mx, **hc_mx_local;
+  short                 *S1, **SS, **S5, **S3;
+  unsigned int          *sn, *se, *ss, n_seq, s, **a2s, n;
+  int                   *rtype, noclose, *my_iindx, *jindx, *hc_up, ij,
+                        with_gquad, with_ud;
+  FLT_OR_DBL            qbt1, q_temp, *qb, **qb_local, *G, *scale;
+  vrna_exp_param_t      *pf_params;
+  vrna_md_t             *md;
+  vrna_ud_t             *domains_up;
+  eval_hc               *evaluate;
+  struct hc_int_def_dat hc_dat_local;
+  struct sc_int_exp_dat sc_wrapper;
 
   sliding_window  = (fc->hc->type == VRNA_HC_WINDOW) ? 1 : 0;
   n               = fc->length;
@@ -150,9 +150,9 @@ exp_E_int_loop(vrna_fold_compound_t *fc,
   with_ud     = ((domains_up) && (domains_up->exp_energy_cb)) ? 1 : 0;
   rtype       = &(md->rtype[0]);
   qbt1        = 0.;
-  evaluate    = prepare_hc_default(fc, &hc_dat_local);
+  evaluate    = prepare_hc_int_def(fc, &hc_dat_local);
 
-  init_sc_wrapper_int(fc, &sc_wrapper);
+  init_sc_int_exp(fc, &sc_wrapper);
 
   ij = (sliding_window) ? 0 : jindx[j] + i;
 
@@ -161,9 +161,8 @@ exp_E_int_loop(vrna_fold_compound_t *fc,
   /* CONSTRAINED INTERIOR LOOP start */
   if (hc_decompose_ij & VRNA_CONSTRAINT_CONTEXT_INT_LOOP) {
     unsigned int  type, type2, *tt;
-    int           k, l, kl, last_k, first_l, u1, u2, turn, noGUclosure;
+    int           k, l, kl, last_k, first_l, u1, u2, noGUclosure;
 
-    turn        = md->min_loop_size;
     noGUclosure = md->noGUclosure;
     tt          = NULL;
     type        = 0;
@@ -240,7 +239,7 @@ exp_E_int_loop(vrna_fold_compound_t *fc,
       /* handle bulges in 5' side */
       l = j - 1;
       if ((l > i + 2) && (sn[j] == sn[l])) {
-        last_k = l - turn - 1;
+        last_k = l - 1;
 
         if (last_k > i + 1 + MAXLOOP)
           last_k = i + 1 + MAXLOOP;
@@ -325,7 +324,7 @@ exp_E_int_loop(vrna_fold_compound_t *fc,
       /* handle bulges in 3' side */
       k = i + 1;
       if ((k < j - 2) && (sn[i] == sn[k])) {
-        first_l = k + turn + 1;
+        first_l = k + 1;
         if (first_l < j - 1 - MAXLOOP)
           first_l = j - 1 - MAXLOOP;
 
@@ -405,7 +404,7 @@ exp_E_int_loop(vrna_fold_compound_t *fc,
       }
 
       /* last but not least, all other internal loops */
-      last_k = j - turn - 3;
+      last_k = j - 3;
 
       if (last_k > i + MAXLOOP + 1)
         last_k = i + MAXLOOP + 1;
@@ -419,7 +418,7 @@ exp_E_int_loop(vrna_fold_compound_t *fc,
       u1 = 1;
 
       for (k = i + 2; k <= last_k; k++, u1++) {
-        first_l = k + turn + 1;
+        first_l = k + 1;
 
         if (first_l < j - 1 - MAXLOOP + u1)
           first_l = j - 1 - MAXLOOP + u1;
@@ -551,7 +550,7 @@ exp_E_int_loop(vrna_fold_compound_t *fc,
     free(tt);
   }
 
-  free_sc_wrapper_int(&sc_wrapper);
+  free_sc_int_exp(&sc_wrapper);
 
   return qbt1;
 }
@@ -562,19 +561,19 @@ exp_E_ext_int_loop(vrna_fold_compound_t *fc,
                    int                  i,
                    int                  j)
 {
-  unsigned char             *hc_mx, eval_loop;
-  short                     *S, *S2, **SS, **S5, **S3;
-  unsigned int              *tt, n_seq, s, **a2s, type, type2;
-  int                       k, l, u1, u2, u3, qmin, with_ud,
-                            n, *my_iindx, *hc_up, turn,
-                            u1_local, u2_local, u3_local;
-  FLT_OR_DBL                q, q_temp, *qb, *scale;
-  vrna_exp_param_t          *pf_params;
-  vrna_md_t                 *md;
-  vrna_ud_t                 *domains_up;
-  eval_hc                   *evaluate;
-  struct default_data       hc_dat_local;
-  struct sc_wrapper_exp_int sc_wrapper;
+  unsigned char         *hc_mx, eval_loop;
+  short                 *S, *S2, **SS, **S5, **S3;
+  unsigned int          *tt, n_seq, s, **a2s, type, type2;
+  int                   k, l, u1, u2, u3, qmin, with_ud,
+                        n, *my_iindx, *hc_up,
+                        u1_local, u2_local, u3_local;
+  FLT_OR_DBL            q, q_temp, *qb, *scale;
+  vrna_exp_param_t      *pf_params;
+  vrna_md_t             *md;
+  vrna_ud_t             *domains_up;
+  eval_hc               *evaluate;
+  struct hc_int_def_dat hc_dat_local;
+  struct sc_int_exp_dat sc_wrapper;
 
   n           = fc->length;
   n_seq       = (fc->type == VRNA_FC_TYPE_SINGLE) ? 1 : fc->n_seq;
@@ -591,7 +590,6 @@ exp_E_ext_int_loop(vrna_fold_compound_t *fc,
   hc_up       = fc->hc->up_int;
   pf_params   = fc->exp_params;
   md          = &(pf_params->model_details);
-  turn        = md->min_loop_size;
   type        = 0;
   tt          = NULL;
   domains_up  = fc->domains_up;
@@ -599,9 +597,9 @@ exp_E_ext_int_loop(vrna_fold_compound_t *fc,
 
   q = 0.;
 
-  evaluate = prepare_hc_default(fc, &hc_dat_local);
+  evaluate = prepare_hc_int_def(fc, &hc_dat_local);
 
-  init_sc_wrapper_int(fc, &sc_wrapper);
+  init_sc_int_exp(fc, &sc_wrapper);
 
   /* CONSTRAINED INTERIOR LOOP start */
   if (hc_mx[n * i + j] & VRNA_CONSTRAINT_CONTEXT_INT_LOOP) {
@@ -624,8 +622,8 @@ exp_E_ext_int_loop(vrna_fold_compound_t *fc,
         break;
 
       qmin = u2 + i - 1 + n - MAXLOOP;
-      if (qmin < k + turn + 1)
-        qmin = k + turn + 1;
+      if (qmin < k + 1)
+        qmin = k + 1;
 
       for (l = n; l >= qmin; l--) {
         u1  = i - 1;
@@ -722,7 +720,7 @@ exp_E_ext_int_loop(vrna_fold_compound_t *fc,
   }
 
   free(tt);
-  free_sc_wrapper_int(&sc_wrapper);
+  free_sc_int_exp(&sc_wrapper);
 
   return q;
 }
@@ -735,19 +733,19 @@ exp_E_interior_loop(vrna_fold_compound_t  *fc,
                     int                   k,
                     int                   l)
 {
-  unsigned char             sliding_window, type, type2;
-  char                      *ptype, **ptype_local;
-  unsigned char             *hc_mx, **hc_mx_local, eval_loop, hc_decompose_ij, hc_decompose_kl;
-  short                     *S1, **SS, **S5, **S3;
-  unsigned int              n, *sn, n_seq, s, **a2s;
-  int                       u1, u2, *rtype, *jindx, *hc_up;
-  FLT_OR_DBL                qbt1, q_temp, *scale;
-  vrna_exp_param_t          *pf_params;
-  vrna_md_t                 *md;
-  vrna_ud_t                 *domains_up;
-  eval_hc                   *evaluate;
-  struct default_data       hc_dat_local;
-  struct sc_wrapper_exp_int sc_wrapper;
+  unsigned char         sliding_window, type, type2;
+  char                  *ptype, **ptype_local;
+  unsigned char         *hc_mx, **hc_mx_local, eval_loop, hc_decompose_ij, hc_decompose_kl;
+  short                 *S1, **SS, **S5, **S3;
+  unsigned int          n, *sn, n_seq, s, **a2s;
+  int                   u1, u2, *rtype, *jindx, *hc_up;
+  FLT_OR_DBL            qbt1, q_temp, *scale;
+  vrna_exp_param_t      *pf_params;
+  vrna_md_t             *md;
+  vrna_ud_t             *domains_up;
+  eval_hc               *evaluate;
+  struct hc_int_def_dat hc_dat_local;
+  struct sc_int_exp_dat sc_wrapper;
 
   sliding_window  = (fc->hc->type == VRNA_HC_WINDOW) ? 1 : 0;
   n               = fc->length;
@@ -783,9 +781,9 @@ exp_E_interior_loop(vrna_fold_compound_t  *fc,
   if (hc_up[i + 1] < u1)
     return qbt1;
 
-  evaluate = prepare_hc_default(fc, &hc_dat_local);
+  evaluate = prepare_hc_int_def(fc, &hc_dat_local);
 
-  init_sc_wrapper_int(fc, &sc_wrapper);
+  init_sc_int_exp(fc, &sc_wrapper);
 
   hc_decompose_ij = (sliding_window) ? hc_mx_local[i][j - i] : hc_mx[n * i + j];
   hc_decompose_kl = (sliding_window) ? hc_mx_local[k][l - k] : hc_mx[n * k + l];
@@ -880,7 +878,7 @@ exp_E_interior_loop(vrna_fold_compound_t  *fc,
     }
   }
 
-  free_sc_wrapper_int(&sc_wrapper);
+  free_sc_int_exp(&sc_wrapper);
 
   return qbt1;
 }
