@@ -63,8 +63,8 @@ extend_fm_3p(int                        i,
              int                        *fm,
              vrna_fold_compound_t       *fc,
              vrna_callback_hc_evaluate  *evaluate,
-             struct default_data        *hc_data,
-             struct sc_wrapper_ml       *sc_wrapper);
+             struct hc_mb_def_dat       *hc_data,
+             struct sc_mb_dat           *sc_wrapper);
 
 
 PRIVATE int
@@ -79,8 +79,8 @@ ml_pair5(vrna_fold_compound_t       *fc,
          int                        j,
          int                        *dmli2,
          vrna_callback_hc_evaluate  *evaluate,
-         struct default_data        *hc_wrapper,
-         struct sc_wrapper_ml       *sc_wrapper);
+         struct hc_mb_def_dat       *hc_wrapper,
+         struct sc_mb_dat           *sc_wrapper);
 
 
 PRIVATE INLINE int
@@ -89,8 +89,8 @@ ml_pair3(vrna_fold_compound_t       *fc,
          int                        j,
          int                        *dmli1,
          vrna_callback_hc_evaluate  *evaluate,
-         struct default_data        *hc_wrapper,
-         struct sc_wrapper_ml       *sc_wrapper);
+         struct hc_mb_def_dat       *hc_wrapper,
+         struct sc_mb_dat           *sc_wrapper);
 
 
 PRIVATE INLINE int
@@ -100,8 +100,8 @@ ml_pair53(vrna_fold_compound_t      *fc,
           int                       *dmli1,
           int                       *dmli2,
           vrna_callback_hc_evaluate *evaluate,
-          struct default_data       *hc_wrapper,
-          struct sc_wrapper_ml      *sc_wrapper);
+          struct hc_mb_def_dat      *hc_wrapper,
+          struct sc_mb_dat          *sc_wrapper);
 
 
 /*
@@ -165,12 +165,12 @@ E_ml_rightmost_stem(int                   i,
   e = INF;
 
   if ((fc) && (fc->matrices) && (fc->matrices->fM1)) {
-    struct default_data       hc_dat_local;
-    struct sc_wrapper_ml      sc_wrapper;
+    struct hc_mb_def_dat      hc_dat_local;
+    struct sc_mb_dat          sc_wrapper;
     vrna_callback_hc_evaluate *evaluate;
 
-    evaluate = prepare_hc_default(fc, &hc_dat_local);
-    init_sc_wrapper(fc, &sc_wrapper);
+    evaluate = prepare_hc_mb_def(fc, &hc_dat_local);
+    init_sc_mb(fc, &sc_wrapper);
 
     e = extend_fm_3p(i, j, fc->matrices->fM1, fc, evaluate, &hc_dat_local, &sc_wrapper);
 
@@ -179,7 +179,7 @@ E_ml_rightmost_stem(int                   i,
       e = MIN2(e, ee);
     }
 
-    free_sc_wrapper(&sc_wrapper);
+    free_sc_mb(&sc_wrapper);
   }
 
   return e;
@@ -197,8 +197,8 @@ ml_pair_d0(vrna_fold_compound_t       *fc,
            int                        j,
            int                        *dmli1,
            vrna_callback_hc_evaluate  *evaluate,
-           struct default_data        *hc_wrapper,
-           struct sc_wrapper_ml       *sc_wrapper)
+           struct hc_mb_def_dat       *hc_wrapper,
+           struct sc_mb_dat           *sc_wrapper)
 {
   short         *S, **SS;
   unsigned int  tt, s, n_seq;
@@ -255,8 +255,8 @@ ml_pair_d1(vrna_fold_compound_t       *fc,
            int                        *dmli1,
            int                        *dmli2,
            vrna_callback_hc_evaluate  *evaluate,
-           struct default_data        *hc_wrapper,
-           struct sc_wrapper_ml       *sc_wrapper)
+           struct hc_mb_def_dat       *hc_wrapper,
+           struct sc_mb_dat           *sc_wrapper)
 {
   int e, en;
 
@@ -285,8 +285,8 @@ ml_pair_d2(vrna_fold_compound_t       *fc,
            int                        j,
            int                        *dmli1,
            vrna_callback_hc_evaluate  *evaluate,
-           struct default_data        *hc_wrapper,
-           struct sc_wrapper_ml       *sc_wrapper)
+           struct hc_mb_def_dat       *hc_wrapper,
+           struct sc_mb_dat           *sc_wrapper)
 {
   short         *S, *S2, **SS, **S5, **S3, si1, sj1;
   unsigned int  tt, strands, *sn, s, n_seq;
@@ -351,8 +351,8 @@ ml_pair5(vrna_fold_compound_t       *fc,
          int                        j,
          int                        *dmli2,
          vrna_callback_hc_evaluate  *evaluate,
-         struct default_data        *hc_wrapper,
-         struct sc_wrapper_ml       *sc_wrapper)
+         struct hc_mb_def_dat       *hc_wrapper,
+         struct sc_mb_dat           *sc_wrapper)
 {
   short         *S, *S2, **SS, **S3, si1;
   unsigned int  tt, strands, *sn, n_seq, s;
@@ -380,7 +380,7 @@ ml_pair5(vrna_fold_compound_t       *fc,
           if (md->noGUclosure && ((tt == 3) || (tt == 4)))
             return INF; /* not allowed */
 
-          si1 = ((strands == 1) || (sn[i] == sn[i + 1])) ? S[i + 1] : -1;
+          si1 = ((strands == 1) || (sn[i] == sn[i + 2])) ? S[i + 1] : -1;
 
           e += E_MLstem(tt, -1, si1, P) +
                P->MLclosing +
@@ -417,8 +417,8 @@ ml_pair3(vrna_fold_compound_t       *fc,
          int                        j,
          int                        *dmli1,
          vrna_callback_hc_evaluate  *evaluate,
-         struct default_data        *hc_wrapper,
-         struct sc_wrapper_ml       *sc_wrapper)
+         struct hc_mb_def_dat       *hc_wrapper,
+         struct sc_mb_dat           *sc_wrapper)
 {
   short         *S, *S2, **SS, **S5, sj1;
   unsigned int  tt, strands, *sn, n_seq, s;
@@ -446,7 +446,7 @@ ml_pair3(vrna_fold_compound_t       *fc,
           if (md->noGUclosure && ((tt == 3) || (tt == 4)))
             return INF; /* not allowed */
 
-          sj1 = ((strands == 1) || (sn[j - 1] == sn[j])) ? S[j - 1] : -1;
+          sj1 = ((strands == 1) || (sn[j - 2] == sn[j])) ? S[j - 1] : -1;
 
           e += E_MLstem(tt, sj1, -1, P) +
                P->MLclosing +
@@ -484,8 +484,8 @@ ml_pair53(vrna_fold_compound_t      *fc,
           int                       *dmli1,
           int                       *dmli2,
           vrna_callback_hc_evaluate *evaluate,
-          struct default_data       *hc_wrapper,
-          struct sc_wrapper_ml      *sc_wrapper)
+          struct hc_mb_def_dat      *hc_wrapper,
+          struct sc_mb_dat          *sc_wrapper)
 {
   short         *S, *S2, **SS, **S3, **S5, si1, sj1;
   unsigned int  tt, strands, *sn, n_seq, s;
@@ -513,8 +513,8 @@ ml_pair53(vrna_fold_compound_t      *fc,
           if (md->noGUclosure && ((tt == 3) || (tt == 4)))
             return INF; /* not allowed */
 
-          si1 = ((strands == 1) || (sn[i] == sn[i + 1])) ? S[i + 1] : -1;
-          sj1 = ((strands == 1) || (sn[j - 1] == sn[j])) ? S[j - 1] : -1;
+          si1 = ((strands == 1) || (sn[i] == sn[i + 2])) ? S[i + 1] : -1;
+          sj1 = ((strands == 1) || (sn[j - 2] == sn[j])) ? S[j - 1] : -1;
 
           e += E_MLstem(tt, sj1, si1, P) +
                P->MLclosing +
@@ -546,6 +546,7 @@ ml_pair53(vrna_fold_compound_t      *fc,
 }
 
 
+#if 0
 PRIVATE int
 E_mb_loop_fake(vrna_fold_compound_t *fc,
                int                  i,
@@ -557,7 +558,7 @@ E_mb_loop_fake(vrna_fold_compound_t *fc,
   vrna_param_t              *P;
   vrna_md_t                 *md;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
+  struct hc_mb_def_dat      hc_dat_local;
 
   S             = fc->sequence_encoding;
   S2            = fc->sequence_encoding2;
@@ -573,7 +574,7 @@ E_mb_loop_fake(vrna_fold_compound_t *fc,
   e       = INF;
   decomp  = INF;
 
-  evaluate = prepare_hc_default_ext(fc, &hc_dat_local);
+  evaluate = prepare_hc_mb_def_ext(fc, &hc_dat_local);
 
   tt = vrna_get_ptype_md(S2[j], S2[i], md);
 
@@ -643,6 +644,7 @@ E_mb_loop_fake(vrna_fold_compound_t *fc,
 
   return e;
 }
+#endif
 
 
 PRIVATE int
@@ -652,15 +654,13 @@ E_mb_loop_fast(vrna_fold_compound_t *fc,
                int                  *dmli1,
                int                  *dmli2)
 {
-  unsigned int              *sn;
   int                       decomp, e, dangle_model;
   vrna_param_t              *P;
   vrna_md_t                 *md;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
-  struct sc_wrapper_ml      sc_wrapper;
+  struct hc_mb_def_dat      hc_dat_local;
+  struct sc_mb_dat          sc_wrapper;
 
-  sn            = fc->strand_number;
   P             = fc->params;
   md            = &(P->model_details);
   dangle_model  = md->dangles;
@@ -669,9 +669,9 @@ E_mb_loop_fast(vrna_fold_compound_t *fc,
   e       = INF;
   decomp  = INF;
 
-  evaluate = prepare_hc_default(fc, &hc_dat_local);
+  evaluate = prepare_hc_mb_def(fc, &hc_dat_local);
 
-  init_sc_wrapper(fc, &sc_wrapper);
+  init_sc_mb(fc, &sc_wrapper);
 
   /* do pointer magic for sliding window implementation */
   if (fc->hc->type == VRNA_HC_WINDOW) {
@@ -697,15 +697,17 @@ E_mb_loop_fast(vrna_fold_compound_t *fc,
       break;
   }
 
-  free_sc_wrapper(&sc_wrapper);
+  free_sc_mb(&sc_wrapper);
 
   e = MIN2(e, decomp);
 
+#if 0
   /* add additional cases for possible strand nicks between i and j */
   if ((fc->type == VRNA_FC_TYPE_SINGLE) && (sn[i] != sn[j])) {
     decomp  = E_mb_loop_fake(fc, i, j);
     e       = MIN2(e, decomp);
   }
+#endif
 
   return e;
 }
@@ -719,13 +721,13 @@ E_mb_loop_stack(vrna_fold_compound_t  *fc,
   char                      *ptype, **ptype_local;
   short                     **SS;
   unsigned int              n_seq, s, *tt, sliding_window;
-  int                       *c, *fML, e, decomp, en, i1k, k1j1, ij, k, *indx, turn,
+  int                       *c, *fML, e, decomp, en, i1k, k1j1, ij, k, *indx,
                             type, type_2, *rtype, **c_local, **fML_local;
   vrna_param_t              *P;
   vrna_md_t                 *md;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
-  struct sc_wrapper_ml      sc_wrapper;
+  struct hc_mb_def_dat      hc_dat_local;
+  struct sc_mb_dat          sc_wrapper;
 
   sliding_window = (fc->hc->type == VRNA_HC_WINDOW) ? 1 : 0;
 
@@ -734,7 +736,6 @@ E_mb_loop_stack(vrna_fold_compound_t  *fc,
   indx        = fc->jindx;
   P           = fc->params;
   md          = &(P->model_details);
-  turn        = md->min_loop_size;
   c           = (sliding_window) ? NULL : fc->matrices->c;
   fML         = (sliding_window) ? NULL : fc->matrices->fML;
   c_local     = (sliding_window) ? fc->matrices->c_local : NULL;
@@ -748,8 +749,8 @@ E_mb_loop_stack(vrna_fold_compound_t  *fc,
   ij    = (sliding_window) ? 0 : indx[j] + i;
   e     = INF;
 
-  evaluate = prepare_hc_default(fc, &hc_dat_local);
-  init_sc_wrapper(fc, &sc_wrapper);
+  evaluate = prepare_hc_mb_def(fc, &hc_dat_local);
+  init_sc_mb(fc, &sc_wrapper);
 
   /* prepare type(s) for enclosing pair (i, j) */
   if (fc->type == VRNA_FC_TYPE_COMPARATIVE) {
@@ -765,7 +766,7 @@ E_mb_loop_stack(vrna_fold_compound_t  *fc,
   if (evaluate(i, j, i + 1, j - 1, VRNA_DECOMP_PAIR_ML, &hc_dat_local)) {
     decomp = INF;
     if (sliding_window) {
-      for (k = i + 2 + turn; k < j - 2 - turn; k++) {
+      for (k = i + 2; k < j - 2; k++) {
         if (evaluate(i, j, i + 1, k, VRNA_DECOMP_ML_COAXIAL, &hc_dat_local)) {
           en = c_local[i + 1][k - i - 1] +
                fML_local[k + 1][j - 1 - k - 1];
@@ -817,8 +818,8 @@ E_mb_loop_stack(vrna_fold_compound_t  *fc,
         }
       }
     } else {
-      k1j1 = indx[j - 1] + i + 2 + turn + 1;
-      for (k = i + 2 + turn; k < j - 2 - turn; k++, k1j1++) {
+      k1j1 = indx[j - 1] + i + 2 + 1;
+      for (k = i + 2; k < j - 2; k++, k1j1++) {
         i1k = indx[k] + i + 1;
 
         if (evaluate(i, j, i + 1, k, VRNA_DECOMP_ML_COAXIAL, &hc_dat_local)) {
@@ -885,7 +886,7 @@ E_mb_loop_stack(vrna_fold_compound_t  *fc,
     e = decomp;
   }
 
-  free_sc_wrapper(&sc_wrapper);
+  free_sc_mb(&sc_wrapper);
   free(tt);
 
   return e;
@@ -904,8 +905,8 @@ extend_fm_3p(int                        i,
              int                        *fm,
              vrna_fold_compound_t       *fc,
              vrna_callback_hc_evaluate  *evaluate,
-             struct default_data        *hc_dat_local,
-             struct sc_wrapper_ml       *sc_wrapper)
+             struct hc_mb_def_dat       *hc_dat_local,
+             struct sc_mb_dat           *sc_wrapper)
 {
   short         *S, **SS, **S5, **S3;
   unsigned int  *sn, n_seq, s, sliding_window;
@@ -1006,6 +1007,93 @@ extend_fm_3p(int                        i,
     }
   }
 
+  if (dangle_model % 2) {
+    if ((i + 1 < j) &&
+        (evaluate(i, j, i + 1, j, VRNA_DECOMP_ML_STEM, hc_dat_local))) {
+      en = (sliding_window) ? c_local[i + 1][j - i - 1] : c[indx[j] + i + 1];
+      if (en != INF) {
+        switch (fc->type) {
+          case VRNA_FC_TYPE_SINGLE:
+            type = (sliding_window) ?
+                    vrna_get_ptype_window(i + 1, j, fc->ptype_local) :
+                    vrna_get_ptype(indx[j] + i + 1, fc->ptype);
+
+            en += E_MLstem(type, S[i], -1, P);
+            break;
+
+          case VRNA_FC_TYPE_COMPARATIVE:
+            for (s = 0; s < n_seq; s++) {
+              type  = vrna_get_ptype_md(SS[s][i + 1], SS[s][j], md);
+              en    += E_MLstem(type, S5[s][i + 1], -1, P);
+            }
+            break;
+        }
+
+        if (sc_wrapper->red_stem)
+          en += sc_wrapper->red_stem(i, j, i + 1, j, sc_wrapper);
+
+        e = MIN2(e, en);
+      }
+    }
+
+    if ((i + 1 < j) &&
+        (evaluate(i, j, i, j - 1, VRNA_DECOMP_ML_STEM, hc_dat_local))) {
+      en = (sliding_window) ? c_local[i][j - 1 - i] : c[indx[j - 1] + i];
+      if (en != INF) {
+        switch (fc->type) {
+          case VRNA_FC_TYPE_SINGLE:
+            type = (sliding_window) ?
+                    vrna_get_ptype_window(i, j - 1, fc->ptype_local) :
+                    vrna_get_ptype(indx[j - 1] + i, fc->ptype);
+
+            en += E_MLstem(type, -1, S[j], P);
+            break;
+
+          case VRNA_FC_TYPE_COMPARATIVE:
+            for (s = 0; s < n_seq; s++) {
+              type  = vrna_get_ptype_md(SS[s][i], SS[s][j - 1], md);
+              en    += E_MLstem(type, -1, S3[s][j], P);
+            }
+            break;
+        }
+
+        if (sc_wrapper->red_stem)
+          en += sc_wrapper->red_stem(i, j, i, j - 1, sc_wrapper);
+
+        e = MIN2(e, en);
+      }
+    }
+
+    if ((i + 2 < j) &&
+        (evaluate(i, j, i + 1, j - 1, VRNA_DECOMP_ML_STEM, hc_dat_local))) {
+      en = (sliding_window) ? c_local[i + 1][j - 1 - i - 1] : c[indx[j - 1] + i + 1];
+      if (en != INF) {
+        switch (fc->type) {
+          case VRNA_FC_TYPE_SINGLE:
+            type = (sliding_window) ?
+                    vrna_get_ptype_window(i + 1, j - 1, fc->ptype_local) :
+                    vrna_get_ptype(indx[j - 1] + i + 1, fc->ptype);
+
+            en += E_MLstem(type, S[i], S[j], P);
+            break;
+
+          case VRNA_FC_TYPE_COMPARATIVE:
+            for (s = 0; s < n_seq; s++) {
+              type  = vrna_get_ptype_md(SS[s][i + 1], SS[s][j - 1], md);
+              en    += E_MLstem(type, S5[s][i], S3[s][j], P);
+            }
+            break;
+        }
+
+        if (sc_wrapper->red_stem)
+          en += sc_wrapper->red_stem(i, j, i + 1, j - 1, sc_wrapper);
+
+        e = MIN2(e, en);
+      }
+    }
+
+  }
+
   if (with_ud) {
     for (cnt = 0; cnt < domains_up->uniq_motif_count; cnt++) {
       u = domains_up->uniq_motif_size[cnt];
@@ -1049,7 +1137,7 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
   short                     *S, **SS, **S5, **S3;
   unsigned int              *sn, *se, n_seq, s;
   int                       k, en, decomp, mm5, mm3, type_2, k1j, length, *indx,
-                            *c, *fm, ij, dangle_model, turn, type, *rtype, circular, e, u,
+                            *c, *fm, ij, dangle_model, type, *rtype, circular, e, u,
                             cnt, with_ud, sliding_window, **c_local, **fm_local;
   vrna_hc_t                 *hc;
   vrna_sc_t                 *sc;
@@ -1057,8 +1145,8 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
   vrna_md_t                 *md;
   vrna_ud_t                 *domains_up;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
-  struct sc_wrapper_ml      sc_wrapper;
+  struct hc_mb_def_dat      hc_dat_local;
+  struct sc_mb_dat          sc_wrapper;
 
   sliding_window = (fc->hc->type == VRNA_HC_WINDOW) ? 1 : 0;
 
@@ -1084,15 +1172,14 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
   md            = &(P->model_details);
   ij            = (sliding_window) ? 0 : indx[j] + i;
   dangle_model  = md->dangles;
-  turn          = md->min_loop_size;
   rtype         = &(md->rtype[0]);
   circular      = md->circ;
   domains_up    = fc->domains_up;
   with_ud       = (domains_up && domains_up->energy_cb) ? 1 : 0;
   e             = INF;
-  evaluate      = prepare_hc_default(fc, &hc_dat_local);
+  evaluate      = prepare_hc_mb_def(fc, &hc_dat_local);
 
-  init_sc_wrapper(fc, &sc_wrapper);
+  init_sc_mb(fc, &sc_wrapper);
 
   /*
    *  extension with one unpaired nucleotide at the right (3' site)
@@ -1105,7 +1192,8 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
    *  and all other variants which are needed for odd
    *  dangle models
    */
-  if (evaluate(i, j, i + 1, j, VRNA_DECOMP_ML_ML, &hc_dat_local)) {
+  if ((i + 1 < j) &&
+      (evaluate(i, j, i + 1, j, VRNA_DECOMP_ML_ML, &hc_dat_local))) {
     en = (sliding_window) ? fm_local[i + 1][j - i - 1] : fm[ij + 1];
     if (en != INF) {
       en += P->MLbase *
@@ -1159,7 +1247,8 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
         mm3 = S[j];
     }
 
-    if (evaluate(i, j, i + 1, j, VRNA_DECOMP_ML_STEM, &hc_dat_local)) {
+    if ((i + 1 < j) &&
+        (evaluate(i, j, i + 1, j, VRNA_DECOMP_ML_STEM, &hc_dat_local))) {
       en = (sliding_window) ? c_local[i + 1][j - (i + 1)] : c[ij + 1];
       if (en != INF) {
         en += P->MLbase *
@@ -1189,7 +1278,8 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
       }
     }
 
-    if (evaluate(i, j, i, j - 1, VRNA_DECOMP_ML_STEM, &hc_dat_local)) {
+    if ((i + 1 < j) &&
+        (evaluate(i, j, i, j - 1, VRNA_DECOMP_ML_STEM, &hc_dat_local))) {
       en = (sliding_window) ? c_local[i][j - 1 - i] : c[indx[j - 1] + i];
       if (en != INF) {
         en += P->MLbase *
@@ -1220,7 +1310,8 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
       }
     }
 
-    if (evaluate(i, j, i + 1, j - 1, VRNA_DECOMP_ML_STEM, &hc_dat_local)) {
+    if ((i + 2 < j) &&
+        (evaluate(i, j, i + 1, j - 1, VRNA_DECOMP_ML_STEM, &hc_dat_local))) {
       en = (sliding_window) ? c_local[i + 1][j - 1 - (i + 1)] : c[indx[j - 1] + i + 1];
       if (en != INF) {
         en += 2 * P->MLbase *
@@ -1266,11 +1357,11 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
     fmi_tmp -= i;
 
     /* copy data */
-    for (k = i + 1 + turn; k <= j - 2 - turn; k++)
+    for (k = i + 1; k <= j - 2; k++)
       fmi_tmp[k] = fmi[k];
 
     /* mask unavailable decompositions */
-    for (k = i + 1 + turn; k <= j - 2 - turn; k++)
+    for (k = i + 1; k <= j - 2; k++)
       if (!hc->f(i, j, k, k + 1, VRNA_DECOMP_ML_ML_ML, hc->data))
         fmi_tmp[k] = INF;
   }
@@ -1281,18 +1372,18 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
       fmi_tmp -= i;
 
       /* copy data */
-      for (k = i + 1 + turn; k <= j - 2 - turn; k++)
+      for (k = i + 1; k <= j - 2; k++)
         fmi_tmp[k] = fmi[k];
     }
 
-    for (k = i + 1 + turn; k <= j - 2 - turn; k++)
+    for (k = i + 1; k <= j - 2; k++)
       if (fmi_tmp[k] != INF)
         fmi_tmp[k] += sc_wrapper.decomp_ml(i, j, k, k + 1, &sc_wrapper);
   }
 
   /* modular decomposition -------------------------------*/
   if (sliding_window) {
-    for (decomp = INF, k = i + 1 + turn; k <= j - 2 - turn; k++) {
+    for (decomp = INF, k = i + 1; k <= j - 2; k++) {
       if ((fmi_tmp[k] != INF) && (fm_local[k + 1][j - (k + 1)] != INF)) {
         en      = fmi_tmp[k] + fm_local[k + 1][j - (k + 1)];
         decomp  = MIN2(decomp, en);
@@ -1300,7 +1391,7 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
     }
   } else {
     decomp  = INF;
-    k       = i + turn + 1;
+    k       = i + 1;
     if (k >= j)
       k = j - 1;
 
@@ -1312,14 +1403,14 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
      *  decomposition
      */
     while (1) {
-      int last_nt = se[sn[k - 1]];  /* go to last nucleotide of current strand */
-      if (last_nt > j - turn - 2)
-        last_nt = j - turn - 2;     /* at most go to last possible decomposition split before reaching j */
+      int last_nt = se[sn[k]];  /* go to last nucleotide of current strand */
+      if (last_nt > j - 2)
+        last_nt = j - 2;     /* at most go to last possible decomposition split before reaching j */
 
       if (last_nt < i)
         last_nt = i; /* do not start before i */
 
-      const int count = last_nt - k + 1;
+      const int count = last_nt - k;
 
       en      = vrna_fun_zip_add_min(fmi_tmp + k, fm + k1j, count);
       decomp  = MIN2(decomp, en);
@@ -1328,7 +1419,7 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
       k   += count + 1;
       k1j += count + 1;
 
-      if (k > j - turn - 2)
+      if (k > j - 2)
         break;
     }
   }
@@ -1349,7 +1440,7 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
     /* additional ML decomposition as two coaxially stacked helices */
     if (sliding_window) {
       /* additional ML decomposition as two coaxially stacked helices */
-      for (decomp = INF, k = i + 1 + turn; k <= j - 2 - turn; k++) {
+      for (decomp = INF, k = i + 1; k <= j - 2; k++) {
         if (evaluate(i, k, k + 1, j, VRNA_DECOMP_ML_COAXIAL_ENC, &hc_dat_local)) {
           type    = rtype[vrna_get_ptype_window(i, k, ptype_local)];
           type_2  = rtype[vrna_get_ptype_window(k + 1, j, ptype_local)];
@@ -1367,9 +1458,9 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
       }
     } else {
       int ik;
-      k1j     = indx[j] + i + turn + 2;
+      k1j     = indx[j] + i + 2;
       decomp  = INF;
-      k       = i + turn + 1;
+      k       = i + 1;
       if (k >= j)
         k = j - 1;
 
@@ -1380,8 +1471,8 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
        */
       while (1) {
         int last_nt = se[sn[k - 1]];  /* go to last nucleotide of current strand */
-        if (last_nt > j - turn - 2)
-          last_nt = j - turn - 2;     /* at most go to last possible decomposition split before reaching j */
+        if (last_nt > j - 2)
+          last_nt = j - 2;     /* at most go to last possible decomposition split before reaching j */
 
         if (last_nt < i)
           last_nt = i; /* do not start before i */
@@ -1423,7 +1514,7 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
         k++;
         k1j++;
 
-        if (k > j - turn - 2)
+        if (k > j - 2)
           break;
       }
     }
@@ -1451,7 +1542,7 @@ E_ml_stems_fast(vrna_fold_compound_t  *fc,
 
   fmi[j] = e;
 
-  free_sc_wrapper(&sc_wrapper);
+  free_sc_mb(&sc_wrapper);
 
   return e;
 }
