@@ -99,403 +99,6 @@ vrna_BT_mb_loop(vrna_fold_compound_t  *fc,
 }
 
 
-PUBLIC int
-vrna_BT_mb_loop_fake(vrna_fold_compound_t *fc,
-                     int                  *u,
-                     int                  *i,
-                     int                  *j,
-                     vrna_bp_stack_t      *bp_stack,
-                     int                  *stack_count)
-{
-  char                      *ptype;
-  short                     mm5, mm3, *S1;
-  unsigned int              *sn, *se;
-  int                       length, ii, jj, k, en, fij, fi, *my_c, *my_fc, *my_ggg,
-                            *idx, with_gquad, dangle_model, turn, type;
-  vrna_param_t              *P;
-  vrna_md_t                 *md;
-  vrna_sc_t                 *sc;
-  vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
-
-  length        = fc->length;
-  P             = fc->params;
-  md            = &(P->model_details);
-  sn            = fc->strand_number;
-  se            = fc->strand_end;
-  sc            = fc->sc;
-  S1            = fc->sequence_encoding;
-  ptype         = fc->ptype;
-  idx           = fc->jindx;
-  my_c          = fc->matrices->c;
-  my_fc         = fc->matrices->fc;
-  my_ggg        = fc->matrices->ggg;
-  turn          = md->min_loop_size;
-  with_gquad    = md->gquad;
-  dangle_model  = md->dangles;
-  evaluate      = prepare_hc_default_ext(fc, &hc_dat_local);
-
-  ii  = *i;
-  jj  = *j;
-
-  if (ii <= se[0]) {
-    /* 'lower' part (fc[i<cut,j=cut-1]) */
-
-    /* nibble off unpaired 5' bases */
-    do {
-      fij = my_fc[ii];
-      fi  = INF;
-
-      if (evaluate(ii, jj, ii + 1, jj, VRNA_DECOMP_EXT_EXT, &hc_dat_local)) {
-        fi = my_fc[ii + 1];
-
-        if (sc)
-          if (sc->energy_up)
-            fi += sc->energy_up[ii][1];
-      }
-
-      if (++ii == jj)
-        break;
-    } while (fij == fi);
-    ii--;
-
-    if (jj < ii + turn + 2) {
-      /* no more pairs */
-      *u = *i = *j = -1;
-      return 1;
-    }
-
-    mm5 = (ii > 1 && (sn[ii - 1] == sn[ii])) ? S1[ii - 1] : -1;
-
-    /* i or i+1 is paired. Find pairing partner */
-    switch (dangle_model) {
-      case 0:
-        for (k = ii + turn + 1; k <= jj; k++) {
-          if (evaluate(ii, jj, k, k + 1, VRNA_DECOMP_EXT_STEM_EXT, &hc_dat_local)) {
-            type = vrna_get_ptype(idx[k] + ii, ptype);
-
-            if (fij == my_fc[k + 1] + my_c[idx[k] + ii] + vrna_E_ext_stem(type, -1, -1, P)) {
-              bp_stack[++(*stack_count)].i  = ii;
-              bp_stack[(*stack_count)].j    = k;
-              *u                            = k + 1;
-              *i                            = ii;
-              *j                            = k;
-              return 1;
-            }
-          }
-
-          if (with_gquad) {
-            if (fij == my_fc[k + 1] + my_ggg[idx[k] + ii]) {
-              *u  = k + 1;
-              *i  = *j = -1;
-              return vrna_BT_gquad_mfe(fc, ii, k, bp_stack, stack_count);
-            }
-          }
-        }
-        break;
-
-      case 2:
-        for (k = ii + turn + 1; k <= jj; k++) {
-          if (evaluate(ii, jj, k, k + 1, VRNA_DECOMP_EXT_STEM_EXT, &hc_dat_local)) {
-            mm3   = (sn[k] == sn[k + 1]) ? S1[k + 1] : -1;
-            type  = vrna_get_ptype(idx[k] + ii, ptype);
-
-            if (fij == my_fc[k + 1] + my_c[idx[k] + ii] + vrna_E_ext_stem(type, mm5, mm3, P)) {
-              bp_stack[++(*stack_count)].i  = ii;
-              bp_stack[(*stack_count)].j    = k;
-              *u                            = k + 1;
-              *i                            = ii;
-              *j                            = k;
-              return 1;
-            }
-          }
-
-          if (with_gquad) {
-            if (fij == my_fc[k + 1] + my_ggg[idx[k] + ii]) {
-              *u  = k + 1;
-              *i  = *j = -1;
-              return vrna_BT_gquad_mfe(fc, ii, k, bp_stack, stack_count);
-            }
-          }
-        }
-        break;
-
-      default:
-        for (k = ii + turn + 1; k <= jj; k++) {
-          type = vrna_get_ptype(idx[k] + ii, ptype);
-          if (evaluate(ii, jj, k, k + 1, VRNA_DECOMP_EXT_STEM_EXT, &hc_dat_local)) {
-            if (fij == my_fc[k + 1] + my_c[idx[k] + ii] + vrna_E_ext_stem(type, -1, -1, P)) {
-              bp_stack[++(*stack_count)].i  = ii;
-              bp_stack[(*stack_count)].j    = k;
-              *u                            = k + 1;
-              *i                            = ii;
-              *j                            = k;
-              return 1;
-            }
-          }
-
-          if (evaluate(ii, jj, k, k + 2, VRNA_DECOMP_EXT_STEM_EXT, &hc_dat_local)) {
-            mm3 = (sn[k] == sn[k + 1]) ? S1[k + 1] : -1;
-            en  = my_c[idx[k] + ii];
-            if (sc)
-              if (sc->energy_up)
-                en += sc->energy_up[k + 1][1];
-
-            if (fij == my_fc[k + 2] + en + vrna_E_ext_stem(type, -1, mm3, P)) {
-              bp_stack[++(*stack_count)].i  = ii;
-              bp_stack[(*stack_count)].j    = k;
-              *u                            = k + 2;
-              *i                            = ii;
-              *j                            = k;
-              return 1;
-            }
-          }
-
-          if (with_gquad) {
-            if (fij == my_fc[k + 1] + my_ggg[idx[k] + ii]) {
-              *u  = k + 1;
-              *i  = *j = -1;
-              return vrna_BT_gquad_mfe(fc, ii, k, bp_stack, stack_count);
-            }
-          }
-
-          if (evaluate(ii, jj, k, k + 1, VRNA_DECOMP_EXT_STEM_EXT1, &hc_dat_local)) {
-            mm5   = (sn[ii] == sn[ii + 1]) ? S1[ii] : -1;
-            mm3   = (sn[k] == sn[k + 1]) ? S1[k + 1] : -1;
-            type  = vrna_get_ptype(idx[k] + ii + 1, ptype);
-
-            en = my_c[idx[k] + ii + 1];
-            if (sc)
-              if (sc->energy_up)
-                en += sc->energy_up[ii][1];
-
-            if (fij == en + my_fc[k + 1] + vrna_E_ext_stem(type, mm5, -1, P)) {
-              bp_stack[++(*stack_count)].i  = ii + 1;
-              bp_stack[(*stack_count)].j    = k;
-              *u                            = k + 1;
-              *i                            = ii + 1;
-              *j                            = k;
-              return 1;
-            }
-          }
-
-          if ((k < jj) && (evaluate(ii, jj, k, k + 2, VRNA_DECOMP_EXT_STEM_EXT1, &hc_dat_local))) {
-            mm5   = (sn[ii] == sn[ii + 1]) ? S1[ii] : -1;
-            mm3   = (sn[k] == sn[k + 1]) ? S1[k + 1] : -1;
-            type  = vrna_get_ptype(idx[k] + ii + 1, ptype);
-
-            en = my_c[idx[k] + ii + 1];
-            if (sc)
-              if (sc->energy_up)
-                en += sc->energy_up[k + 1][1];
-
-            if (fij == en + my_fc[k + 2] + vrna_E_ext_stem(type, mm5, mm3, P)) {
-              bp_stack[++(*stack_count)].i  = ii + 1;
-              bp_stack[(*stack_count)].j    = k;
-              *u                            = k + 2;
-              *i                            = ii + 1;
-              *j                            = k;
-              return 1;
-            }
-          }
-        }
-        break;
-    }
-  } else {
-    /* 'upper' part (fc[i=cut,j>cut]) */
-
-    /* nibble off unpaired 3' bases */
-    do {
-      fij = my_fc[jj];
-      fi  = INF;
-
-      if (evaluate(ii, jj, ii, jj - 1, VRNA_DECOMP_EXT_EXT, &hc_dat_local)) {
-        fi = my_fc[jj - 1];
-
-        if (sc)
-          if (sc->energy_up)
-            fi += sc->energy_up[jj][1];
-      }
-
-      if (--jj == ii)
-        break;
-    } while (fij == fi);
-    jj++;
-
-    if (jj < ii + turn + 2) {
-      /* no more pairs */
-      *u = *i = *j = -1;
-      return 1;
-    }
-
-    /* j or j-1 is paired. Find pairing partner */
-    mm3 = ((jj < length) && (sn[jj] == sn[jj + 1])) ? S1[jj + 1] : -1;
-    switch (dangle_model) {
-      case 0:
-        for (k = jj - turn - 1; k >= ii; k--) {
-          if (with_gquad) {
-            if (fij == my_fc[k - 1] + my_ggg[idx[jj] + k]) {
-              *u  = k - 1;
-              *i  = *j = -1;
-              return vrna_BT_gquad_mfe(fc, k, jj, bp_stack, stack_count);
-            }
-          }
-
-          if (evaluate(ii, jj, k - 1, k, VRNA_DECOMP_EXT_EXT_STEM, &hc_dat_local)) {
-            type = vrna_get_ptype(idx[jj] + k, ptype);
-
-            en = my_c[idx[jj] + k];
-            if (sn[k] != sn[jj])
-              en += P->DuplexInit;
-
-            if (fij == my_fc[k - 1] + en + vrna_E_ext_stem(type, -1, -1, P)) {
-              bp_stack[++(*stack_count)].i  = k;
-              bp_stack[(*stack_count)].j    = jj;
-              *u                            = k - 1;
-              *i                            = k;
-              *j                            = jj;
-              return 1;
-            }
-          }
-        }
-        break;
-
-      case 2:
-        for (k = jj - turn - 1; k >= ii; k--) {
-          if (with_gquad) {
-            if (fij == my_fc[k - 1] + my_ggg[idx[jj] + k]) {
-              *u  = k - 1;
-              *i  = *j = -1;
-              return vrna_BT_gquad_mfe(fc, k, jj, bp_stack, stack_count);
-            }
-          }
-
-          if (evaluate(ii, jj, k - 1, k, VRNA_DECOMP_EXT_EXT_STEM, &hc_dat_local)) {
-            mm5   = ((k > 1) && (sn[k - 1] == sn[k])) ? S1[k - 1] : -1;
-            type  = vrna_get_ptype(idx[jj] + k, ptype);
-
-            en = my_c[idx[jj] + k];
-            if (sn[k] != sn[jj])
-              en += P->DuplexInit;
-
-            if (fij == my_fc[k - 1] + en + vrna_E_ext_stem(type, mm5, mm3, P)) {
-              bp_stack[++(*stack_count)].i  = k;
-              bp_stack[(*stack_count)].j    = jj;
-              *u                            = k - 1;
-              *i                            = k;
-              *j                            = jj;
-              return 1;
-            }
-          }
-        }
-        break;
-
-      default:
-        for (k = jj - turn - 1; k >= ii; k--) {
-          if (with_gquad) {
-            if (fij == my_fc[k - 1] + my_ggg[idx[jj] + k]) {
-              *u  = k - 1;
-              *i  = *j = -1;
-              return vrna_BT_gquad_mfe(fc, k, jj, bp_stack, stack_count);
-            }
-          }
-
-          if (evaluate(ii, jj, k - 1, k, VRNA_DECOMP_EXT_EXT_STEM, &hc_dat_local)) {
-            type = vrna_get_ptype(idx[jj] + k, ptype);
-
-            en = my_c[idx[jj] + k];
-            if (sn[k] != sn[jj])
-              en += P->DuplexInit;
-
-            if (fij == my_fc[k - 1] + en + vrna_E_ext_stem(type, -1, -1, P)) {
-              bp_stack[++(*stack_count)].i  = k;
-              bp_stack[(*stack_count)].j    = jj;
-              *u                            = k - 1;
-              *i                            = k;
-              *j                            = jj;
-              return 1;
-            }
-          }
-
-          if ((k > 1) && (sn[k - 1] == sn[k]) &&
-              evaluate(ii, jj, k - 2, k, VRNA_DECOMP_EXT_EXT_STEM, &hc_dat_local)) {
-            type = vrna_get_ptype(idx[jj] + k, ptype);
-
-            en = my_c[idx[jj] + k];
-            if (sn[k] != sn[jj])
-              en += P->DuplexInit;
-
-            mm5 = S1[k - 1];
-
-            if (sc)
-              if (sc->energy_up)
-                en += sc->energy_up[k - 1][1];
-
-            if (fij == my_fc[k - 2] + en + vrna_E_ext_stem(type, mm5, -1, P)) {
-              bp_stack[++(*stack_count)].i  = k;
-              bp_stack[(*stack_count)].j    = jj;
-              *u                            = k - 2;
-              *i                            = k;
-              *j                            = jj;
-              return 1;
-            }
-          }
-
-          if ((sn[jj - 1] == sn[jj]) &&
-              evaluate(ii, jj, k - 1, k, VRNA_DECOMP_EXT_EXT_STEM1, &hc_dat_local)) {
-            type = vrna_get_ptype(idx[jj - 1] + k, ptype);
-
-            mm3 = S1[jj];
-            en  = my_c[idx[jj - 1] + k];
-            if (sn[k] != sn[jj - 1])
-              en += P->DuplexInit;         /* ??? */
-
-            if (sc)
-              if (sc->energy_up)
-                en += sc->energy_up[jj][1];
-
-            if (fij == en + my_fc[k - 1] + vrna_E_ext_stem(type, -1, mm3, P)) {
-              bp_stack[++(*stack_count)].i  = k;
-              bp_stack[(*stack_count)].j    = jj - 1;
-              *u                            = k - 1;
-              *i                            = k;
-              *j                            = jj - 1;
-              return 1;
-            }
-          }
-
-          if ((k > ii) && (sn[jj - 1] == sn[jj]) &&
-              evaluate(ii, jj, k - 2, k, VRNA_DECOMP_EXT_EXT_STEM1, &hc_dat_local)) {
-            type = vrna_get_ptype(idx[jj - 1] + k, ptype);
-
-            mm3 = S1[jj];
-            en  = my_c[idx[jj - 1] + k];
-            if (sn[k] != sn[jj - 1])
-              en += P->DuplexInit;         /* ??? */
-
-            mm5 = (sn[k - 1] == sn[k]) ? S1[k - 1] : -1;
-            if (sc)
-              if (sc->energy_up)
-                en += sc->energy_up[k - 1][1];
-
-            if (fij == my_fc[k - 2] + en + vrna_E_ext_stem(type, mm5, mm3, P)) {
-              bp_stack[++(*stack_count)].i  = k;
-              bp_stack[(*stack_count)].j    = jj - 1;
-              *u                            = k - 2;
-              *i                            = k;
-              *j                            = jj - 1;
-              return 1;
-            }
-          }
-        }
-        break;
-    }
-  }
-
-  return 0;
-}
-
-
 PRIVATE int
 BT_mb_loop_split(vrna_fold_compound_t *fc,
                  int                  *i,
@@ -512,14 +115,14 @@ BT_mb_loop_split(vrna_fold_compound_t *fc,
   short                     *S1, **SS, **S5, **S3;
   unsigned int              n_seq, s;
   int                       ij, ii, jj, fij, fi, u, en, *my_c, *my_fML, *my_ggg,
-                            turn, *idx, with_gquad, dangle_model, *rtype, kk, cnt,
+                            *idx, with_gquad, dangle_model, *rtype, kk, cnt,
                             with_ud, type, type_2, en2, **c_local, **fML_local, **ggg_local;
   vrna_param_t              *P;
   vrna_md_t                 *md;
   vrna_ud_t                 *domains_up;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
-  struct sc_wrapper_ml      sc_wrapper;
+  struct hc_mb_def_dat      hc_dat_local;
+  struct sc_mb_dat          sc_wrapper;
 
   sliding_window  = (fc->hc->type == VRNA_HC_WINDOW) ? 1 : 0;
   n_seq           = (fc->type == VRNA_FC_TYPE_SINGLE) ? 1 : fc->n_seq;
@@ -542,14 +145,12 @@ BT_mb_loop_split(vrna_fold_compound_t *fc,
   fML_local = (sliding_window) ? fc->matrices->fML_local : NULL;
   ggg_local = (sliding_window) ? fc->matrices->ggg_local : NULL;
 
-
-  turn          = md->min_loop_size;
   with_gquad    = md->gquad;
   with_ud       = (domains_up && domains_up->energy_cb) ? 1 : 0;
   dangle_model  = md->dangles;
-  evaluate      = prepare_hc_default(fc, &hc_dat_local);
+  evaluate      = prepare_hc_mb_def(fc, &hc_dat_local);
 
-  init_sc_wrapper(fc, &sc_wrapper);
+  init_sc_mb(fc, &sc_wrapper);
 
   ii  = *i;
   jj  = *j;
@@ -702,7 +303,7 @@ BT_mb_loop_split(vrna_fold_compound_t *fc,
     } while (fij == fi);
     ii--;
 
-    if (jj < ii + turn + 1) /* no more pairs */
+    if (jj <= ii) /* no more pairs */
       return 0;
   }
 
@@ -905,7 +506,10 @@ BT_mb_loop_split(vrna_fold_compound_t *fc,
           case VRNA_FC_TYPE_COMPARATIVE:
             for (s = 0; s < n_seq; s++) {
               type  = vrna_get_ptype_md(SS[s][ii], SS[s][jj], md);
-              en2   += E_MLstem(type, S5[s][ii], S3[s][jj], P);
+              en2   += E_MLstem(type,
+                                S5[s][ii],
+                                S3[s][jj],
+                                P);
             }
             break;
         }
@@ -926,7 +530,7 @@ BT_mb_loop_split(vrna_fold_compound_t *fc,
   }
 
   /* 2. Test for possible split point */
-  for (u = ii + 1 + turn; u <= jj - 2 - turn; u++) {
+  for (u = ii + 1; u <= jj - 2; u++) {
     if (evaluate(ii, jj, u, u + 1, VRNA_DECOMP_ML_ML_ML, &hc_dat_local)) {
       if (sliding_window)
         en = fML_local[ii][u - ii] +
@@ -951,8 +555,8 @@ BT_mb_loop_split(vrna_fold_compound_t *fc,
   /* 3. last chance! Maybe coax stack */
   if (dangle_model == 3) {
     int ik, k1j;
-    k1j = (sliding_window) ? 0 : idx[jj] + ii + turn + 2;
-    for (u = ii + 1 + turn; u <= jj - 2 - turn; u++, k1j++) {
+    k1j = (sliding_window) ? 0 : idx[jj] + ii + 2;
+    for (u = ii + 1; u <= jj - 2; u++, k1j++) {
       ik = (sliding_window) ? 0 : idx[u] + ii;
       if (evaluate(ii, u, u + 1, jj, VRNA_DECOMP_ML_COAXIAL_ENC, &hc_dat_local)) {
         en = 2 * P->MLintern[1] *
@@ -964,8 +568,9 @@ BT_mb_loop_split(vrna_fold_compound_t *fc,
           case VRNA_FC_TYPE_SINGLE:
             type =
               (sliding_window) ? rtype[vrna_get_ptype_window(ii, u,
-                                                             ptype_local)] : rtype[vrna_get_ptype(ik,
-                                                                                                  ptype)
+                                                             ptype_local)] : rtype[vrna_get_ptype(
+                                                                                     ik,
+                                                                                     ptype)
               ];
             type_2 =
               (sliding_window) ? rtype[vrna_get_ptype_window(u + 1, jj,
@@ -1014,17 +619,14 @@ BT_mb_loop(vrna_fold_compound_t *fc,
   unsigned char             sliding_window;
   char                      *ptype, **ptype_local;
   short                     s5, s3, *S1, **SS, **S5, **S3;
-  unsigned int              *sn, *se, n_seq, s, *tt;
-  int                       ij, p, q, r, e, tmp_en, *idx, turn, dangle_model,
-                            *my_c, *my_fML, *my_fc, *rtype, type, type_2, **c_local, **fML_local;
+  unsigned int              *sn, n_seq, s, *tt;
+  int                       ij, p, q, r, e, tmp_en, *idx, dangle_model,
+                            *my_c, *my_fML, *rtype, type, type_2, **c_local, **fML_local;
   vrna_param_t              *P;
   vrna_md_t                 *md;
-  vrna_sc_t                 *sc;
   vrna_callback_hc_evaluate *evaluate;
-  struct default_data       hc_dat_local;
-  vrna_callback_hc_evaluate *evaluate_ext;
-  struct default_data       hc_dat_local_ext;
-  struct sc_wrapper_ml      sc_wrapper;
+  struct hc_mb_def_dat      hc_dat_local;
+  struct sc_mb_dat          sc_wrapper;
 
   sliding_window  = (fc->hc->type == VRNA_HC_WINDOW) ? 1 : 0;
   n_seq           = (fc->type == VRNA_FC_TYPE_SINGLE) ? 1 : fc->n_seq;
@@ -1037,131 +639,29 @@ BT_mb_loop(vrna_fold_compound_t *fc,
   P               = fc->params;
   md              = &(P->model_details);
   sn              = fc->strand_number;
-  se              = fc->strand_end;
-  sc              = fc->sc;
   my_c            = (sliding_window) ? NULL : fc->matrices->c;
   my_fML          = (sliding_window) ? NULL : fc->matrices->fML;
   c_local         = (sliding_window) ? fc->matrices->c_local : NULL;
   fML_local       = (sliding_window) ? fc->matrices->fML_local : NULL;
-  my_fc           = (sliding_window) ? NULL : fc->matrices->fc;
-  turn            = md->min_loop_size;
   ptype           = (fc->type == VRNA_FC_TYPE_SINGLE) ? (sliding_window ? NULL : fc->ptype) : NULL;
   ptype_local     = (sliding_window) ? fc->ptype_local : NULL;
   rtype           = &(md->rtype[0]);
   type            = (fc->type == VRNA_FC_TYPE_SINGLE) ?
-                    (sliding_window ? rtype[vrna_get_ptype_window(*i, *j, ptype_local)] : rtype[vrna_get_ptype(ij, ptype)]) :
+                    (sliding_window ? rtype[vrna_get_ptype_window(*i, *j,
+                                                                  ptype_local)] : rtype[
+                       vrna_get_ptype(ij, ptype)]) :
                     0;
   tt            = NULL;
   dangle_model  = md->dangles;
-  evaluate      = prepare_hc_default(fc, &hc_dat_local);
-  evaluate_ext  = prepare_hc_default_ext(fc, &hc_dat_local_ext);
+  evaluate      = prepare_hc_mb_def(fc, &hc_dat_local);
 
-  init_sc_wrapper(fc, &sc_wrapper);
+  init_sc_mb(fc, &sc_wrapper);
 
   p = *i + 1;
   q = *j - 1;
 
-  r = q - turn - 1;
+  r = q - 1;
 
-  /* is it a fake multi-loop? */
-  if (sn[*i] != sn[*j]) {
-    if (evaluate_ext(*i, *j, *i, *j, VRNA_DECOMP_EXT_STEM, &hc_dat_local)) {
-      int ii, jj;
-      ii  = jj = 0;
-      e   = my_fc[p] + my_fc[q];
-      if (sc)
-        if (sc->energy_bp)
-          e += sc->energy_bp[ij];
-
-      s5  = (sn[q] == sn[*j]) ? S1[q] : -1;
-      s3  = (sn[*i] == sn[p]) ? S1[p] : -1;
-
-      switch (dangle_model) {
-        case 0:
-          if (en == e + vrna_E_ext_stem(type, -1, -1, P))
-            ii = p, jj = q;
-
-          break;
-
-        case 2:
-          if (en == e + vrna_E_ext_stem(type, s5, s3, P))
-            ii = p, jj = q;
-
-          break;
-
-        default:
-          if (en == e + vrna_E_ext_stem(type, -1, -1, P)) {
-            ii = p, jj = q;
-            break;
-          }
-
-          if (evaluate_ext(p, q, p + 1, q, VRNA_DECOMP_EXT_EXT, &hc_dat_local_ext)) {
-            e = my_fc[p + 1] + my_fc[q];
-            if (sc) {
-              if (sc->energy_up)
-                e += sc->energy_up[p][1];
-
-              if (sc->energy_bp)
-                e += sc->energy_bp[ij];
-            }
-
-            if (en == e + vrna_E_ext_stem(type, -1, s3, P)) {
-              ii  = p + 1;
-              jj  = q;
-              break;
-            }
-          }
-
-          if (evaluate_ext(p, q, p, q - 1, VRNA_DECOMP_EXT_EXT, &hc_dat_local_ext)) {
-            e = my_fc[p] + my_fc[q - 1];
-            if (sc) {
-              if (sc->energy_up)
-                e += sc->energy_up[q][1];
-
-              if (sc->energy_bp)
-                e += sc->energy_bp[ij];
-            }
-
-            if (en == e + vrna_E_ext_stem(type, s5, -1, P)) {
-              ii  = p;
-              jj  = q - 1;
-              break;
-            }
-          }
-
-          if (evaluate_ext(p, q, p + 1, q - 1, VRNA_DECOMP_EXT_EXT, &hc_dat_local_ext)) {
-            e = my_fc[p + 1] + my_fc[q - 1];
-            if (sc) {
-              if (sc->energy_up)
-                e += sc->energy_up[p][1] + sc->energy_up[q][1];
-
-              if (sc->energy_bp)
-                e += sc->energy_bp[ij];
-            }
-
-            if (en == e + vrna_E_ext_stem(type, s5, s3, P)) {
-              ii  = p + 1;
-              jj  = q - 1;
-              break;
-            }
-          }
-
-          break;
-      }
-
-      if (ii) {
-        /* found a decomposition */
-        *component1 = 3;
-        *i          = ii;
-        *k          = se[0];
-        *j          = jj;
-        *component2 = 4;
-        return 1;
-      }
-    }
-  }
-
-  /* true multi loop? */
   *component1 = *component2 = 1;  /* both components are MB loop parts by default */
 
   s5  = -1;
@@ -1211,7 +711,7 @@ BT_mb_loop(vrna_fold_compound_t *fc,
     if (sc_wrapper.pair)
       e -= sc_wrapper.pair(*i, *j, &sc_wrapper);
 
-    for (r = *i + 2 + turn; r < *j - 2 - turn; ++r) {
+    for (r = *i + 2; r < *j - 2; ++r) {
       if (evaluate(p, q, r, r + 1, VRNA_DECOMP_ML_ML_ML, &hc_dat_local)) {
         if (sliding_window)
           tmp_en = fML_local[p][r - p] +
@@ -1229,7 +729,8 @@ BT_mb_loop(vrna_fold_compound_t *fc,
     }
   }
 
-  if (dangles % 2) { /* odd dangles need more special treatment */
+  if (dangles % 2) {
+    /* odd dangles need more special treatment */
     if (evaluate(*i, *j, p + 1, q, VRNA_DECOMP_PAIR_ML, &hc_dat_local)) {
       e = en -
           (P->MLclosing + P->MLbase) *
@@ -1249,7 +750,7 @@ BT_mb_loop(vrna_fold_compound_t *fc,
           break;
       }
 
-      for (r = p + turn + 1; r < q - turn - 1; ++r) {
+      for (r = p + 1; r < q - 1; ++r) {
         if (evaluate(p + 1, q, r, r + 1, VRNA_DECOMP_ML_ML_ML, &hc_dat_local)) {
           if (sliding_window)
             tmp_en = fML_local[p + 1][r - (p + 1)] +
@@ -1288,7 +789,7 @@ BT_mb_loop(vrna_fold_compound_t *fc,
           break;
       }
 
-      for (r = p + turn + 1; r < q - turn - 1; ++r) {
+      for (r = p + 1; r < q - 1; ++r) {
         if (evaluate(p, q - 1, r, r + 1, VRNA_DECOMP_ML_ML_ML, &hc_dat_local)) {
           if (sliding_window)
             tmp_en = fML_local[p][r - p] +
@@ -1329,7 +830,7 @@ BT_mb_loop(vrna_fold_compound_t *fc,
           break;
       }
 
-      for (r = p + turn + 1; r < q - turn - 1; ++r) {
+      for (r = p + 1; r < q - 1; ++r) {
         if (evaluate(p + 1, q - 1, r, r + 1, VRNA_DECOMP_ML_ML_ML, &hc_dat_local)) {
           if (sliding_window)
             tmp_en = fML_local[p + 1][r - (p + 1)] +
@@ -1365,7 +866,7 @@ BT_mb_loop(vrna_fold_compound_t *fc,
       if (fc->type == VRNA_FC_TYPE_SINGLE)
         type = rtype[type];
 
-      for (r = p + turn + 1; r < q - turn - 1; ++r) {
+      for (r = p + 1; r < q - 1; ++r) {
         if (evaluate(*i, *j, p, r, VRNA_DECOMP_ML_COAXIAL, &hc_dat_local)) {
           if (sliding_window)
             tmp_en = c_local[p][r - p] +
@@ -1443,7 +944,7 @@ odd_dangles_exit:
     free(tt);
   }
 
-  if (r <= *j - turn - 3) {
+  if (r <= *j - 3) {
     *i  = p;
     *k  = r;
     *j  = q;
