@@ -901,6 +901,13 @@ process_record(struct record_data *record)
   vc = vrna_fold_compound_comparative((const char **)alignment,
                                       &(opt->md),
                                       VRNA_OPTION_DEFAULT);
+
+  if (!vc) {
+    vrna_message_warning("Skipping computations for \"%s\"",
+                         (record->MSA_ID) ? record->MSA_ID : "identifier unavailable");
+    return;
+  }
+
   n = vc->length;
 
   if (fold_constrained)
@@ -1265,7 +1272,7 @@ Boltzmann_sampling(vrna_fold_compound_t *fc,
                    struct options       *opt,
                    vrna_cstr_t          rec_output)
 {
-  unsigned int i, options;
+  unsigned int options;
 
   options = (opt->non_red) ?
             VRNA_PBACKTRACK_NON_REDUNDANT :
@@ -1333,7 +1340,6 @@ compute_MEA(vrna_fold_compound_t  *fc,
             vrna_cstr_t           rec_output)
 {
   char  *MEA_structure;
-  int   gq;
   float mea, *ens;
 
   /* compute MEA */
