@@ -43,6 +43,8 @@
 
 #define DBL_ROUND(a, digits) (round((a) * pow(10., (double)(digits))) / pow(10., (double)(digits)))
 
+#define MULTISTRAND_EVAL
+
 struct options {
   unsigned int    msa_format;
   int             filename_full;
@@ -358,7 +360,6 @@ main(int  argc,
   }
 
   UNINIT_PARALLELIZATION
-
   /*
    ################################################
    # post processing
@@ -392,7 +393,7 @@ process_input(FILE            *input_stream,
   /* print user help if we get input from tty */
   if (istty_in && istty_out) {
     read_opt |= VRNA_INPUT_NOSKIP_BLANK_LINES;
-    vrna_message_input_seq("Use '&' to connect 2 sequences that shall form a complex.\n"
+    vrna_message_input_seq("Use '&' to connect sequences that shall form a complex.\n"
                            "Input sequence (upper or lower case) followed by structure");
   }
 
@@ -457,7 +458,7 @@ process_input(FILE            *input_stream,
 
     /* print user help for the next round if we get input from tty */
     if (istty_in && istty_out)
-      vrna_message_input_seq("Use '&' to connect 2 sequences that shall form a complex.\n"
+      vrna_message_input_seq("Use '&' to connect sequences that shall form a complex.\n"
                              "Input sequence (upper or lower case) followed by structure");
   } while (1);
 
@@ -631,6 +632,12 @@ process_record(struct record_data *record)
                           &(opt->md),
                           VRNA_OPTION_MFE | VRNA_OPTION_EVAL_ONLY);
 
+  if (!vc) {
+    vrna_message_warning("Skipping computations for \"%s\"",
+                         (record->id) ? record->id : "identifier unavailable");
+    return;
+  }
+
   n = (int)vc->length;
 
   if (opt->shape) {
@@ -655,6 +662,33 @@ process_record(struct record_data *record)
     vrna_message_error("structure missing for record %d\n", record->number);
 
   {
+#ifdef MULTISTRAND_EVAL
+    char **structures = vrna_strsplit(tmp, "&");
+    for (unsigned int a = 0; a < vc->strands; a++) {
+      if (!structures[a])
+        vrna_message_error("Sequence and Structure have different number of strand delimiters");
+
+      unsigned int l = strlen(structures[a]);
+      switch (vc->type) {
+        case VRNA_FC_TYPE_SINGLE:
+          if (vc->nucleotides[a].length != l)
+            vrna_message_error(
+              "Structure and sequence part of strand %u differ in length (%u vs. %u)",
+              a,
+              l,
+              vc->nucleotides[a].length);
+
+          break;
+
+        case VRNA_FC_TYPE_COMPARATIVE:
+          break;
+      }
+    }
+    structure = vrna_strjoin((const char **)structures, NULL);
+    for (unsigned int a = 0; a < vc->strands; a++)
+      free(structures[a]);
+    free(structures);
+#else
     int cp = -1;
     structure = vrna_cut_point_remove(tmp, &cp);
     if (cp != vc->cutpoint) {
@@ -666,10 +700,36 @@ process_record(struct record_data *record)
     if (n != vc->length)
       vrna_message_error("structure and sequence differ in length!");
 
+#endif
+
     free(tmp);
   }
 
   if (record->tty) {
+#ifdef MULTISTRAND_EVAL
+    if (vc->strands == 1) {
+      vrna_message_info(stdout, "length = %u", vc->length);
+    } else {
+      switch (vc->type) {
+        case VRNA_FC_TYPE_SINGLE:
+          for (unsigned int a = 0; a < vc->strands; a++)
+            vrna_message_info(stdout,
+                              "length%u = %u",
+                              a + 1,
+                              vc->nucleotides[a].length);
+          break;
+
+        case VRNA_FC_TYPE_COMPARATIVE:
+          for (unsigned int a = 0; a < vc->strands; a++)
+            vrna_message_info(stdout,
+                              "length%u = %u",
+                              a + 1,
+                              vc->alignment[a].sequences[0].length);
+          break;
+      }
+    }
+
+#else
     if (vc->cutpoint == -1) {
       vrna_message_info(stdout, "length = %d", n);
     } else {
@@ -678,6 +738,8 @@ process_record(struct record_data *record)
                         vc->cutpoint - 1,
                         n - vc->cutpoint + 1);
     }
+
+#endif
   }
 
   /*
@@ -752,6 +814,13 @@ process_alignment_record(struct record_data_msa *record)
   vc = vrna_fold_compound_comparative((const char **)alignment,
                                       &(opt->md),
                                       VRNA_OPTION_DEFAULT | VRNA_OPTION_EVAL_ONLY);
+
+  if (!vc) {
+    vrna_message_warning("Skipping computations for \"%s\"",
+                         (record->MSA_ID) ? record->MSA_ID : "identifier unavailable");
+    return;
+  }
+
   n = vc->length;
 
   /* retrieve string stream bound to stdout, 6*length should be enough memory to start with */
