@@ -48,7 +48,6 @@
 #include "ViennaRNA/datastructures/stream_output.h"
 #include "ViennaRNA/combinatorics.h"
 #include "ViennaRNA/color_output.inc"
-#include "ViennaRNA/ugly_bt.h"
 
 #include "RNAfold_cmdl.h"
 #include "gengetopt_helper.h"
@@ -330,7 +329,7 @@ collect_unnamed_options(struct RNAfold_args_info  *ggostruct,
   *num_files = 0;
 
   /* collect all unnamed options */
-  if ((ggostruct->inputs_num > 0) && (!sanitize_input(ggostruct->inputs[0]))) {
+  if (ggostruct->inputs_num > 0) {
     input_files = (char **)vrna_realloc(input_files, sizeof(char *) * ggostruct->inputs_num);
     for (i = 0; i < ggostruct->inputs_num; i++)
       input_files[(*num_files)++] = strdup(ggostruct->inputs[i]);
@@ -828,6 +827,12 @@ process_record(struct record_data *record)
 
   vc = vrna_fold_compound(rec_sequence, &(opt->md), VRNA_OPTION_DEFAULT);
 
+  if (!vc) {
+    vrna_message_warning("Skipping computations for \"%s\"",
+                         (record->id) ? record->id : "identifier unavailable");
+    return;
+  }
+
   length = vc->length;
 
   if ((opt->md.circ) && (vrna_rotational_symmetry(rec_sequence) > 1))
@@ -966,11 +971,10 @@ process_record(struct record_data *record)
                                  energy);
 
       if (!opt->noDP) {
-        char  *filename_dotplot, *filename_stackplot;
+        char  *filename_dotplot;
         plist *pl1, *pl2;
 
-        filename_dotplot    = NULL;
-        filename_stackplot  = NULL;
+        filename_dotplot = NULL;
 
         /* generate initial element probability lists for dot-plot */
         pl1 = vrna_plist_from_probs(vc, opt->bppmThreshold);
@@ -1146,6 +1150,7 @@ compute_MEA(vrna_fold_compound_t  *fc,
 
   fc->exp_params->model_details.gquad = 0;
   plist *pl = vrna_plist_from_probs(fc, 1e-4 / (1 + MEAgamma));
+
   fc->exp_params->model_details.gquad = gq;
 
   structure = vrna_MEA(fc, MEAgamma, &mea);
