@@ -25,6 +25,7 @@
 #include "ViennaRNA/utils/strings.h"
 #include "ViennaRNA/params/io.h"
 #include "ViennaRNA/subopt.h"
+#include "ViennaRNA/subopt_zuker.h"
 #include "ViennaRNA/params/basic.h"
 #include "ViennaRNA/constraints/basic.h"
 #include "ViennaRNA/constraints/SHAPE.h"
@@ -37,8 +38,9 @@
 
 #include "ViennaRNA/color_output.inc"
 
-PRIVATE void putoutzuker(FILE                   *output,
-                         vrna_subopt_solution_t *zukersolution);
+PRIVATE void
+putoutzuker(FILE                    *output,
+            vrna_subopt_solution_t  *zukersolution);
 
 
 struct nr_en_data {
@@ -268,7 +270,7 @@ main(int  argc,
   /* print user help if we get input from tty */
   if (istty) {
     if (!zuker)
-      print_comment(stdout, "Use '&' to connect 2 sequences that shall form a complex.");
+      print_comment(stdout, "Use '&' to connect sequences that shall form a complex.");
 
     if (fold_constrained) {
       vrna_message_constraint_options(
@@ -347,10 +349,7 @@ main(int  argc,
     /* convert sequence to uppercase letters only */
     vrna_seq_toupper(rec_sequence);
 
-    vrna_fold_compound_t *vc =
-      vrna_fold_compound(rec_sequence, &md,
-                         VRNA_OPTION_MFE | (circ ? 0 : VRNA_OPTION_HYBRID) |
-                         ((n_back > 0) ? VRNA_OPTION_PF : 0));
+    vrna_fold_compound_t *vc = vrna_fold_compound(rec_sequence, &md, VRNA_OPTION_DEFAULT);
     length = vc->length;
 
     structure = (char *)vrna_alloc(sizeof(char) * (length + 1));
@@ -441,7 +440,8 @@ main(int  argc,
                               VRNA_PBACKTRACK_DEFAULT;
 
       if (vc->cutpoint != -1)
-        vrna_message_error("Boltzmann sampling for cofolded structures not implemented (yet)!");
+        vrna_message_error(
+          "Boltzmann sampling for multiple interacting sequences not implemented (yet)!");
 
       print_fasta_header(output, rec_id);
 
@@ -450,6 +450,9 @@ main(int  argc,
       mfe = vrna_mfe(vc, structure);
       /* rescale Boltzmann factors according to predicted MFE */
       vrna_exp_params_rescale(vc, &mfe);
+
+      vrna_mx_mfe_free(vc);
+
       /* ignore return value, we are not interested in the free energy */
       ens_en  = vrna_pf(vc, structure);
       kT      = vc->exp_params->kT / 1000.;
@@ -551,7 +554,7 @@ main(int  argc,
     /* print user help for the next round if we get input from tty */
     if (istty) {
       if (!zuker)
-        print_comment(stdout, "Use '&' to connect 2 sequences that shall form a complex.");
+        print_comment(stdout, "Use '&' to connect sequences that shall form a complex.");
 
       if (fold_constrained) {
         vrna_message_constraint_options(
