@@ -5,11 +5,12 @@ RNApath.addSwigInterfacePath()
 
 import RNA
 import unittest
+from py_include import taprunner
 
 
 class CombinatoricsTests(unittest.TestCase):
     def test_enumerate_necklaces1(self):
-        print "test_enumerate_necklaces (4 sequences, 1 strand per sequences)"
+        """Enumerate necklaces (4 sequences, 1 strand per sequences)"""
         content   = (1, 1, 1, 1)
         perm_real = ( (0, 3, 2, 1),
                       (0, 3, 1, 2),
@@ -22,7 +23,7 @@ class CombinatoricsTests(unittest.TestCase):
 
 
     def test_enumerate_necklaces2(self):
-        print "test_enumerate_necklaces (3 sequences, different number of strands per sequence)"
+        """Enumerate necklaces (3 sequences, different number of strands per sequence)"""
         content = (3, 1, 2)
         perm_real = ( (1, 0, 0, 0, 2, 2),
                       (1, 0, 0, 2, 0, 2),
@@ -39,4 +40,4 @@ class CombinatoricsTests(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    unittest.main()
+    unittest.main(testRunner=taprunner.TAPTestRunner())
